@@ -18,11 +18,11 @@
  *
  * Created 22.4.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Tuesday 21 May 02 13:02:59 tews@ithif51>
  *
  * type hierarchy for CCSL
  *
- * $Id: classtypes.ml,v 1.14 2002/01/23 16:00:14 tews Exp $
+ * $Id: classtypes.ml,v 1.16 2002/05/22 13:42:38 tews Exp $
  *
  *)
 
@@ -43,13 +43,17 @@ open Top_classtypes
  * 				       |
  * 				       |
  * 			  ccsl_pre_theory_body_type
- * 				/               \
- * 			       /                 \
- * ccsl_pre_virtual_adt_theory_type         ccsl_pre_virtual_class_theory_type
- * 		 | 		 	       	  |
- * 		 |                                |
- * ccsl_pre_adt_theory_type                 ccsl_pre_class_theory_type
- * 
+ * 				/      |        \
+ * 			       /       |         \
+ * ccsl_pre_virtual_sig_theory_type    |    ccsl_pre_virtual_class_theory_type
+ * 		 | 		       |       	  |
+ * 		 |                     |          |
+ * ccsl_pre_sig_theory_type            |    ccsl_pre_class_theory_type
+ *				       |
+ *			ccsl_pre_virtual_adt_theory_type
+ *				       |
+ *				       |
+ *			ccsl_pre_adt_theory_type
  * 
  * At the bottom ccsl type abbreviations are defined.
  * 
@@ -148,19 +152,23 @@ object
     method get_parameters : 
       ('super_class, 'member_type) top_pre_parameter_type list
 
-	(* check if argument list is a valid instanciation for this iface *)
+(*	(* check if argument list is a valid instanciation for this iface *)
     method check_parameters : 
       ('super_class, 'member_type) top_pre_argument_type list -> bool
-
+*)
 	(* imports *)
     method add_iface_import :  
       ('super_class, 'member_type) top_pre_importing_type -> unit
-    method add_assertion_import : 
-      ('super_class, 'member_type) top_pre_importing_type -> unit
     method get_iface_imports : 
       ('super_class, 'member_type) top_pre_importing_type list
+    method add_assertion_import : 
+      ('super_class, 'member_type) top_pre_importing_type -> unit
     method get_assertion_imports : 
       ('super_class, 'member_type) top_pre_importing_type list
+    method add_theorem_import : 
+      ('super_class, 'member_type) top_pre_importing_type -> unit
+    method get_theorem_imports : 
+      ('super_class, 'member_type) top_pre_importing_type list
 
       (*
        * all components with all instanciations must be added !!
@@ -183,6 +191,8 @@ object
       ('super_class, 'member_type) top_pre_assertion_record -> unit
     method add_creation : 
       ('super_class, 'member_type) top_pre_assertion_record  -> unit
+    method add_theorem : 
+      ('super_class, 'member_type) top_pre_assertion_record  -> unit
     method set_assertions:
       ('super_class, 'member_type) top_pre_assertion_record list -> unit
     method get_assertions :
@@ -191,6 +201,8 @@ object
       ('super_class, 'member_type) top_pre_assertion_record list
     method set_creations :
        ('super_class, 'member_type) top_pre_assertion_record list -> unit     
+    method get_theorems :
+      ('super_class, 'member_type) top_pre_assertion_record list
 
 	(*****************************************
 	 *
@@ -300,6 +312,19 @@ object
     method get_all_ground_types : 
       ('super_class, 'member_type) top_pre_identifier_record_type list
 
+    method has_own_output : bool
+
+	(* If true, importings with this class and importings for 
+	 * this class are fully instanciated.
+	 * If false generic importings are used.
+	 * The flag is reset to false in ground signatures that have 
+	 * some item with local type parameters. In this case the 
+	 * instaciation for an importing might contain a local type 
+	 * variable, or it might have too much arguments. In both 
+	 * cases the instanciation must be supressed.
+	 *)
+    method instanciate_importings : bool
+
   end (* ccsl_pre_iface_type *)
 
 
@@ -367,6 +392,17 @@ class type ['class_type, 'member_type] ccsl_pre_member_type
     method register_recognizer : 'member_type -> unit
     method get_recognizer : 'member_type
 
+    (*******************************************************************
+     *
+     * Ground signature stuff
+     *)
+
+    method set_local_parameters : 
+      ('class_type, 'member_type) top_pre_parameter_type list -> unit
+
+    method get_local_parameters : 
+      ('class_type, 'member_type) top_pre_parameter_type list
+
   end (* ccsl_pre_member_type *)
 
 
@@ -452,7 +488,17 @@ class type virtual ['class_type, 'member_type] ccsl_pre_theory_body_type =
 
     val ns : ('class_type, 'member_type) Name_space.t
 
-  
+    val coalgebra_term : ('class_type, 'member_type) top_pre_expressions
+
+    val algebra_term : ('class_type, 'member_type) top_pre_expressions
+
+    method get_member_fun : 
+      'member_type -> ('class_type, 'member_type) top_pre_expressions option
+
+    method get_iface_fun : 
+      ('class_type, 'member_type) top_pre_types -> 
+	('class_type, 'member_type) top_pre_expressions
+
   end (* ccsl_pre_theory_body_type *)
 
 
@@ -485,13 +531,6 @@ class type virtual
 
     method coalgebra2_type : ('class_type, 'member_type) top_pre_types
 
-    method get_member_fun : 
-      'member_type -> ('class_type, 'member_type) top_pre_expressions
-
-    method get_iface_fun : 
-      ('class_type, 'member_type) top_pre_types -> 
-	('class_type, 'member_type) top_pre_expressions
-
     method assert_coalgebra_type : ('class_type, 'member_type) top_pre_types
 
     method assert_coalgebra1_type : ('class_type, 'member_type) top_pre_types
@@ -517,7 +556,7 @@ class type virtual
     method assert_coalgebra2_decl :
       ('class_type, 'member_type) top_pre_theory_declaration
 
-    val coalgebra_term : ('class_type, 'member_type) top_pre_expressions
+      (* coalgebra term is in ccsl_pre_theory_body_type *)
 
     val coalgebra1_term : ('class_type, 'member_type) top_pre_expressions
 
@@ -542,7 +581,26 @@ class type virtual
     method algebra_decl :
       ('class_type, 'member_type) top_pre_theory_declaration
 
-    val algebra_term : ('class_type, 'member_type) top_pre_expressions
+      (* helper method for the struct versions of invariant,
+       * bisimulation, and assert
+       *
+       * !! this code relies on the traversion order of 
+       * !! get_all_members 
+       *)
+    method coalgebra_as_tuple : 
+      ('class_type, 'member_type) Name_space.t -> 
+	(('class_type, 'member_type) definition_parameter list *
+	 ('class_type, 'member_type) top_pre_expressions)
+
+      (* same as coalgebra_as_tuple but returns two tuples corresponding 
+       * to two coalgebras
+       *)
+    method two_coalgebras_as_tuple : 
+      ('class_type, 'member_type) Name_space.t -> 
+	(('class_type, 'member_type) definition_parameter list *
+	 ('class_type, 'member_type) definition_parameter list *
+	 ('class_type, 'member_type) top_pre_expressions *
+	 ('class_type, 'member_type) top_pre_expressions)
 
   end (* ccsl_pre_virtual_class_theory_type *)
 
@@ -634,6 +692,57 @@ class type ['class_type, 'member_type] ccsl_pre_adt_theory_type =
   end (* ccsl_pre_adt_theory_type *)
 
 
+(***********************************************************************
+ ***********************************************************************
+ * 
+ *        Theories for ground signatures (virtual)
+ * 
+ ***********************************************************************
+ ***********************************************************************
+ *)
+
+
+class type virtual ['class_type, 'member_type] 
+  ccsl_pre_virtual_sig_theory_type =
+  object
+    inherit ['class_type, 'member_type] ccsl_pre_theory_body_type
+end
+
+
+
+(***********************************************************************
+ ***********************************************************************
+ * 
+ *        Theories for ground signatures
+ * 
+ ***********************************************************************
+ ***********************************************************************
+ *)
+
+
+class type ['class_type, 'member_type] ccsl_pre_sig_theory_type =
+  object
+    inherit ['class_type, 'member_type] ccsl_pre_virtual_sig_theory_type
+
+      (* make the inherited virtual method real *)
+    method get_name :
+      string
+
+    method get_parameters :
+      ('class_type, 'member_type) top_pre_parameter_type list
+
+    method get_file_name :
+      string
+
+    method make_body :
+      ('class_type, 'member_type) top_pre_theory_declaration list
+
+    method get_proofs :
+      ('class_type, 'member_type) top_pre_proof_file_entry list
+
+  end (* ccsl_pre_sig_theory_type *)
+
+
 
 (***********************************************************************
  ***********************************************************************
@@ -653,6 +762,8 @@ and  ccsl_member_type =
     [ccsl_iface_type, ccsl_member_type] ccsl_pre_member_type
 
 
+type ccsl_member_sort = ccsl_member_type top_pre_member_sort
+
 type ccsl_top_types = (ccsl_iface_type, ccsl_member_type) top_pre_types
 
 
@@ -669,6 +780,9 @@ class type ccsl_adt_theory_type =
 class type ccsl_class_theory_type = 
 	  [ccsl_iface_type, ccsl_member_type] ccsl_pre_class_theory_type
 
+class type ccsl_sig_theory_type = 
+	  [ccsl_iface_type, ccsl_member_type] ccsl_pre_sig_theory_type
+
 
 (* From top_variant_types *)
 
@@ -982,7 +1096,7 @@ type ccsl_ast =
  *
  **********************************************************************)
        
-class type ccsl_method_inv_helper_type 
+class type ['class_type, 'member_type] ccsl_pre_method_inv_helper_type 
   = object
 
 (* 
@@ -998,25 +1112,37 @@ class type ccsl_method_inv_helper_type
 
     val mutable name_of_path2 : string
 
-    method private list_adt : ccsl_iface_type
+    method private list_adt : 'class_type
 
-    method private list_type :  ccsl_top_types
+    method private list_type :  ('class_type, 'member_type) top_pre_types
 
-    method private list_decl :  ccsl_theory_declaration
+    method private list_decl : 
+      ('class_type, 'member_type) top_pre_theory_declaration
 
-    method private list_term :  ccsl_expressions
+    method private list_term :  
+      ('class_type, 'member_type) top_pre_expressions
 
-    method private method_enum_type : ccsl_top_types
+    method private method_enum_type : 
+      ('class_type, 'member_type) top_pre_types
 
-    method private method_decl : ccsl_theory_declaration
+    method private method_decl : 
+      ('class_type, 'member_type) top_pre_theory_declaration
 
-    method private method_term : ccsl_expressions
+    method private method_term : 
+      ('class_type, 'member_type) top_pre_expressions
+
+      (* return those methods that need predicate lifting *)
+    method private get_lifting_methods : 'member_type list
   end       
 
 
+class type ccsl_method_inv_helper_type = 
+	  [ccsl_iface_type, ccsl_member_type] ccsl_pre_method_inv_helper_type
+
 
 (*** Local Variables: ***)
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
