@@ -18,11 +18,11 @@
  *
  * Created 14.5.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Tuesday 21 May 02 18:41:48 tews@ithif51>
  *
  * the class for ccsl adt's and classes
  *
- * $Id: iface_class.ml,v 1.13 2002/01/23 16:00:15 tews Exp $
+ * $Id: iface_class.ml,v 1.15 2002/05/22 13:42:40 tews Exp $
  *
  *)
 
@@ -32,15 +32,17 @@ open Top_variant_types
 open Top_variant_types_util
 open Top_classes
 open Classtypes
-open Types_util
 open Interface_theory
 open Methodinv_theory
 open Bisim_theory
 open Morphism_theory
 open Full_liftings_theory
 open Semantics_theory
-open Adt_theory;;
-
+open Class_theorems_theory
+open Adt_theory
+open Sig_theory
+open Types_util
+;;
 
 class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
   name					(* the token *)
@@ -165,6 +167,7 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
   
     val mutable iface_importings = []
     val mutable assertion_importings = []
+    val mutable theorem_importings = []
 
     method add_iface_import i = 
       assert(not instanciated);
@@ -182,6 +185,14 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
       assert(not instanciated);
       assertion_importings
 
+    method add_theorem_import i = 
+      assert(not instanciated);
+      theorem_importings <- theorem_importings @ [i]
+
+    method get_theorem_imports = 
+      assert(not instanciated);
+      theorem_importings
+
     (*******************************************************************
      *
      * parameters and arguments
@@ -198,10 +209,11 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
       assert(not instanciated);
       parameters
 
-    method check_parameters args = 
-      assert(not instanciated);
-      check_parameters parameters args
-
+(*     method check_parameters args = 
+ * 	 assert(not instanciated);
+ * 	 check_parameters parameters args
+ *)
+    
     (*******************************************************************
      *
      * components
@@ -216,18 +228,27 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
        * argument list and add components in there first
        * see iter_component_arg and iter_components
        *)
-    method add_component comp =
-      let (v, c, args) = comp in 
-      let rec do_it = function
+    method add_component (v,c,inpargs) =
+      let rec do_it args = function
 	| ((v',c',args') as comp') :: l ->
-	    if ((c'#get_name = c#get_name) &&
-		(List.for_all2 eq_ccsl_args args args'))
+	    if (c'#get_name = c#get_name) &&
+	      (List.for_all2 eq_ccsl_args args args')
 	    then
 	      ((Logic_util.variance_join v v'), c', args') :: l
-	    else comp' :: (do_it l)
-	| [] -> [comp]
+	    else comp' :: (do_it args l)
+	| [] -> [(v,c,args)]
+      in let this_parameters = self#get_parameters in
+			(* returns true, if id is a local typeparameter *)
+      let other_params_p id = 
+	  List.for_all (function TypeParameter oid ->
+			  oid.id_token.token_name <> id.id_token.token_name)
+	    this_parameters
       in
-      	components <- do_it components
+	if c#instanciate_importings && self#instanciate_importings
+	then
+      	  components <- do_it inpargs components
+	else
+      	  components <- do_it [] components
 
     method get_components = components
 
@@ -247,7 +268,7 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
  * 		  (fun c -> self#add_component c)
  * 		  args
  *
- *	 | _ -> (assert (false); raise Internal_error)
+ *	 | _ -> assert (false)
  *      );
  *)
       top_iface#add_ancestor a
@@ -268,18 +289,6 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 
     method add_member m =
       Symbol.create_local local m#get_name (MemberSymb m);
-(* component_pass
- * 	 (match m#get_sort with
- * 	    | Adt_Reduce | Class_Coreduce -> ()
- * 	    | _ -> 
- * 		begin
- * 		  iter_components false (fun c -> self#add_component c)
- * 		    m#get_domain;
- * 		  iter_components true (fun c -> self#add_component c)
- * 		    m#get_codomain
- * 		end
- * 	 );
- *)
       if not has_constructors && m#is_constructor 
       then has_constructors <- true;
       top_iface#add_member m
@@ -321,7 +330,7 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 	      (match Table.find_local loc CCSL_ID name with
 		| MemberSymb m -> m
 			(* a method is always stored as Member *)
-		| _ -> raise Member_not_found
+		| _ -> assert(false)
 	      )
 	    with Table.Not_defined -> doit rest
       in
@@ -338,6 +347,58 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 	  | _ -> raise Member_not_found
       with Table.Not_defined -> raise Member_not_found
 
+    (*******************************************************************
+     *
+     * Signature stuff
+     *)
+
+    val mutable ground_types = 
+        ( [] : 
+	    ('super_class, 'member_type) top_pre_identifier_record_type list)
+
+    val mutable sequence_counter = 0
+
+    val mutable has_own_output = false
+				   
+    method has_own_output = 
+      assert(self#get_kind = Spec_sig);
+      has_own_output
+
+	(* If true, importings with this class and importings for 
+	 * this class are fully instanciated.
+	 * If false generic importings are used.
+	 * The flag is reset to false in ground signatures that have 
+	 * some item with local type parameters. In this case the 
+	 * instaciation for an importing might contain a local type 
+	 * variable, or it might have too much arguments. In both 
+	 * cases the instanciation must be supressed.
+	 *)
+    val mutable instanciated_importings = true
+
+    method instanciate_importings = instanciated_importings
+
+    method add_ground_type gt  = 
+      assert(self#get_kind = Spec_sig);
+      Symbol.create_local local gt.id_token.token_name
+	(CCSL_GroundTypeSymb gt);
+      (match gt.id_origin with
+	 | CCSL_GroundTypeDef _ ->
+	     begin
+	       gt.id_sequence <- sequence_counter;
+	       sequence_counter <- sequence_counter +1;
+	       has_own_output <- true
+	     end
+	 | CCSL_GroundTypeDecl _ -> ()
+	 | _ -> assert(false)
+      );
+      if gt.id_parameters <> [] then
+	instanciated_importings <- false;
+      ground_types <- ground_types @ [gt]
+
+    method get_all_ground_types = 
+      assert(self#get_kind = Spec_sig);
+      ground_types
+
     (*******************************************************************
      *
      * definitional extensions
@@ -347,8 +408,15 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 
     method add_definition def =
       self#add_member def.defined_method;
-      definitions <- definitions @ [def]
-
+      definitions <- definitions @ [def];
+      if self#get_kind = Spec_sig then begin
+	has_own_output <- true;
+	def.def_sequence <- sequence_counter;
+	sequence_counter <- sequence_counter +1;
+      end;
+      if def.defined_method#get_local_parameters <> [] then
+	instanciated_importings <- false
+      
     method get_definitions = definitions
 
     (*******************************************************************
@@ -360,6 +428,8 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
       ( [] : ('super_class, 'member_type) top_pre_assertion_record list)
     val mutable creations = 
       ( [] : ('super_class, 'member_type) top_pre_assertion_record list)
+    val mutable theorems = 
+      ( [] : ('super_class, 'member_type) top_pre_assertion_record list)
 
     method add_assertion a = 
       assert(not instanciated);
@@ -379,6 +449,12 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
     method set_creations cre =
       creations <- cre
 
+    method add_theorem c = 
+      assert(not instanciated);
+      theorems <- theorems @ [c]
+
+    method get_theorems = theorems
+
     (*******************************************************************
      *
      * Liftings, doku see iface_type 
@@ -455,10 +531,18 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 	          ccsl_pre_iface_type,
 		  ('super_class, 'member_type) ccsl_pre_member_type)
 	       ccsl_pre_theory_body_type) in
+      let sig_typeconvert 
+	(th : ccsl_sig_theory_type ) =
+	(th :> (('member_type, 'theory_body_type, 'super_class)
+	          ccsl_pre_iface_type,
+		  ('super_class, 'member_type) ccsl_pre_member_type)
+	       ccsl_pre_theory_body_type) in
+
       match self#get_kind with 
 					(* generate classes *)
   	| Spec_class ->			
 	    [(class_typeconvert (new ccsl_interface_theory self' ));
+	     (class_typeconvert (new ccsl_method_idtype_adt self'));
 	     (class_typeconvert (new ccsl_method_lift_theory self'));
 (* 		(class_typeconvert
  * 		   (new ccsl_lift_theory self'));
@@ -469,13 +553,17 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
  *)
 	     (class_typeconvert
 		(new ccsl_method_invariance_rewrite_theory self'));
+	     (class_typeconvert
+		(new ccsl_method_invariance_inherit_theory self'));
 (* Jan: what is this GI stuff good for ??
  *	     (class_typeconvert
  *		(new ccsl_greatest_invariance_theory self'));
  * !!!  I also changed the importing in the box theory !!!
  *)
 	     (class_typeconvert
-		(new ccsl_box_theory self'))
+		(new ccsl_box_theory self'));
+	     (class_typeconvert
+		(new ccsl_box_inherit_theory self'))
 	    ] @ ( 
 
 	      if (!ccsl_generate_paths) then
@@ -525,14 +613,28 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 		[ (class_typeconvert
 		     (new ccsl_req_bisim_theory self'))
 		]
-	    ) @ [
-	      (class_typeconvert 
-	         (new ccsl_semantics_theory self'));
-	      (class_typeconvert 
-	         (new ccsl_basic_theory self'));
-              (class_typeconvert
-                 (new ccsl_full_invariant_theory self'));
-	    ] @ (
+	    ) @ (
+
+	      if self#has_feature HasMorphismFeature
+	      then 
+		[
+		  (class_typeconvert 
+		     (new ccsl_morphism_theory self'));
+		  (class_typeconvert 
+		     (new ccsl_morphism_rewrite_theory self'));
+		]
+	      else []
+
+	    ) @ (
+	      [
+		(class_typeconvert 
+	           (new ccsl_semantics_theory self'));
+		(class_typeconvert 
+	           (new ccsl_basic_theory self'));
+		(class_typeconvert
+                   (new ccsl_full_invariant_theory self'));
+	      ] 
+	    ) @ (
 
 	      if self#has_feature HasFullRelLiftingFeature then
 		[
@@ -546,10 +648,6 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 	      if self#has_feature HasMorphismFeature
 	      then 
 		[
-		  (class_typeconvert 
-		     (new ccsl_morphism_theory self'));
-		  (class_typeconvert 
-		     (new ccsl_morphism_rewrite_theory self'));
 		  (class_typeconvert 
 		     (new ccsl_finality_theory self'));
 		]
@@ -557,27 +655,39 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 
 	    ) @ (
 
-	    if self#has_feature FinalSemanticsFeature then
+	      if self#has_feature FinalSemanticsFeature then
+		[
+		  (class_typeconvert (new ccsl_final_theory self'));
+		  (class_typeconvert (new ccsl_final_props_theory self'));
+		] @ (
+		  if self#has_feature HasMapFeature then
+		    [
+		      (class_typeconvert (new ccsl_map_struct_theory self'));
+		      (class_typeconvert (new ccsl_map_theory self'));
+		    ]
+		  else
+		    []
+		)
+	      else
+		[
+		  (class_typeconvert (new ccsl_loose_theory self'));
+		]
+
+	    ) @ (
+
 	      [
-		(class_typeconvert (new ccsl_final_theory self'));
-		(class_typeconvert (new ccsl_final_props_theory self'));
-	      ] @ (
-		if self#has_feature HasMapFeature then
-		  [
-		    (class_typeconvert (new ccsl_map_struct_theory self'));
-		    (class_typeconvert (new ccsl_map_theory self'));
-		  ]
-		else
-		  []
-	      )
-	    else
-	     [
-	       (class_typeconvert (new ccsl_loose_theory self'));
-	     ]
-	    ) @
-	    [
-	     (class_typeconvert (new ccsl_coadt_theory self'));
-	    ]
+		(class_typeconvert (new ccsl_coadt_theory self'));
+	      ]
+
+	    ) @ (
+
+	      if self#get_theorems <> [] then
+		[
+		  class_typeconvert (new ccsl_class_theorem_theory self')
+		]
+	      else
+		[]
+	    )
 					(* generate adt's *)
   	| Spec_adt -> 
 	    (match !output_mode with
@@ -614,29 +724,17 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 		[]
 	    )
 
-	| Spec_sig
+	| Spec_sig ->
+	    (if self#has_own_output
+	     then
+	       List.map sig_typeconvert (ccsl_sig_def_theories self')
+	     else
+	       []
+	    )
+	    
   	| Spec_Spec 
-	  -> (assert(false); raise Internal_error)
+	  -> assert(false)
 	  
-    (*******************************************************************
-     *
-     * Signature stuff
-     *)
-
-    val mutable ground_types = 
-        ( [] : 
-	    ('super_class, 'member_type) top_pre_identifier_record_type list)
-
-    method add_ground_type gt  = 
-      assert(self#get_kind = Spec_sig);
-      Symbol.create_local local gt.id_token.token_name
-	(CCSL_GroundTypeSymb gt);
-      ground_types <- ground_types @ [gt]
-
-    method get_all_ground_types = 
-      assert(self#get_kind = Spec_sig);
-      ground_types
-
     (*******************************************************************
      *
      * Dumping
@@ -663,5 +761,6 @@ class ['member_type, 'theory_body_type, 'super_class] ccsl_pre_iface_class
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
