@@ -18,11 +18,11 @@
  *
  * Created 17.6.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Tuesday 21 May 02 15:42:14 tews@ithif51>
  *
  * the ccsl interface theory
  *
- * $Id: interface_theory.ml,v 1.8 2002/01/24 14:44:00 tews Exp $
+ * $Id: interface_theory.ml,v 1.10 2002/05/22 13:42:40 tews Exp $
  *
  *)
 
@@ -69,8 +69,11 @@ class ['class_type, 'member_type] ccsl_pre_interface_theory
 	  name_of_method_functor cl
        	]
 	@ 
-	if cl#has_constructors then [name_of_constructor_functor cl]
-	else []
+	(if cl#has_constructors 
+	 then [name_of_constructor_functor cl]
+	 else []
+	) 
+	 (* definitions are reserved because all members are reserved *)
       )
 
     (*******************************************************************
@@ -120,18 +123,29 @@ class ['class_type, 'member_type] ccsl_pre_interface_theory
 		     | Pvs_mode -> (ccsl_adt_theory_name c, args) :: accu
 		  )
 	      | Spec_sig -> 
-		  let subst = make_substitution c#get_parameters args 
-		  in
-		    (List.map 
-		       (fun (name,args) ->
-			  (name, 
-			   substitute_arguments_types_only
-			     eq_types subst args)
-		       )
-		       c#get_iface_imports
+		  if c#has_own_output
+		  then			(* does have an own theory *)
+		    (ccsl_sig_def_theory_name c, args) :: accu
+		  else			(* no own theory for c *)
+		    (if c#instanciate_importings && 
+		       cl#instanciate_importings
+		     then
+		       let subst = make_substitution c#get_parameters args 
+		       in
+			 (List.map 
+			    (fun (name,args) ->
+			       (name, 
+				substitute_arguments_types_only
+				  eq_types subst args)
+			    )
+			    c#get_iface_imports
+			 )
+		     else
+		       List.map (fun (name,_) -> (name, [])) 
+			 c#get_iface_imports
 		    )
 		    @ accu
-	      | Spec_Spec -> (assert(false); raise Internal_error)
+	      | Spec_Spec -> assert(false)
 	   ))
 	cl#get_components []
 			  
@@ -159,6 +173,9 @@ class ['class_type, 'member_type] ccsl_pre_interface_theory
 	[Comment("Class " ^ cl#get_name ^ " has no constructors")]
 
 
+	(* This indirectly depends on the traversion order 
+	 * of get_all_members 
+	 *)
     method private struct_of =
       let sub_ns = sub_space ns in 
       let method_name m = (("o_" ^ m#get_name), m#get_full_type) in
@@ -215,12 +232,15 @@ class ['class_type, 'member_type] ccsl_pre_interface_theory
     method private all_inherited_methods = 
       let ancestor_flatten = 
 	List.map (function
-		    | Resolved_renaming (_,args,_,ianc) -> ianc
+		    | Resolved_renaming (_,args,_,ianc,_) -> ianc
 				(* no other stuff *)
 		    | Unresolved_renaming _
-		    | Resolved_ancestor _ 
-		    | Unresolved_ancestor _ -> 
-			(assert(false); raise Internal_error)
+(* NO JAVA ANCESTORS
+ * 		       | Resolved_ancestor _ 
+ * 		       | Unresolved_ancestor _ 
+ *)
+		      -> 
+			assert(false)
 		 ) in
          (* cl_list comes reversed ! *)
       let ancestor_abbrev_comment cl_list = 
@@ -246,28 +266,37 @@ class ['class_type, 'member_type] ccsl_pre_interface_theory
 
 
     method private do_definitions =
-      List.map
-	(fun def ->
-	   let mem = def.defined_method 
-	   in
-	     match def.definition with
-	       | Symbolic eq ->
-		   let pretty_eq =
-		     ccsl_pre_pretty_formula cl
-		       self#get_member_fun self#get_iface_fun eq
-		   in 
-		     Defn(mem#get_name,
-			  [[Declared(name_of_coalgebra, self#coalgebra_type)]],
-			  Function(mem#get_domain,mem#get_codomain),
-			  Abstraction(
-			    List.map (fun id -> (id.id_token.token_name,
-						 id.id_type))
-					def.variables,
-					(Expression pretty_eq)
-			  ))
-	       | _ -> assert(false); raise Internal_error		    
-	)
-	cl#get_definitions
+      let rec do_abstraction innerex = function
+	| [] -> innerex
+	| vars :: rest ->
+	    Abstraction(
+	      List.map 
+		(fun id -> (id.id_token.token_name,
+			    id.id_type))
+		vars,
+	      do_abstraction innerex rest)
+      in
+	List.map
+	  (fun def ->
+	     let mem = def.defined_method 
+	     in
+	       match def.definition with
+		 | Symbolic eq ->
+		     let pretty_eq =
+		       ccsl_pre_pretty_formula cl
+			 self#get_member_fun self#get_iface_fun eq
+		     in 
+		       Defn(mem#get_name,
+			    [[Declared(name_of_coalgebra, 
+				       self#coalgebra_type)]],
+			    mem#get_full_type,
+			    do_abstraction
+			      (Expression pretty_eq)
+			      def.variables
+			   )
+		 | _ -> assert(false)		    
+	  )
+	  cl#get_definitions
 
     method make_body =
       Import( 
@@ -304,260 +333,6 @@ class ccsl_interface_theory cl =
   eq_ccsl_types 
 
 
-(* class ['class_type, 'member_type] old_ccsl_pre_interface_theory 
- *   (cl : 'class_type) 
- *   (* the equality relation for initializing the name space *)
- *   (eq_types : ('class_type, 'member_type) top_pre_types
- * 	-> ('class_type, 'member_type) top_pre_types -> bool)
- * 
- *   : ['class_type, 'member_type] ccsl_pre_class_theory_type =
- *   object (self : 'self)
- * 
- *     inherit 
- * 	 ['class_type, 'member_type] ccsl_pre_class_theory_class cl eq_types
- * 	 as top_theory
- * 
- * 	  (* reserve all names, that we want to declare *)
- *     initializer reserve ns 
- * 	 (
- * 	   [ name_of_coalgebra; 
- * 	     name_of_method_functor cl
- * 	   ]
- * 	   @ 
- * 	   if cl#has_constructors then [name_of_constructor_functor cl]
- * 	   else []
- * 	 )
- * 
- *     (*******************************************************************
- * 	*
- * 	* override section
- * 	* 
- * 	* the functors get defined in this theory, therefore we have to 
- * 	* change algebra_type and coalgebra_type
- * 	* this in turn affects algebra_decl and coalgebra_decl
- * 	*)      
- * 
- *     method coalgebra_type =
- * 	 Function(Self, IFace( cl, Isabelle_only,self#simple_arguments))
- * 
- *     method algebra_type =
- * 	 TypeConstant(name_of_constructor_functor cl, Isabelle_only,
- * 		      self#simple_arguments)
- * 
- *     (*******************************************************************
- * 	*
- * 	* standard section
- * 	* 
- * 	*)
- * 
- *     method get_name = ccsl_interface_theory_name cl
- * 
- *     method get_parameters = self#simple_parameters 
- * 
- *     initializer top_theory#override_file_name (ccsl_class_file_name cl)
- * 
- *     method get_proofs = []
- * 
- * 
- *     method private user_imports = 
- * 	 List.map (fun s -> s,[]) cl#get_iface_imports 
- * 
- *     method private component_imports = 
- * 	 List.map (fun (c,args) -> 
- * 		     (match c#get_kind with
- * 			| Spec_class -> (ccsl_loose_theory_name c, args)
- * 			| Spec_adt ->  (ccsl_adtutil_theory_name c, args)
- * 			| Spec_sig 
- * 			| Spec_Spec -> (assert(false); raise Internal_error)
- * 		     ))
- * 	   cl#get_components
- * 			     
- * 
- *     method private super_imports = 
- * 	 List.map 
- * 	   (function
- * 	      | Resolved_renaming (ifa,args,_,_) -> 
- * 		  ifa#get_name, self_argument::args
- * 					   (* no other stuff *)
- * 	      | Unresolved_renaming _
- * 	      | Resolved_ancestor _ 
- * 	      | Unresolved_ancestor _ -> (assert(false); raise Internal_error)
- * 	   )
- * 	   cl#get_ancestors
- * 
- *     method private functor_declaration = 
- * 	 [Comment("Declare coalgebra functor for class " ^ cl#get_name);
- * 	  Typedecl(self#simple_parameters,
- * 		   name_of_method_functor cl, self#get_method_functor_type)
- * 	 ]
- * 
- *     method private constructor_declaration = 
- * 	 if cl#has_constructors then
- * 	   [Comment("Declare algebra functor for class " ^ cl#get_name);
- * 	    Typedecl(self#simple_parameters,
- * 		     name_of_constructor_functor cl, 
- * 		     self#get_constructor_functor_type)
- * 	   ]
- * 	 else 
- * 	   [Comment("Class " ^ cl#get_name ^ " has no constructors")]
- * 
- * 	     (* declarations for direct ancestors *)
- *     method private super_access = 
- * 	 (* get a variable for Self *)
- * 	 let self_var = create_one_id (sub_space ns) Self in
- * 	     (* untility function for one ancestor *)
- * 	 let do_ancestor anc_cl anc_args = 
- * 	   Defn(super_access_method anc_cl,
- * 		[[Declared(name_of_coalgebra, self#coalgebra_type)]],
- * 		Function(Self,
- * 			 IFace(anc_cl,Always,self_argument :: anc_args)),
- * 		   Abstraction(
- * 		     [self_var, Self],
- * 		     Application(
- * 		       Term(super_label anc_cl,Always,[]),
- * 		       Application(coalgebra_term, Term(self_var,Always,[])))))
- * 	 in
- * 	   if cl#get_ancestors <> [] 
- * 	   then 
- * 	     (Comment("Define projections for super classes")) ::
- * 	     List.map 
- * 	       (function
- * 		  | Resolved_renaming (_,args,_,ianc) -> 
- * 		      do_ancestor ianc args
- * 			(* no other  stuff *)
- * 		  | Unresolved_renaming _
- * 		  | Resolved_ancestor _ 
- * 		  | Unresolved_ancestor _ -> 
- * 		      (assert(false); raise Internal_error)
- * 	       )
- * 	       cl#get_ancestors
- * 	   else 
- * 	     [Comment("Class " ^ cl#get_name ^ " has no superclasses")]
- * 
- * 
- * 	   (* collect method abbreviations for class cl *)
- *     method private method_abbreviations = 
- * 	   (* utility function to do one method of cl *)
- * 	 let do_method m =
- * 	   let sub_ns = sub_space ns in
- * 	   let argtypes = member_arg_list m in
- * 	   let arglist = create_ids sub_ns argtypes in
- * 	     (* we assume here that self is at first position *)
- * 	   let sv,_ = List.hd arglist in
- * 	   let inner_expr =
- * 	     Application( Term(method_label m,Always,[]), 
- * 			 Application(coalgebra_term,Term(sv,Always,[])))
- * 	   in 
- * 	     Defn( m#get_name, 
- * 		   [[Declared(name_of_coalgebra, self#coalgebra_type)]],
- * 		   Function(m#get_domain,m#get_codomain),
- * 		   Abstraction(
- * 		     arglist,
- * 		     match List.tl arglist with
- * 		       | [] -> inner_expr
- * 			    (* fiddle around one tuples, see get_curried_type *)
- * 		       | [(a,t)] -> 
- * 			   Application(
- * 			     inner_expr,
- * 			     Term(a,Always,[]))
- * 		       | many_args ->
- * 			   Application(
- * 			     inner_expr,
- * 			     Tuple(
- * 			       List.map
- * 				     (fun (v,t) -> Term(v,Always,[]))
- * 				     many_args))))
- * 	 in
- * 	   List.map (fun m -> do_method m) cl#get_actions
- * 
- * 
- * 	   (* collect inherited methods for class acl,
- * 	    * they are inherited via path starting in super
- * 	    *)
- *     method private inherited_methods super acl = 
- * 	   (* utility function to do one method of acl *)
- * 	 let do_method m =
- * 	   Defn( m#get_name, 
- * 		 [[Declared(name_of_coalgebra, self#coalgebra_type)]],
- * 		 Function(m#get_domain,m#get_codomain),
- * (* Hendrik: find solution for this QualifiedTerm *)	      
- * 			     (* Qualified term for pvs not neccessary *)
- * 		 Application(QualifiedTerm(
- * 			       ccsl_interface_theory_name super,
- * 			       Always,[],
- * 			       m#get_name), 
- * 			     Application(Term(super_access_method(super),
- * 					      Always,[]), 
- * 					 coalgebra_term)))
- * 	 in
- * 	   List.map (fun m -> do_method m) acl#get_actions
- * 
- * 
- *     method private all_inherited_methods = 
- * 	 let ancestor_flatten = 
- * 	   List.map (function
- * 		       | Resolved_renaming (_,args,_,ianc) -> ianc
- * 				   (* no other stuff *)
- * 		       | Unresolved_renaming _
- * 		       | Resolved_ancestor _ 
- * 		       | Unresolved_ancestor _ -> 
- * 			   (assert(false); raise Internal_error)
- * 		    ) in
- * 	    (* cl_list comes reversed ! *)
- * 	 let ancestor_abbrev_comment cl_list = 
- * 	   Comment( List.fold_right
- * 		      (fun cl s -> s ^ cl#get_name ^ " ")
- * 		      cl_list
- * 		      "Inherited via " 
- * 		  )
- * 	 in
- * 	     (* do indirect ancestors from acl
- * 	      * they are coming to this class (cl) via cl_path
- * 	      * the last element of cl_path is super
- * 	      *)
- * 	 let rec recurse super cl_path acl = 
- * 	   (ancestor_abbrev_comment (acl::cl_path)) 
- * 	   :: self#inherited_methods super acl
- * 	   @  List.flatten(List.map (recurse super (acl::cl_path))
- * 			     (ancestor_flatten acl#get_ancestors))
- * 	    in 
- * 	   List.flatten(List.map (fun acl -> recurse acl [] acl)
- * 			  (ancestor_flatten cl#get_ancestors))
- * 	       
- * 
- * 
- *     method make_body =
- * 	 Import( 
- * 	   (match !output_mode with
- * 	      | Pvs_mode -> []
- * 	      | Isa_mode -> [isabelle_top_theory, []])
- * 	   @
- * 	   self#user_imports @ self#component_imports @ self#super_imports)
- * 	   ::
- * 	 self#functor_declaration
- * 	 @
- * 	 self#constructor_declaration
- * 	 @
- * 	 [self#coalgebra_decl]
- * 	 @
- * 	 self#super_access
- * 	 @
- * 	 ((Comment( "Method abbreviations for " ^ cl#get_name ))
- * 	  :: (self#method_abbreviations))
- * 	 @
- * 	 if cl#get_ancestors <> [] then 
- * 	   ((Comment "Inherit method from super classes") ::
- * 	    self#all_inherited_methods)
- * 	 else []
- * 
- * end
- * 
- * class old_ccsl_interface_theory cl = 
- *   [ccsl_iface_type, ccsl_member_type] old_ccsl_pre_interface_theory cl 
- *   eq_ccsl_types 
- * 
- *)
-
 
 
 (***********************************************************************
@@ -597,24 +372,28 @@ class ['class_type, 'member_type] ccsl_pre_coadt_theory
 	   ccsl_basic_theory_name cl, [];
 	   ccsl_greatest_invariance_theory_name cl, []
 	 ]
-       @ 
-       (if cl#has_feature HasFullRelLiftingFeature 
-       then [ccsl_greatest_bibisim_theory_name cl,[]]
-       else [])
-       @
-       (if cl#has_feature HasMorphismFeature
-	then [ccsl_morphism_rewrite_theory_name cl,[];
-	      ccsl_finality_theory_name cl,[]]
-	else [])
-       @
-       (if cl#has_feature FinalSemanticsFeature
-	then [ccsl_final_props_theory_name cl,[]]
-	else [ccsl_loose_theory_name cl, []])
-       @
-       (if cl#has_feature HasMapFeature
-	then [ccsl_map_struct_theory_name cl,[];
-	      ccsl_map_theory_name cl,[]]
-	else [])
+	 @
+	 (if cl#has_feature HasBisimulationFeature 
+	  then [ccsl_bibisim_theory_name cl,[]]
+	  else [])
+	 @ 
+	 (if cl#has_feature HasFullRelLiftingFeature 
+	  then [ccsl_greatest_bibisim_theory_name cl,[]]
+	  else [])
+	 @
+	 (if cl#has_feature HasMorphismFeature
+	  then [ccsl_morphism_rewrite_theory_name cl,[];
+		ccsl_finality_theory_name cl,[]]
+	  else [])
+	 @
+	 (if cl#has_feature FinalSemanticsFeature
+	  then [ccsl_final_props_theory_name cl,[]]
+	  else [ccsl_loose_theory_name cl, []])
+	 @
+	 (if cl#has_feature HasMapFeature
+	  then [ccsl_map_struct_theory_name cl,[];
+		ccsl_map_theory_name cl,[]]
+	  else [])
        )]
 
 end
@@ -929,4 +708,5 @@ class ccsl_final_props_theory cl =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
