@@ -18,11 +18,11 @@
  *
  * Created 17.6.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Wednesday 15 May 02 18:42:48 tews@ithif51>
  *
  * Lifttings and invariants
  *
- * $Id: invariance_theory.ml,v 1.5 2002/01/23 16:00:16 tews Exp $
+ * $Id: invariance_theory.ml,v 1.7 2002/05/22 13:42:40 tews Exp $
  *
  *)
 
@@ -33,7 +33,6 @@ open Top_classtypes
 open Name_space
 open Names
 open Top_names
-open Logic_util
 open Classtypes
 open Types_util
 open Pvs_proof_util
@@ -101,7 +100,7 @@ class ['class_type, 'member_type] ccsl_pre_lift_theory
 		    | Resolved_renaming(_,args,_,ianc) -> ianc,args
 					(* no other stuff here *)
 		    | Unresolved_renaming _
-		    | _ -> (assert(false); raise Internal_error)
+		    | _ -> assert(false)
 		 ) cl#get_ancestors
       in
       	Defn(class_lifting cl,
@@ -390,7 +389,7 @@ class virtual ['class_type, 'member_type] ccsl_pre_invariance_rewrite_theory
 			   collect_anc_names anc
 					(* no other stuff here *)
 		       | Unresolved_renaming _
-		       | _ -> (assert(false); raise Internal_error)
+		       | _ -> assert(false)
 		    ) cl#get_ancestors))
 	 @
 	 (List.map (function
@@ -398,7 +397,7 @@ class virtual ['class_type, 'member_type] ccsl_pre_invariance_rewrite_theory
 			  ccsl_lift_theory_name anc
 					(* no other stuff here *)
 		      | Unresolved_renaming _
-		      | _ -> (assert(false); raise Internal_error)
+		      | _ -> assert(false)
 		   ) cl#get_ancestors) in	
 
       let anc_inv_th_list = 
@@ -503,7 +502,7 @@ class virtual ['class_type, 'member_type] ccsl_pre_invariance_rewrite_theory
     val list_adt = 
 		(match Symbol.find "list" with 
 		   | AdtSymb( adt ) -> adt
-		   | _ -> (assert(false); raise Internal_error)
+		   | _ -> assert(false)
 		)
 		       
     method private inv_list =
@@ -714,5 +713,6 @@ class ccsl_public_invariance_rewrite_theory cl =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
