@@ -18,11 +18,11 @@
  *
  * Created 17.6.99 by Jan
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Wednesday 22 May 02 10:24:14 tews@ithif51>
  *
  * Liftings and invariants
  *
- * $Id: methodinv_theory.ml,v 1.9 2002/01/24 14:44:00 tews Exp $
+ * $Id: methodinv_theory.ml,v 1.11 2002/05/22 13:42:41 tews Exp $
  *
  *)
 
@@ -33,7 +33,6 @@ open Top_classtypes
 open Name_space
 open Names
 open Top_names
-open Logic_util
 open Classtypes
 open Types_util
 open Pvs_proof_util
@@ -44,7 +43,70 @@ open Theory_class;;
 (***********************************************************************
  ***********************************************************************
  *
- * Liftings
+ * method identifier datatype
+ *
+ *)
+
+class ['class_type, 'member_type] ccsl_pre_method_idtype_adt
+  (cl : 'class_type) 
+  (eq_types : ('class_type, 'member_type) top_pre_types
+     -> ('class_type, 'member_type) top_pre_types -> bool)
+  : ['class_type, 'member_type] ccsl_pre_class_theory_type =
+  object (self : 'self)
+    
+    constraint 'class_type = ccsl_iface_type
+    constraint 'member_type = ccsl_member_type
+
+    inherit ccsl_method_inv_helper_class cl
+      as helper
+
+    inherit 
+      ['class_type, 'member_type] ccsl_pre_class_theory_class cl eq_types
+      as top_theory
+
+      (* This is a datatype declaration ! *)
+    method kind = Datatype
+
+    method get_name = ccsl_method_idtype_adt_name cl
+
+    method get_parameters = []
+
+    method get_proofs = []
+
+    initializer top_theory#override_file_name (ccsl_class_file_name cl)
+
+
+
+    method private method_id_enum =
+      let methods = self#get_lifting_methods
+      in
+        if (List.length methods) > 0 then
+	  Datatypedecl("", [],
+		       List.map
+			 (fun m -> (name_of_method_tag m, [], 
+				    name_of_method_tag_recognizer m)
+			 )
+			 methods
+		      )
+        else
+	  Datatypedecl("", [],
+		       [("no_method", [], "no_method?")]
+		      )
+
+    method make_body =
+      [self#method_id_enum]
+end
+
+
+class ccsl_method_idtype_adt cl = 
+  [ccsl_iface_type, ccsl_member_type] 
+  ccsl_pre_method_idtype_adt cl eq_ccsl_types
+
+
+(***********************************************************************
+ ***********************************************************************
+ *
+ * method liftings
  *
  *)
 
@@ -54,9 +116,13 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
      -> ('class_type, 'member_type) top_pre_types -> bool)
   : ['class_type, 'member_type] ccsl_pre_class_theory_type =
   object (self : 'self)
+    
+    constraint 'class_type = ccsl_iface_type
+    constraint 'member_type = ccsl_member_type
 
     inherit ccsl_method_inv_helper_class cl
       as helper
+
     inherit 
       ['class_type, 'member_type] ccsl_pre_class_theory_class cl eq_types
       as top_theory
@@ -72,11 +138,6 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
          name_of_initially cl;
          name_of_method_invariance cl;
        ]
-       @
-       (List.map
-	  (fun m -> m#get_name)
-	  cl#get_methods
-       )
       )
       
     initializer name_of_list <- create_one_id ns self#list_type
@@ -85,22 +146,37 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
 
     method get_parameters = self#simple_parameters 
 
-    initializer top_theory#override_file_name (ccsl_box_file_name cl)
+    initializer top_theory#override_file_name (ccsl_class_file_name cl)
 
-    method private enum_type_decl =
-      let methods = 
-	List.filter (fun m -> m#needs_lifting && m#is_method) 
-	  cl#get_all_members
-      in
-	if (List.length methods) > 0 then
-	  Enumdecl(name_of_method_enum_type cl,
-		   List.map
-                     (fun m -> name_of_method_tag m)
-                     methods
-		  )
-	else
-	  Typevardecl(name_of_method_enum_type cl)
-      
+(* 	  (* compute list of methods that need lifting *)
+ *     val mutable lifting_methods = None
+ * 
+ *     method private get_lifting_methods =
+ * 	 match lifting_methods with
+ * 	   | Some ms -> ms
+ * 	   | None -> 
+ * 	       let methods = 
+ * 		 List.filter (fun m -> m#needs_lifting && m#is_method) 
+ * 		   cl#get_all_members
+ * 	       in
+ * 		 lifting_methods <- Some methods;
+ * 		 methods
+ * 
+ *)
+(*     method private enum_type_decl =
+ * 	 let methods = self#get_lifting_methods
+ * 	 in
+ * 	   if (List.length methods) > 0 then
+ * 	     Enumdecl(name_of_method_enum_type cl,
+ * 		      List.map
+ * 			(fun m -> name_of_method_tag m)
+ * 			methods
+ * 		     )
+ * (* HENDRIK : this should not produce an anonymous type *)
+ * 	   else
+ * 	     Typevardecl(name_of_method_enum_type cl)
+ * 	 
+ *)
     method private pred_type_decl =
       Typedecl([self_parameter],name_of_method_pred_type cl,
                Function(
@@ -123,7 +199,7 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
      * 		    (* no other stuff *)
      * 	      | Unresolved_renaming _
      * 	      | Resolved_ancestor _ 
-     * 	      | Unresolved_ancestor _ -> (assert(false); raise Internal_error)
+     * 	      | Unresolved_ancestor _ -> assert(false)
      * 	   )
      * 	   cl#get_ancestors
      *)
@@ -140,7 +216,7 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
 		 (ccsl_greatest_invariance_theory_name comp, 
 		  (comp#get_model_type_argument args) :: args) :: accu
 	     | Spec_sig -> accu
-	     | Spec_Spec -> (assert(false); raise Internal_error)
+	     | Spec_Spec -> assert(false)
 	)
 	cl#get_components
 	[]
@@ -159,8 +235,7 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
 		  self#coalgebra_type
                )
 	 (* DO NOT RELY ON SUPERCLASSES (see above or below) *)
-	(List.filter (fun m -> m#needs_lifting && m#is_method) 
-	   cl#get_all_members)
+	self#get_lifting_methods
 
     method private do_class_lifting =
       let sub_ns = sub_space ns in
@@ -258,9 +333,7 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
       let p = create_one_id sub_ns (Function(Self,Bool)) in
       let self_var = create_one_id sub_ns Self in
       let pred_ex = Term(p,Always,[]) in
-      let methods = 
-	List.filter (fun m -> m#needs_lifting && m#is_method) 
-	  cl#get_all_members
+      let methods = self#get_lifting_methods
       in let full_list = 
 	  List( List.map (fun m -> Term(name_of_method_tag m,Always,[]))
 		  methods)
@@ -279,6 +352,26 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
 		  full_list),
 		Term(p,Always,[]))))
 
+
+    method private method_struct_invariance =
+      let sub_ns = sub_space ns in 
+      let (decl_list, coalgebra_from_tuple ) = 
+	self#coalgebra_as_tuple sub_ns in
+      let pred_type = Function(Self,Bool) in
+      let p = create_one_id sub_ns pred_type 
+      in
+	Defn(name_of_private_struct_invariance cl,
+	     [decl_list; [ Undeclared(p, pred_type) ]],
+	     Bool,
+	     Application(
+	       Application(
+		 Term(name_of_private_invariance cl, Always, []),
+		 coalgebra_from_tuple
+	       ),
+	       Term(p,Always,[])
+	     ))
+
+
     method private complete_list =
       let sub_ns = sub_space ns in
       let m = create_one_id sub_ns self#method_enum_type 
@@ -301,12 +394,8 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
 
     method make_body =
       [ Import(
-          [ccsl_interface_theory_name cl, self#simple_arguments]
-	(* DO NOT RELY ON SUPERCLASSES (see above)
-	 * 	     @
-	 * 	     self#super_imports
-	 *)
-	  @
+          (ccsl_interface_theory_name cl, self#simple_arguments) ::
+	  (ccsl_method_idtype_adt_name cl, []) ::
 	  self#component_imports
         );
         (self#coalgebra_decl)
@@ -316,9 +405,10 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
          [(self#algebra_decl)]
        else [])
       @
-      [Comment("Enumeration type for methods");
-       self#enum_type_decl;
-       Comment("Type of the lifting wrt. a single  method");
+(* 	 [Comment("Enumeration type for methods");
+ * 	  self#enum_type_decl;
+ *)
+      [Comment("Type of the lifting wrt. a single  method");
        self#pred_type_decl;
        Comment("Definitions of the liftings for methods")
       ]
@@ -335,6 +425,7 @@ class ['class_type, 'member_type] ccsl_pre_method_lift_theory
       [
         self#method_invariance;
 	self#invariance;
+	self#method_struct_invariance;
 	self#complete_list
       ]
         
@@ -379,15 +470,16 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
           name_of_inv_true_lemma;
 	  name_of_inv_and_lemma;
 	  name_of_inv_list_lemma;
-	  name_of_inv_linear_lemma;
+	  name_of_inv_linear_cons_lemma;
+	  name_of_inv_linear_append_lemma;
 	  name_of_method_id;
         ])
 
-    initializer name_of_list <- create_one_id ns self#list_type
-
+(*     initializer name_of_list <- create_one_id ns self#list_type
+*)
      method get_parameters = self#simple_parameters
                                
-     initializer top_theory#override_file_name (ccsl_box_file_name cl)
+     initializer top_theory#override_file_name (ccsl_class_file_name cl)
        
      method get_proofs = []
                            
@@ -399,6 +491,20 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
        Expression(ConstantPredicate(name_of_method_invariance cl))
          
 
+       (* compute list of methods that get a rewrite lemma *)
+    val mutable rewrite_methods = None
+
+    method private get_rewrite_methods =
+      match rewrite_methods with
+	| Some ms -> ms
+	| None -> 
+	    let methods = 
+	      List.filter (fun m -> m#needs_lifting && m#is_action) 
+		cl#get_all_members
+	    in
+	      rewrite_methods <- Some methods;
+	      methods
+
 
       (* 
        *   inv_add : LEMMA
@@ -415,6 +521,7 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
       let sub_ns = sub_space ns in
       let _ = make_flat sub_ns in
       let self_var = create_one_id sub_ns Self in
+      let l = create_one_id sub_ns self#list_type in
       let self_lifting = 
 	(None, 
 	 Some(Expression(
@@ -428,14 +535,14 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
 	  Formula(
 	    Application(
 	      Term(name_of_complete_inv_id_list cl, Always,[]),
-	      self#list_term))
+	      Term(l,Always,[])))
 	else
 	  Formula(
 	    SmartApplication(
 	      Term(name_of_list_member(), Always, []),
 	      [
 		Term(name_of_method_tag m, Always, []);
-		self#list_term]
+		Term(l,Always,[])]
 	    ))
       in
       let prem = And([
@@ -444,7 +551,7 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
 			   Application(
 		      	     Application(self#invariant_pred,
 					 coalgebra_term),
-			     self#list_term),
+			     Term(l,Always,[])),
 		      	   Term("P",Always,[]))
 		       );
 		       member_prem;
@@ -453,7 +560,7 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
 				  Term(self_var,Always,[])))
 		     ]) in
       let form = 
-	Forall([self_var,Self],
+	Forall([(self_var,Self); (l, self#list_type)],
 	       (Implies
 		  (prem,
 		   class_pred_lifting
@@ -494,8 +601,8 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
      method private pqlm_declaration = 
         [Vardecl("P", Function(Self,Bool)); 
          Vardecl("Q", Function(Self,Bool)); 
-         (helper#list_decl);
-	 (helper#method_decl)
+(*         (helper#list_decl);
+*)	 (helper#method_decl)
         ] 
 
      (*   
@@ -503,24 +610,26 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
       *) 
      method private inv_true =  
        let sub_ns = sub_space ns in 
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in 
 	 Proved(
-	   Lemma(  
+	   Lemma(
              name_of_inv_true_lemma, 
+	     Forall([l, self#list_type],
              Formula( 
                Application( 
 		 Application( 
                    Application( 
                      self#invariant_pred, 
                      coalgebra_term), 
-                   self#list_term 
+                   Term(l,Always,[]) 
 		 ), 
 		 Abstraction( 
                    [x,Self], 
                    Expression(True) 
 		 ) 
                ) 
-             ) 
+             )) 
            ),
 	   Anon_proof( make_simple_proof(
 			 [ grind ])
@@ -535,35 +644,37 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
   
      method private inv_and = 
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
 	 Lemma(
 	   name_of_inv_and_lemma,
+	   Forall([(l, self#list_type)],
 	   Implies(
 	     And(
 	       [Formula(
 		  Application(
 		    Application(
 		      Application(self#invariant_pred,coalgebra_term),
-		      self#list_term),
+		      Term(l,Always,[])),
 		    p_pred));
 	       	Formula(
 		  Application(
 		    Application(
 		      Application(self#invariant_pred,coalgebra_term),
-		      self#list_term),
+		      Term(l,Always,[])),
 		    q_pred))]),
 	     Formula(
 	       Application(
 		 Application(
 	      	   Application(self#invariant_pred,coalgebra_term),
-		   self#list_term),
+		   Term(l,Always,[])),
 	      	 Abstraction(
 		   [x,Self],
 		   Expression(
 		     And(
 		       [Formula(Application(p_pred,Term(x,Always,[])));
 		       	Formula(Application(q_pred,Term(x,Always,[])))
-		       ])))))))   
+		       ]))))))))
 	   
       (* 
        *   inv_list : LEMMA
@@ -582,15 +693,16 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
        match !output_mode with
 	 | Pvs_mode -> 
 	     let sub_ns = sub_space ns in
+	     let l = create_one_id sub_ns self#list_type in
 	     let x = create_one_id sub_ns Self in
-	     let list_type = Adt(self#list_adt, Always,
+	     let pred_list_type = Adt(self#list_adt, Always,
 				 [TypeArgument(
 				    Function(Self,Bool))]) in
-	     let z = create_one_id sub_ns list_type in
+	     let z = create_one_id sub_ns pred_list_type in
 	       Lemma(
 		 name_of_inv_list_lemma,
 		 Forall(
-		   [z, list_type],
+		   [(z, pred_list_type); (l, self#list_type) ],
 		   Implies(
 		     Formula(
 		       Application(
@@ -598,13 +710,13 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
 	      		   Term(name_of_adt_every self#list_adt,Always,[]),
 			   Application(
 			     Application(self#invariant_pred,coalgebra_term),
-			     self#list_term)),
+			     Term(l,Always,[]))),
 			 Term(z,Always,[]))),
 		     Formula(
 		       Application(
 			 Application(
 			   Application(self#invariant_pred,coalgebra_term),
-			   self#list_term),
+			   Term(l,Always,[])),
 			 Application(
 			   Application(
 			     Term(name_of_adt_reduce self#list_adt, Always,[]),
@@ -631,16 +743,19 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
 		       ^ "To be implemented.")
 
 
-     method private inv_linear =
+     method private inv_linear_cons =
+       let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        Lemma(
-	 name_of_inv_linear_lemma,
+	 name_of_inv_linear_cons_lemma,
+	 Forall([l, self#list_type],
 	 Implies(
 	   Formula(
 	     Application(
 	       Application(
 		 Application(self#invariant_pred,coalgebra_term),
 		 Application(Term("cons",Always, []),
-			     Tuple([self#method_term;self#list_term]))
+			     Tuple([self#method_term;Term(l,Always,[])]))
 	       ),
 	       p_pred
 	     )
@@ -649,13 +764,48 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
 	     Application(
 	       Application(
 		 Application(self#invariant_pred,coalgebra_term),
-		 self#list_term),
+		 Term(l,Always,[])),
 	       p_pred
 	     )
 	   )
-	 )
+	 ))
        )
-	       
+
+     method private inv_linear_append =
+       let sub_ns = sub_space ns in 
+       let (l1,l2) = create_id_pairs sub_ns [self#list_type] in
+       let l1_term = Term(fst (List.hd l1), Always, []) in
+       let l2_term = Term(fst (List.hd l2), Always, []) in
+	 Lemma(
+	   name_of_inv_linear_append_lemma,
+	   Forall(
+	     l1@l2, 
+	     Iff(
+	       And[
+		 Formula(
+		   Application(
+		     Application(
+		       Application(self#invariant_pred,coalgebra_term),
+		       l1_term),
+		     p_pred));
+		 Formula(
+		   Application(
+		     Application(
+		       Application(self#invariant_pred,coalgebra_term),
+		       l2_term),
+		     p_pred))
+	       ],
+	       Formula(
+		 Application(
+		   Application(
+		     Application(self#invariant_pred,coalgebra_term),
+		     Application(
+		       Term("append",Always,[]),
+		       Tuple[l1_term; l2_term])
+		   ),
+		   p_pred))
+	     )))
+
 	      
      method get_name = ccsl_method_invariance_rewrite_name cl
 			 
@@ -676,19 +826,178 @@ class ['class_type, 'member_type] ccsl_pre_method_invariance_rewrite_theory
          self#inv_true;
          self#inv_and; 
          self#inv_list;
-	 self#inv_linear
+	 self#inv_linear_cons;
+	 self#inv_linear_append
        ]  
-       @ 
+       @
        Comment("Rewrite lemmas for methods"):: 
-       (List.map self#do_method 
-	  (List.filter (fun m -> m#needs_lifting && m#is_action) 
-	     cl#get_all_members)) 
+       (List.map self#do_method self#get_rewrite_methods) 
 
   end (* ccsl_pre_method_invariance_rewrite_theory *)
 
 class ccsl_method_invariance_rewrite_theory cl = 
   [ccsl_iface_type, ccsl_member_type] ccsl_pre_method_invariance_rewrite_theory cl eq_ccsl_types
 
+
+(***********************************************************************
+ ***********************************************************************
+ *
+ * inherit methodwise invariance
+ *
+ *
+ * this theory is reused below to generate the lemmas for inheriting Box
+ *
+ *)
+
+
+class ['class_type, 'member_type] ccsl_pre_method_invariance_inherit_theory
+  (cl : 'class_type) 
+  (* the equality relation for initializing the name space *)
+  (eq_types : ('class_type, 'member_type) top_pre_types
+     -> ('class_type, 'member_type) top_pre_types -> bool)
+
+  (* without type restriction -- we want to inherit from this one *)
+  = 
+  object (self : 'self)
+
+     inherit 
+       ['class_type, 'member_type] ccsl_pre_class_theory_class 
+       cl eq_types    as top_theory
+
+     method get_parameters = self#simple_parameters
+                               
+     initializer top_theory#override_file_name (ccsl_class_file_name cl)
+       
+     method get_name = ccsl_method_invariance_inherit_name cl
+			 
+     method get_proofs = []
+                           
+     (* this gets overriden in derived classes to specialize
+      * this class for private or public invariance
+      *)
+                           
+     method private invariant_pred =
+       Expression(ConstantPredicate(name_of_method_invariance cl))
+         
+
+(* Apparently not needed
+ *
+ * 	method private super_imports = 
+ * 	  List.map 
+ * 	    (fun ianc -> 
+ * 	       ccsl_basic_theory_name ianc, self_argument:: ianc#get_arguments)
+ * 	    cl#get_resolved_ancestors
+ * 	    
+ *)
+
+	(**********************************************************
+	 *
+	 * override section: the following methods are adopted 
+	 * for the Box case
+	 *)
+	 
+     method private describe = "methodwise invariants"
+
+     method private lemma_name acl = name_of_inv_inherit_lemma acl
+
+	 (* there is no x here, but one in Box *)
+     method private get_x_decl_list = []
+
+     method private this_class_pred l_term p_term = 
+       Application(
+	 Application(
+	   Application(self#invariant_pred, coalgebra_term),
+	   l_term),
+	 p_term)       
+
+     method private super_class_pred acl l_term p_term = 
+       Application(
+	 Application(
+	   Application(
+	     self#invariant_pred, 
+	     Application(Term(super_access_method(acl),
+			      Always,[]), 
+			 coalgebra_term)),
+	   l_term),
+	 p_term)
+
+
+	(*
+	 * end override section
+	 *
+	 **************************************************************)
+
+
+     method private do_inv_inherit acl amethods = 
+       let sub_ns = sub_space ns in 
+       let p_type = Function(Self, Bool) in
+       let p = create_one_id sub_ns p_type in
+       let p_pred = Term(p, Always,[]) in
+       let mlist = 
+	 List(List.map (fun m -> Term(name_of_method_tag m, Always, []))
+		amethods)
+       in let alist = mlist		(* anticipate change here *)
+       in
+	 Lemma(
+	   self#lemma_name acl,
+	   Forall(
+	     [(p,p_type)] @ self#get_x_decl_list,
+	     Iff(
+	       Formula(
+		 self#this_class_pred mlist p_pred),
+	       Formula(
+		 self#super_class_pred acl alist p_pred)
+	     )))
+
+
+     method private do_inv_inherit_maybe acl =
+       if count_self_args acl#get_arguments <> 0
+       then
+	 [Comment 
+	    ("No lemma for inherited " ^ self#describe ^ " from " ^
+	     acl#get_name ^ ":\n" ^
+	     "instanciation for " ^ acl#get_name ^ " is not constant")
+	 ]
+       else
+	 let amethods = 
+	   List.filter (fun m -> m#needs_lifting && m#is_method) 
+	     acl#get_all_members
+	 in
+	   if amethods = [] 
+	   then
+	     [Comment("No inherited methods from " ^ acl#get_name ^ 
+		      "have a successor state")
+	     ]
+	   else
+	     [Comment
+		("Inherit " ^ self#describe ^ " from " ^ acl#get_name );
+	      self#do_inv_inherit acl amethods
+	     ]
+	     
+     method make_body =
+       [ 
+         Import([ccsl_method_lift_theory_name cl, self#simple_arguments]); 
+         self#coalgebra_decl; 
+       ] 
+       @
+       [Comment "Link methodwise invariance with ancestor classes"]
+       @
+       (List.flatten
+	  (List.map
+	     self#do_inv_inherit_maybe
+	     cl#get_resolved_ancestors)
+       )
+
+  end (* ccsl_pre_method_invariance_rewrite_theory *)
+
+class ccsl_method_invariance_inherit_theory cl = 
+  ([ccsl_iface_type, ccsl_member_type] 
+     ccsl_pre_method_invariance_inherit_theory cl eq_ccsl_types
+     : [ccsl_iface_type, ccsl_member_type] ccsl_pre_class_theory_type)
+
+
+
+
 (***********************************************************************
  ***********************************************************************
  *
@@ -990,7 +1299,9 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 	]
        )
 
-     initializer name_of_list <- create_one_id ns self#list_type
+(* work with item wise list declarations
+ * 	initializer name_of_list <- create_one_id ns self#list_type
+ *)
 
      method private diamond_term = Term(name_of_diamond, Always, []) 
 
@@ -1027,19 +1338,22 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
      method private pqlm_declaration = 
         [Vardecl("P", pred_type); 
          Vardecl("Q", pred_type); 
-         (helper#list_decl);
+(* work with itemwise list declarations
+ * 	    (helper#list_decl);
+ *)
 	 (helper#method_decl)
         ] 
 
      method private box_def =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
        let y = create_one_id sub_ns Self in
        let y_term = Term(y, Always, []) in
 	 Defn(
 	   name_of_box,
 	   [[Declared(name_of_coalgebra, self#coalgebra_type)];
-	    [Declared(name_of_list, self#list_type)];
+	    [Undeclared(l, self#list_type)];
 	    [Declared("P",pred_type)]],
 	   pred_type,
 	   Abstraction(
@@ -1060,7 +1374,7 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 				    Isabelle_only,
 				    self#simple_arguments),
 			       coalgebra_term),
-			     self#list_term),
+			     Term(l,Always,[])),
 			   q_pred));
 		       Forall(
 			 [y, Self],
@@ -1084,13 +1398,14 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 
      method private diamond_def =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let vars = create_ids sub_ns [Self;Self] in
        let x : string = fst(List.nth vars 0) in
        let y : string = fst(List.nth vars 1) in
 	 Defn(
 	   name_of_diamond,
 	   [[Declared(name_of_coalgebra, self#coalgebra_type)];
-	    [Declared(name_of_list, self#list_type)];
+	    [Undeclared(l, self#list_type)];
 	    [Declared("P",pred_type)]],
 	   pred_type,
 	   Abstraction(
@@ -1099,7 +1414,7 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 	       Not(
 		 Formula(
 		   Application(
-		     self#box_csp self#list_term
+		     self#box_csp (Term(l,Always,[]))
 				 (Abstraction(
 				    [x, Self],
 				    Expression(
@@ -1114,10 +1429,12 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 
      method private box_K =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
        let x_term = Term(x, Always, []) in
          Lemma(
 	   name_of_box_K_lemma cl,
+	   Forall([l, self#list_type],
 	   Implies(
 	     Forall(
 	       [x,Self],
@@ -1135,32 +1452,33 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 	       Implies(
 		 Formula(
 		   Application(
-		     self#box_csp self#list_term p_pred,
+		     self#box_csp (Term(l,Always,[])) p_pred,
 		     x_term)
 		 ),
 		 Formula(
 		   Application(
-		     self#box_csp self#list_term q_pred,
+		     self#box_csp (Term(l,Always,[])) q_pred,
 		     x_term)
 		 )
 	       )
 	     )
-	   )
+	   ))
 	 )
 
 	   
      method private box_T =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
        let x_term = Term(x, Always, []) in
 	 Lemma(
 	   name_of_box_T_lemma cl,
 	   Forall(
-	     [x,Self],
+	     [(l, self#list_type); (x,Self)],
 	     Implies(
 	       Formula(
 		 Application(
-		   self#box_csp self#list_term p_pred,
+		   self#box_csp (Term(l,Always,[])) p_pred,
 		   x_term
 		 )
 	       ),
@@ -1176,12 +1494,13 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 
      method private diamond_T =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
        let x_term = Term(x, Always, []) in
 	 Lemma(
 	   name_of_diamond_T_lemma cl,
 	   Forall(
-	     [x,Self],
+	     [(l, self#list_type); (x,Self)],
 	     Implies(
 	       Formula(
 		 Application(
@@ -1191,7 +1510,7 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 	       ),
 	       Formula(
 		 Application(
-		   self#diamond_csp self#list_term p_pred,
+		   self#diamond_csp (Term(l,Always,[])) p_pred,
 		   x_term
 		 )
 	       )
@@ -1201,22 +1520,23 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 	   
      method private box_4 =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
        let x_term = Term(x, Always, []) in
 	 Lemma(
 	   name_of_box_4_lemma cl,
 	   Forall(
-	     [x,Self],
+	     [(l, self#list_type); (x,Self)],
 	     Implies(
 	       Formula(
 		 Application(
-		   self#box_csp self#list_term p_pred,
+		   self#box_csp (Term(l,Always,[])) p_pred,
 		   x_term)
 	       ),
 	       Formula(
 		 Application(
-		   self#box_csp self#list_term
-			       (self#box_csp self#list_term p_pred),
+		   self#box_csp (Term(l,Always,[]))
+			       (self#box_csp (Term(l,Always,[])) p_pred),
 			       x_term
 		 )
 	       )
@@ -1226,23 +1546,24 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 	   
      method private diamond_4 =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
        let x_term = Term(x, Always, []) in
 	 Lemma(
 	   name_of_diamond_4_lemma cl,
 	   Forall(
-	     [x,Self],
+	     [(l, self#list_type); (x,Self)],
 	     Implies(
 	       Formula(
 		 Application(
-		   self#diamond_csp self#list_term
-			       (self#diamond_csp self#list_term p_pred),
+		   self#diamond_csp (Term(l,Always,[]))
+			       (self#diamond_csp (Term(l,Always,[])) p_pred),
 			       x_term
 		 )
 	       ),
 	       Formula(
 		 Application(
-		   self#diamond_csp self#list_term p_pred,
+		   self#diamond_csp (Term(l,Always,[])) p_pred,
 		   x_term)
 	       )
 	     )
@@ -1250,45 +1571,52 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 	 )
 
      method private box_inv =
+       let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        Lemma(
 	 name_of_box_inv_lemma cl,
+	 Forall([l, self#list_type],
 	 Formula(
 	   Application(
 	     Application(
 	       Application(
 		 Term(name_of_method_invariance cl, Always, []),
 		 coalgebra_term),
-	       self#list_term),
-	     (self#box_csp (self#list_term) p_pred)
+	       Term(l,Always,[])),
+	     (self#box_csp (Term(l,Always,[])) p_pred)
 	   )
-	 )
+	 ))
        )
 
-     method private diamond_inv =
-       let sub_ns = sub_space ns in
-       let x = create_one_id sub_ns Self in
-	 Lemma(
-	   name_of_diamond_inv_lemma cl,
-	   Not(
-	     Formula(
-	       Application(
-		 Application(
-		   Application(
-		     Term(name_of_method_invariance cl, Always, []),
-		     coalgebra_term),
-		   self#list_term),
-		 Abstraction(
-		   [x,Self],
-		   Expression(
-		     Not(
-		       Formula(
-			 Application(
-			   self#diamond_csp (self#list_term) p_pred,
-			   Term(x,Always,[]))))))))))
+(* This lemma is not true :-(
+ *
+ * 	method private diamond_inv =
+ * 	  let sub_ns = sub_space ns in
+ * 	  let x = create_one_id sub_ns Self in
+ * 	    Lemma(
+ * 	      name_of_diamond_inv_lemma cl,
+ * 	      Not(
+ * 		Formula(
+ * 		  Application(
+ * 		    Application(
+ * 		      Application(
+ * 			Term(name_of_method_invariance cl, Always, []),
+ * 			coalgebra_term),
+ * 		      Term(l,Always,[])),
+ * 		    Abstraction(
+ * 		      [x,Self],
+ * 		      Expression(
+ * 			Not(
+ * 			  Formula(
+ * 			    Application(
+ * 			      self#diamond_csp (Term(l,Always,[])) p_pred,
+ * 			      Term(x,Always,[]))))))))))
+ *)
 	 
 
      method private box_greatest_inv =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
        let y = create_one_id sub_ns Self in
        let x_term = Term(x, Always, []) in
@@ -1296,7 +1624,7 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 	 Lemma(
 	   name_of_box_greatest_inv_lemma cl,
 	   Forall(
-	     [y, Self],
+	     [(l, self#list_type); (y,Self)],
 	     Implies(
 	       And([
 		     Formula(
@@ -1305,7 +1633,7 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 			   Application(
 			     Term(name_of_method_invariance cl, Always, []),
 			     coalgebra_term),
-			   self#list_term),
+			   Term(l,Always,[])),
 			 p_pred)
 		     );
 		     Forall(
@@ -1325,21 +1653,22 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 		   ]),
 	       Formula(
 		 Application(
-		   (self#box_csp self#list_term q_pred),
+		   (self#box_csp (Term(l,Always,[])) q_pred),
 		   y_term)
 	       )
 	     )
 	   )
 	 )
 			       
-     method private box_linear =
+     method private box_linear_cons =
        let sub_ns = sub_space ns in
+       let l = create_one_id sub_ns self#list_type in
        let x = create_one_id sub_ns Self in
        let x_term = Term(x, Always, []) in
        Lemma(
-	 name_of_box_linear_lemma cl,
+	 name_of_box_linear_cons_lemma cl,
 	 Forall(
-	   [x,Self],
+	   [(l, self#list_type); (x,Self)],
 	   Implies(
 	     Formula(
 	       Application(
@@ -1347,32 +1676,65 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 		   (Application(Term("cons", Always, []),
 			       	Tuple([
 				       	self#method_term;
-				       	self#list_term])
+				       	Term(l,Always,[])])
 			       ))
 		   p_pred,
 		 x_term)
 	     ),
 	     Formula(
 	       Application(
-		 self#box_csp self#list_term p_pred,
+		 self#box_csp (Term(l,Always,[])) p_pred,
 		 x_term)
 	     )
 	   )
 	 )
        )
 
+     method private box_linear_append =
+       let sub_ns = sub_space ns in
+       let (l1,l2) = create_id_pairs sub_ns [self#list_type] in
+       let l1_term = Term(fst (List.hd l1), Always, []) in
+       let l2_term = Term(fst (List.hd l2), Always, []) in
+       let x = create_one_id sub_ns Self in
+       let x_term = Term(x, Always, []) in
+       Lemma(
+	 name_of_box_linear_append_lemma cl,
+	 Forall(
+	   l1 @ l2 @ [(x,Self)],
+	   Iff(
+	     And[
+	       Formula(
+		 Application(
+		   self#box_csp l1_term p_pred,
+		   x_term)
+	       );
+	       Formula(
+		 Application(
+		   self#box_csp l2_term p_pred,
+		   x_term)
+	       )],
+	     Formula(
+	       Application(
+		 self#box_csp 
+		    (Application(
+		       Term("append", Always, []),
+		       Tuple([l1_term; l2_term])))
+		    p_pred,
+		 x_term))
+	   )))
+
 
      method get_name = ccsl_box_theory_name cl
 
      method get_parameters = self#simple_parameters
                                
-     initializer top_theory#override_file_name (ccsl_box_file_name cl)
+     initializer top_theory#override_file_name (ccsl_class_file_name cl)
 
      method get_proofs = []
 			 
      method make_body =
        [ 
-	    Import([ccsl_method_invariance_rewrite_name cl, 
+	 Import([ccsl_method_invariance_rewrite_name cl, 
 		 self#simple_arguments;
 		]); 
          self#coalgebra_decl; 
@@ -1399,16 +1761,123 @@ class  ['class_type, 'member_type] ccsl_pre_box_theory
 
 	 Comment(" Box is an Invariant ");
 	 self#box_inv;
-	 Comment(" Dual Diamond Rule: Diamond holds  `in one step' ");
-	 self#diamond_inv;
+(* This lemma ist not true :-(
+ *
+ * 	    Comment(" Dual Diamond Rule: Diamond holds  `in one step' ");
+ * 	    self#diamond_inv;
+ *)
 	 self#box_greatest_inv;
-	 self#box_linear
+	 self#box_linear_cons;
+	 self#box_linear_append
        ]
   end (* ccsl_pre_box_theory *)
 
 class ccsl_box_theory cl = 
   [ccsl_iface_type, ccsl_member_type] ccsl_pre_box_theory cl eq_ccsl_types
 
+
+(***********************************************************************
+ ***********************************************************************
+ *
+ * inherit Box
+ *
+ * reuse the inherit method wise theory
+ *)
+
+class ['class_type, 'member_type] ccsl_pre_box_inherit_theory
+  (cl : 'class_type) 
+  (* the equality relation for initializing the name space *)
+  (eq_types : ('class_type, 'member_type) top_pre_types
+     -> ('class_type, 'member_type) top_pre_types -> bool)
+
+  : ['class_type, 'member_type] ccsl_pre_class_theory_type
+  = 
+  object (self : 'self)
+
+     inherit 
+       ['class_type, 'member_type] ccsl_pre_method_invariance_inherit_theory
+       cl eq_types    as top_theory
+
+     method get_name = ccsl_box_inherit_theory_name cl
+			 
+
+	(**********************************************************
+	 *
+	 * override section: the following methods are adopted 
+	 *)
+
+         (* term for "x", this is changed before things get hot *)
+     val mutable x_term = Term("x",Always,[])
+
+	 (* this is changed, once x is allocated *)
+     val mutable x_decl_list = []
+
+     method private get_x_decl_list = x_decl_list
+
+     method private describe = "Box"
+
+     method private lemma_name acl = name_of_box_inherit_lemma acl
+
+     method private this_class_pred l_term p_term = 
+       Application(
+	 Application(
+	   Application(
+	     Application(
+	       Term(name_of_box,Always,[]), 
+	       coalgebra_term),
+	     l_term),
+	   p_term),
+	 x_term)
+
+     method private super_class_pred acl l_term p_term = 
+       Application(
+	 Application(
+	   Application(
+	     Application(
+	       Term(name_of_box,Always,[]),
+	       Application(Term(super_access_method(acl),
+				Always,[]), 
+			   coalgebra_term)),
+	     l_term),
+	   p_term),
+	 x_term)
+
+	(*
+	 * end override section
+	 *
+	 **************************************************************)
+
+     method make_body =
+       let x = create_one_id ns Self in
+       let _ = x_term <- Term(x, Always, []) in
+       let _ = x_decl_list <- [(x, Self)] 
+       in
+	 [ 
+           Import
+	     [(ccsl_box_theory_name cl, self#simple_arguments);
+	      (ccsl_method_invariance_inherit_name cl, self#simple_arguments)
+	     ]; 
+           self#coalgebra_decl; 
+	 ] 
+	 @
+	 [Comment "Link Box with ancestor classes"]
+	 @
+	 (List.flatten
+	    (List.map
+	       self#do_inv_inherit_maybe
+	       cl#get_resolved_ancestors)
+	 )
+
+  end (* ccsl_pre_box_inherit_theory *)
+
+class ccsl_box_inherit_theory cl = 
+  [ccsl_iface_type, ccsl_member_type] 
+  ccsl_pre_box_inherit_theory cl eq_ccsl_types
+
+
+
+
+
 (***********************************************************************
  ***********************************************************************
  *
@@ -1436,7 +1905,7 @@ object (self : 'self)
 
   method get_parameters = cl#get_parameters
 
-  initializer top_theory#override_file_name (ccsl_box_file_name cl)
+  initializer top_theory#override_file_name (ccsl_class_file_name cl)
        
   method make_body = 
     [
@@ -1562,7 +2031,7 @@ class ['class_type, 'member_type] ccsl_pre_path_theory
 	self#list_decl
        ]
 	
-     initializer top_theory#override_file_name (ccsl_box_file_name cl)
+     initializer top_theory#override_file_name (ccsl_class_file_name cl)
 
      method private admissible_def =
        let sub_ns = sub_space ns in
@@ -2023,10 +2492,10 @@ class ['class_type, 'member_type] ccsl_pre_path_theory
 			 
      method make_body =
        [	    
-	 Import([ccsl_box_theory_name cl, 
-		 self#simple_arguments;
-		 ccsl_step_theory_name cl,
-		 orig_arguments;
+	 Import([(ccsl_box_theory_name cl, 
+		  self#simple_arguments);
+		 (ccsl_step_theory_name cl,
+		  orig_arguments);
 		])
        ]
        @
@@ -2054,6 +2523,7 @@ class ccsl_path_theory cl =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
 
