@@ -18,11 +18,11 @@
  *
  * Created 16.03.00 by Jan
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Wednesday 15 May 02 19:27:35 tews@ithif51>
  *
  * Coalgebra morphism theories
  *
- * $Id: morphism_theory.ml,v 1.8 2002/01/24 14:44:00 tews Exp $
+ * $Id: morphism_theory.ml,v 1.10 2002/05/22 13:42:42 tews Exp $
  *
  *)
 
@@ -78,14 +78,13 @@ class ['class_type, 'member_type] ccsl_pre_morphism_theory
       let anc_imports =
 	List.fold_right
 	(fun parent accu -> match parent with
-	   | Resolved_renaming (anc,args,_,_) -> 
+	   | Resolved_renaming (anc,args,_,_,_) -> 
 	       if (count_self_args args) = 0 
 	       then (do_anc anc args) :: accu
 	       else accu
 					(* no other stuff *)
 	   | Unresolved_renaming _
-	   | Resolved_ancestor _ 
-	   | Unresolved_ancestor _ -> (assert(false); raise Internal_error)
+	     -> assert(false)
 	)
 	  cl#get_ancestors 
 	  []
@@ -121,7 +120,7 @@ class ['class_type, 'member_type] ccsl_pre_morphism_theory
 			   :: accu)
 		 | Mixed -> (do_v (do_v accu th_name Pos)
 			       th_name Neg)
-		 | _ -> assert(false) ; raise Internal_error
+		 | _ -> assert(false)
 	       in
 	         match comp#get_kind with
 		 | Spec_class -> 
@@ -136,7 +135,7 @@ class ['class_type, 'member_type] ccsl_pre_morphism_theory
 		       do_v accu (ccsl_adt_map_theory_name comp) v
 		     else accu
 		 | Spec_sig -> accu
-		 | Spec_Spec -> (assert(false); raise Internal_error)
+		 | Spec_Spec -> assert(false)
 	    )
 	    (List.flatten
 	       (cl#get_components ::
@@ -176,7 +175,7 @@ class ['class_type, 'member_type] ccsl_pre_morphism_theory
       let method_list_list, const_ancestor_list = 
 	List.fold_right 
 	  (fun parent (accu_m, accu_anc) -> match parent with 
-	     | Resolved_renaming(_, args,_,ianc) -> 
+	     | Resolved_renaming(_, args,_,ianc,_) -> 
 		 if (count_self_args args) = 0
 		 then (accu_m, (ianc,args) :: accu_anc)
 		 else ((List.filter method_filter ianc#get_all_actions)
@@ -184,7 +183,7 @@ class ['class_type, 'member_type] ccsl_pre_morphism_theory
 
 					(* no other stuff here *)
 	     | Unresolved_renaming _
-	     | _ -> (assert(false); raise Internal_error)
+	     | _ -> assert(false)
 	  ) 
 	  cl#get_ancestors
 	  ([List.filter method_filter cl#get_actions], [])
@@ -221,12 +220,38 @@ class ['class_type, 'member_type] ccsl_pre_morphism_theory
       self#morphism_decl name_of_public_morphism_pred
 	(fun m -> m#is_sig_action && (m#get_visibility = Public))
 
+    method private struct_morphism =
+      let sub_ns = sub_space ns in 
+      let (decl1_list, decl2_list, c1_tuple, c2_tuple ) = 
+	self#two_coalgebras_as_tuple sub_ns in
+      let type_of_morphism = 	  
+	Function(
+	  BoundTypeVariable(self1),
+	  BoundTypeVariable(self2)
+      	)
+      in
+      let f = create_one_id sub_ns type_of_morphism 
+      in
+	Defn(name_of_full_morphism_struct_pred cl,
+	     [decl1_list; decl2_list; [ Undeclared(f, type_of_morphism) ]],
+	     Bool,
+	     Application(
+	       Application(
+		 Term(name_of_full_morphism_pred cl, Always, []),
+		 Tuple([c1_tuple; c2_tuple])
+	       ),
+	       Term(f,Always,[])
+	     ))
+
+
+
     method make_body = 
       [
       	self#imports;
 	self#coalgebra1_decl;
 	self#coalgebra2_decl;
 	self#full_morphism_decl;
+	self#struct_morphism;
 	self#public_morphism_decl
       ]
 
@@ -274,8 +299,8 @@ class ['class_type, 'member_type] ccsl_pre_morphism_rewrite_theory
       Import(
 	[((ccsl_morphism_theory_name cl),
 	  self#double_self_arguments);
-	   ((ccsl_bibisim_theory_name cl),
-	    self#double_self_arguments)
+	 ((ccsl_bibisim_theory_name cl),
+	  self#double_self_arguments)
 	]
       )
 
@@ -605,7 +630,7 @@ class ['class_type, 'member_type] ccsl_pre_finality_theory
       let self2_type = BoundTypeVariable(self2) in
       let (x1,x2) = match create_ids sub_ns [self2_type;self2_type] with
 	  [a1,t1;a2,t2] -> (a1,a2)
-	| _ -> (raise Internal_error)
+	| _ -> assert false;
       in
       let x1_term = Term(x1,Never,[]) in
       let x2_term = Term(x2,Never,[]) in
@@ -735,26 +760,26 @@ class ['class_type, 'member_type] ccsl_pre_map_struct_theory
 	   @ (substitute_arguments_types_only eq_types self#to_subst args))
       in
       let component_imports =
-	  List.fold_right
-	    (fun (v,comp,args) accu ->
-	       let from_args = substitute_arguments_types_only 
-				 eq_types self#from_subst args
-	       in let to_args = substitute_arguments_types_only 
-				  eq_types self#to_subst args
-	       in let rec do_v th_name accu = function
-		 | Pos -> (th_name, from_args @ to_args) :: accu
-		 | Neg -> (th_name, to_args @ from_args) :: accu
-		 | Mixed -> do_v th_name (do_v th_name accu Pos) Neg
-		 | _ -> assert(false); raise Internal_error
-	       in
-		 match comp#get_kind with
-		   | Spec_class -> do_v (ccsl_map_theory_name comp) accu v
-		   | Spec_adt -> 
-		       if comp#has_feature NeedsMapFeature
-		       then do_v (ccsl_adt_map_theory_name comp) accu v
-		       else accu
+	List.fold_right
+	  (fun (v,comp,args) accu ->
+	     let from_args = substitute_arguments_types_only 
+			       eq_types self#from_subst args
+	     in let to_args = substitute_arguments_types_only 
+				eq_types self#to_subst args
+	     in let rec do_v th_name accu = function
+	       | Pos -> (th_name, from_args @ to_args) :: accu
+	       | Neg -> (th_name, to_args @ from_args) :: accu
+	       | Mixed -> do_v th_name (do_v th_name accu Pos) Neg
+	       | _ -> assert(false)
+	     in
+	       match comp#get_kind with
+		 | Spec_class -> do_v (ccsl_map_theory_name comp) accu v
+		 | Spec_adt -> 
+		     if comp#has_feature NeedsMapFeature
+		     then do_v (ccsl_adt_map_theory_name comp) accu v
+		     else accu
 		 | Spec_sig -> accu
-		 | Spec_Spec -> (assert(false); raise Internal_error)
+		 | Spec_Spec -> assert(false)
 	    )
 	    cl#get_components []
       in
@@ -762,10 +787,10 @@ class ['class_type, 'member_type] ccsl_pre_map_struct_theory
 	  (ccsl_basic_theory_name cl, self_argument :: left) ::
 	  (ccsl_basic_theory_name cl, self_argument :: right) ::
 	  (List.map (function
-		       | Resolved_renaming(oanc, args,_,_) -> 
+		       | Resolved_renaming(oanc, args,_,_,_) -> 
 			   do_ancestor oanc args
 						(* no other stuff here *)
-		       | _ -> (assert(false); raise Internal_error)
+		       | _ -> assert(false)
 		    ) 
 	     cl#get_ancestors)
 	  @
@@ -807,7 +832,7 @@ class ['class_type, 'member_type] ccsl_pre_map_struct_theory
 		   | (None, _) as x -> x
 		   | (Some(neg_name, Function(dom,codom)), pos_id) ->
 		       (Some(neg_name, Function(codom, dom)), pos_id)
-		   | _ -> (assert(false); raise Internal_error)
+		   | _ -> assert(false)
 		) 
 		pre_fun_ids_var
 	    in
@@ -852,7 +877,7 @@ class ['class_type, 'member_type] ccsl_pre_map_struct_theory
       let ancestor_funs =
 	List.map 
 	  (function
-	     | Resolved_renaming(oanc, args,_,_) -> 
+	     | Resolved_renaming(oanc, args,_,_,_) -> 
 		 let arg_funs =
 		   functor_action_argument_list
 		     sub_ns				(* name_space *)
@@ -885,7 +910,7 @@ class ['class_type, 'member_type] ccsl_pre_map_struct_theory
 		 in
 		     super_label oanc, term
 						(* no other stuff here *)
-	     | _ -> (assert(false); raise Internal_error)
+	     | _ -> assert(false)
 	  ) 
 	  cl#get_ancestors
       in
@@ -1145,9 +1170,5 @@ class ccsl_map_theory cl =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
-
-
-
-
-
