@@ -18,11 +18,11 @@
  *
  * Created 14.5.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Monday 20 May 02 12:32:33 tews@ithif56.inf.tu-dresden.de>
  *
  * Typechecking with unification
  *
- * $Id: newtypecheck.ml,v 1.14 2002/01/24 14:44:00 tews Exp $
+ * $Id: newtypecheck.ml,v 1.16 2002/05/22 13:42:42 tews Exp $
  *
  *
  * the algorithm is homegrown and works in two passes
@@ -101,7 +101,7 @@ let undefined_method cl token loc = begin
 end;;
 
 
-let is_not_bool loc typ constrainttyp =
+let types_dont_match loc typ constrainttyp =
   error_message loc
     ("This expression has type " ^
      (string_of_ccsl_type typ) ^
@@ -162,8 +162,8 @@ let if_mismatch if_or_case loc type1 type2 =
      if_or_case ^ 
      " have different types. Type\n" ^
      (string_of_ccsl_type type1) ^
-     " is not compatible with type " ^
-     (string_of_ccsl_type type2) ^ "."
+     "\nis not compatible with type\n" ^
+     (string_of_ccsl_type type2)
     )
 
 let case_mismatch loc typex adttype =
@@ -171,7 +171,7 @@ let case_mismatch loc typex adttype =
     ("This case expression matches values of " ^
      (match adttype with
 	| Adt(adt,_,_) -> adt#get_name
-	| _ -> (assert(false); raise Internal_error)
+	| _ -> assert(false)
      ) ^
      ". Type\n" ^
      (string_of_ccsl_type typex) ^
@@ -320,6 +320,7 @@ let rec unify tv_bindings t1 t2 =
 	     BoundTypeVariable id1
 	   else
 	     raise Unify
+							  (* we are in unify *)
        | FreeTypeVariable t1, FreeTypeVariable t2
 	   when Type_variable.eq t1 t2 
 	     ->
@@ -338,6 +339,7 @@ let rec unify tv_bindings t1 t2 =
 	     Product( List.map2 recurse type_list1 type_list2 )
 	   else
 	     raise Unify
+							  (* we are in unify *)
        | Class(cl1, args1),Class(cl2, args2) ->
 	   if cl1#get_name = cl2#get_name 
 	   then
@@ -356,6 +358,7 @@ let rec unify tv_bindings t1 t2 =
 	     end
 	   else
 	     raise Unify
+							  (* we are in unify *)
 
 					(* new bindings *)
        | FreeTypeVariable tv1, FreeTypeVariable tv2 ->
@@ -379,6 +382,7 @@ let rec unify tv_bindings t1 t2 =
 			)
 	   )		  
 
+							  (* we are in unify *)
        | FreeTypeVariable tv, t ->
 	   (try
 	      let binding = find_tv tv_bindings tv
@@ -407,6 +411,19 @@ let rec unify tv_bindings t1 t2 =
 		     | Occur_check -> raise Unify
 		  )
 	   )
+							  (* we are in unify *)
+       | Groundtype(id1,args1), t2 when is_type_def id1 ->
+	   let subst = 
+	     make_substitution (get_ground_type_parameters id1) args1
+	   in
+	     recurse (ccsl_substitute_types subst id1.id_type) t2
+
+       | t1, Groundtype(id2,args2) when is_type_def id2 ->
+	   let subst = 
+	     make_substitution (get_ground_type_parameters id2) args2
+	   in
+	     recurse t1 (ccsl_substitute_types subst id2.id_type)
+
 	   
 					(* not allowed cases *)
        | SmartFunction _,_
@@ -421,7 +438,7 @@ let rec unify tv_bindings t1 t2 =
        | _, Predtype _
        | _, Record _
        | _, IFace _
-       | _, Array _          -> (assert(false); raise Internal_error)
+       | _, Array _          -> assert(false)
 	   
 	   
 				(* cases where the constructors mismatch *)
@@ -434,7 +451,7 @@ let rec unify tv_bindings t1 t2 =
        | Product _ ,_
        | Class _ ,_
        | Adt _ ,_
-	   
+							  (* we are in unify *)
        | _, Groundtype _
        | _, BoundTypeVariable _
        | _, Self
@@ -472,6 +489,16 @@ let unify_types tv_bindings t1 t2 error_msg_fun =
 	  raise Typecheck_error
 
 
+let types_must_unify t1 t2 =
+  let tv_bindings = new_tv_binding () 
+  in
+    try
+      ignore(unify tv_bindings t1 t2);
+      !tv_bindings = []
+    with
+      | Unify -> false
+
+
 (***********************************************************************
  ***********************************************************************
  *
@@ -554,6 +581,7 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
     match expr with
       | ExprLoc(ex,l) -> 
 	  ExprLoc(check_expression ccl l tv_bindings expr_typ ex, l)
+					       (* we are in check_expression *)
       | TypeAnnotation(BasicExpr(Member(instiface, 
 					Resolved(m))), typ) ->
 	  let ntyp = 
@@ -562,72 +590,83 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 	  in
 	  let ninstiface = match instiface with
 	    | CurrentIface -> CurrentIface
-	    | InstIface(iface, args) -> 
+	    | InstIface(iface, args, instloc) -> 
 		let nargs = check_ground_arglist loc tv_bindings 
 			      iface args
 			      ("hosting class of " ^
 			       m#get_name)
-		in
-		let _ = iter_component_arglist Pos
-			  (fun c -> ccl#add_component c) nargs
-			  iface#get_parameters
-		in let _ = ccl#add_component (Pos,iface,nargs) in
-		let _ = 
-		    if m#get_sort = Class_Coreduce 
+		in begin
+			(* iterate components unless ccl is a signature 
+			 * and m is a ground term from it
+			 *)
+		    if ccl#is_sig && m#hosting_class#get_name = ccl#get_name
 		    then
-		      let self1,self2 = match ntyp with
-			| Function(_,Function(self1,self2)) -> self1,self2
-			| _ -> assert(false); raise Internal_error
-		      in let _ = assert(iface#is_class) 
-		      in
-			ccl#add_iface_import
-			  (Names.ccsl_finality_theory_name iface,
-			   TypeArgument(self1) :: 
-			     TypeArgument(self2) :: 
-			     (normalize_args tv_bindings nargs));
-		in
-		  InstIface(iface,nargs)
+		      ()
+		    else begin
+		      iter_component_arglist_add_component ccl
+			nargs (get_member_parameters m);
+		      ccl#add_component (Pos,iface,nargs) 
+		    end;
+				(* add a special import if m is coreduce *)
+		    (if m#get_sort = Class_Coreduce 
+		     then
+		       let self1,self2 = match ntyp with
+			 | Function(_,Function(self1,self2)) -> self1,self2
+			 | _ -> assert(false)
+		       in let _ = assert(iface#is_class) 
+		       in
+			 ccl#add_iface_import
+			   (Names.ccsl_finality_theory_name iface,
+			    TypeArgument(self1) :: 
+			      TypeArgument(self2) :: 
+			      (normalize_args tv_bindings nargs))
+		    );
+		    InstIface(iface,nargs,instloc)
+		  end
 	    | NoIface -> 
-		assert(false); raise Internal_error
+		assert(false)
 	  in 
 	  let _ = dc ("derive " ^
-		      (string_of_ccsl_inst_iface instiface) ^
+		      (string_of_ccsl_inst_iface ninstiface) ^
 		      " for member " ^
-		      m#get_name)
-	  in let _ = assert(eq_ccsl_types expr_typ ntyp) 
+		      m#get_name) 
+	  in
+	  let _ = assert(types_must_unify expr_typ ntyp) 
 	  in
 	    TypeAnnotation(
 	      BasicExpr(Member(ninstiface, Resolved(m))),
 	      expr_typ)
 	      
+					       (* we are in check_expression *)
       | BasicExpr(TermVar(Resolved(id_rec))) -> 
 	  (* if the type of id_rec was not determined completely, 
 	   * then this is checked in Let or CCSL_Case
 	   *)
-	  let _ = assert(eq_ccsl_types expr_typ id_rec.id_type)
+	  let _ = assert(types_must_unify expr_typ id_rec.id_type)
 	  in
 	    expr
 				(* other BasicExpr cannot appear here *)
       | BasicExpr(TermVar(Unresolved _))
       | BasicExpr(Member _) -> 
-	  (assert(false); raise Internal_error)
+	  assert(false)
 
+					       (* we are in check_expression *)
       | TypedTerm(Term _, typ) -> 
 	  begin
-	    assert(eq_ccsl_types expr_typ typ);
+	    assert(types_must_unify expr_typ typ);
 	    expr
 	  end
 
       | TypedTerm(ex, typ) -> 
 	  begin
-	    assert(eq_ccsl_types expr_typ typ);
+	    assert(types_must_unify expr_typ typ);
 	    TypedTerm( recurse_exp expr_typ ex, typ)
 	  end
 	  
 			(* ex_typ is normalized, see derive_expression *)
       | MethodSelection(TypeAnnotation(ex,ex_typ), typeopt, m) ->
 	  let _ = 
-	    iter_components Pos (fun c -> ccl#add_component c) ex_typ
+	    iter_components_add_component ccl ex_typ
 	  in 
 	    MethodSelection(
 	      TypeAnnotation(recurse_exp ex_typ ex, ex_typ), 
@@ -635,24 +674,26 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 
 				(* MethodSelection needs TypeAnnotation *)
       | MethodSelection _ ->
-	  (assert(false); raise Internal_error)
+	  assert(false)
 
+					       (* we are in check_expression *)
       | Tuple(ex_list) -> 
-	  let tl = match expr_typ with
+	  let tl = match expand_type_def eq_ccsl_types expr_typ with
 	    | Product tl -> tl
-	    | _ -> (assert(false); raise Internal_error)
+	    | _ -> assert(false)
 	  in
 	    Tuple(List.map2 recurse_exp tl ex_list)
 
       | Projection(i,_) -> 
-	  let domtl, codom = match expr_typ with
-	    | Function(Product tl, codom) -> tl, codom
-	    | Function(domtyp,_) -> 
-		begin
-		  proj_dom_error loc domtyp;
-		  raise Typecheck_error
-		end
-	    | _ -> (assert(false);raise Internal_error)
+	  let domtl, codom = 
+	    match expand_type_def eq_ccsl_types expr_typ with
+	      | Function(Product tl, codom) -> tl, codom
+	      | Function(domtyp,_) -> 
+		  begin
+		    proj_dom_error loc domtyp;
+		    raise Typecheck_error
+		  end
+	      | _ -> assert(false)
 	  in
 	  let tuple_len = List.length domtl in
 	  let _ = if i > tuple_len then
@@ -670,10 +711,11 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 	  in
 	    TypeAnnotation(Projection(i,tuple_len), expr_typ)
 
+					       (* we are in check_expression *)
       | Abstraction(decl_list,ex) ->
-	  let codom = match expr_typ with
+	  let codom = match expand_type_def eq_ccsl_types expr_typ with
 	    | Function(_, codom) -> codom
-	    | _ -> (assert(false);raise Internal_error)
+	    | _ -> assert(false)
 	  in 
 	    Abstraction(decl_list, recurse_exp codom ex)
 
@@ -683,10 +725,10 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 	  in
 	  let _ = dc ("derive " ^ (string_of_ccsl_type nex1_typ) ^
 		      " for fun " ^ (string_of_ccsl_expression ex1))
-	  in let dom,codom = match nex1_typ with
+	  in let dom,codom = match expand_type_def eq_ccsl_types nex1_typ with
 	    | Function(dom,codom) -> dom,codom
-	    | _ -> (assert(false);raise Internal_error)
-	  in let _ = assert(eq_ccsl_types codom expr_typ)
+	    | _ -> assert(false)
+	  in let _ = assert(types_must_unify codom expr_typ)
 	  in
 	    Application( 
 	      TypeAnnotation(recurse_exp nex1_typ ex1, nex1_typ),
@@ -694,26 +736,37 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 
 					(* Application needs TypeAnnotation *)
       | Application _ ->
-	  (assert(false); raise Internal_error)
+	  assert(false)
 
-      | TypeAnnotation(InfixApplication(ex1,instiface, (Resolved m), ex2), 
+					       (* we are in check_expression *)
+      | TypeAnnotation(InfixApplication(ex1, instiface, (Resolved m), ex2), 
 		       memtype) ->
 	  let ninstiface = match instiface with
-	    | InstIface(iface, args) -> 
+	    | InstIface(iface, args, instloc) -> 
 		let nargs = check_ground_arglist loc tv_bindings 
 			      iface args
 			      ("infix operator " ^
 			       m#get_name)
 		in 
 		  begin
-		    iter_component_arglist Pos (fun c -> ccl#add_component c)
-		      nargs iface#get_parameters;
-		    ccl#add_component (Pos, iface,nargs);
-		    InstIface(iface,nargs)
+			(* iterate components unless ccl is a signature 
+			 * and m is a ground term from it
+			 *)
+		    if ccl#is_sig && m#hosting_class#get_name = ccl#get_name
+		    then
+		      ()
+		    else begin
+		      iter_component_arglist_add_component ccl
+			nargs (get_member_parameters m);
+		      ccl#add_component (Pos,iface,nargs) 
+		    end;
+		    InstIface(iface,nargs,instloc)
 		  end
-	    | CurrentIface 
+				(* may happen in a top level definition *)
+	    | CurrentIface -> CurrentIface
+
 	    | NoIface -> 
-		assert(false); raise Internal_error
+		assert(false)
 	  in 
 	  let nmemtype = check_ground loc tv_bindings memtype 
 			   "infix operator"
@@ -722,12 +775,12 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 		      " for infix " ^ m#get_name )
 	  in
 	  let lefttype, righttype, resulttype =
-	    (match nmemtype with
+	    (match expand_type_def eq_ccsl_types nmemtype with
 	       | Function(Product [t1;t2], t3) -> t1, t2, t3
 	       | Function(t1, Function(t2,t3)) -> t1, t2, t3
-	       | _ -> (assert(false);raise Internal_error)
+	       | _ -> assert(false)
 	    )
-	  in let _ = assert(eq_ccsl_types resulttype expr_typ)
+	  in let _ = assert(types_must_unify resulttype expr_typ)
 	  in
 	    TypeAnnotation(
 	      InfixApplication(
@@ -735,14 +788,17 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 		ninstiface, Resolved m,
 		recurse_exp righttype ex2),
 	      nmemtype)
+					       (* we are in check_expression *)
+
 				(* InfixApplication needs TypeAnnotation *)
       | InfixApplication _ ->
-	  (assert(false); raise Internal_error)
+	  assert(false)
 
       | FunUpdate(fex, changes) -> 
-	  let dom_type, codom_type = match expr_typ with
-	    | Function(dom,codom) -> dom,codom
-	    | _ -> (assert(false);raise Internal_error)
+	  let dom_type, codom_type = 
+	    match expand_type_def eq_ccsl_types expr_typ with
+	      | Function(dom,codom) -> dom,codom
+	      | _ -> assert(false)
 	  in 
 	    FunUpdate(
 	      recurse_exp expr_typ fex,
@@ -770,6 +826,7 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 	    Let(ndecl_list,
 		recurse_exp expr_typ ex)
 
+					       (* we are in check_expression *)
       | If(conds,else_ex) -> 
 	  let nconds = 
 	    List.map 
@@ -781,6 +838,7 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 	    If(nconds, 
 	       recurse_exp expr_typ else_ex)
 
+					       (* we are in check_expression *)
       | CCSL_Case(TypeAnnotation(ex,ex_typ), variants) -> 
 	  let adt_type = check_ground (get_ex_loc ex) tv_bindings
 			   ex_typ "expression"
@@ -797,8 +855,7 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 					    id_rec.id_type "INTERNAL ERROR"
 					with
 					  | Typecheck_error -> 
-					      (assert(false); 
-					       raise Internal_error)
+					      assert(false)
 				       )
 			 in 
 			   id_rec.id_type <- nid_typ
@@ -814,13 +871,14 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
 	      TypeAnnotation(nex, adt_type),
 	      nvariants)
 
+					       (* we are in check_expression *)
 
 					(* CCSL_Case needs a TypeAnnotation *)
       | CCSL_Case _ -> 
-	  (assert(false); raise Internal_error)
+	  assert(false)
 
       | Box(typ,pred,tlist) ->
-	  let _ = assert( eq_ccsl_types expr_typ (Function(typ,Bool)) )
+	  let _ = assert( types_must_unify expr_typ (Function(typ,Bool)) )
 	  in
 	    Box(typ,
 		recurse_exp expr_typ pred,
@@ -828,7 +886,7 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
     
 
       | Diamond(typ,pred,tlist) ->
-	  let _ = assert( eq_ccsl_types expr_typ (Function(typ,Bool)) )
+	  let _ = assert( types_must_unify expr_typ (Function(typ,Bool)) )
 	  in
 	    Diamond(typ,
 		    recurse_exp expr_typ pred,
@@ -837,6 +895,8 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
       | Expression form ->
 	  Expression(recurse_form expr_typ form)
 
+					       (* we are in check_expression *)
+
 			(* type annotations are already catched above *)
       | TypeAnnotation _
 					(* not allowed in ccsl_input_types *)
@@ -857,7 +917,7 @@ let rec check_expression ccl loc tv_bindings expr_typ expr =
       | Comment_str _
       | Comment_expr _
       | Comprehension _ ->
-	  (assert(false); raise Internal_error)
+	  assert(false)
 	  in 
 	    nexpression
 
@@ -870,37 +930,38 @@ and check_formula ccl loc tv_bindings formula_type formula =
     match formula with
       | FormLoc(f,l)    -> 
 	  FormLoc(check_formula ccl l tv_bindings formula_type f, l)
-      | True 		-> assert(eq_ccsl_types formula_type Bool); formula
-      | False 		-> assert(eq_ccsl_types formula_type Bool); formula
+      | True 		-> assert(types_must_unify formula_type Bool); formula
+      | False 		-> assert(types_must_unify formula_type Bool); formula
       | Not f 		-> 
 	  begin
-	    assert(eq_ccsl_types formula_type Bool);	    
+	    assert(types_must_unify formula_type Bool);	    
 	    Not(recurse_form formula_type f)
 	  end
       | And f_list 	-> 
 	  begin
-	    assert(eq_ccsl_types formula_type Bool);	    
+	    assert(types_must_unify formula_type Bool);	    
 	    And(List.map (recurse_form formula_type) f_list)
 	  end
       | Or f_list 	-> 
 	  begin
-	    assert(eq_ccsl_types formula_type Bool);	    
+	    assert(types_must_unify formula_type Bool);	    
 	    Or(List.map (recurse_form formula_type) f_list)
 	  end
+						  (* we are in check_formula *)
       | Implies(assum,concl) -> 
 	  begin
-	    assert(eq_ccsl_types formula_type Bool);	    
+	    assert(types_must_unify formula_type Bool);	    
 	    Implies( recurse_form formula_type assum, 
 		     recurse_form formula_type concl)
 	  end
       | Iff(assum,concl) -> 
 	  begin
-	    assert(eq_ccsl_types formula_type Bool);	    
+	    assert(types_must_unify formula_type Bool);	    
 	    Iff( recurse_form formula_type assum, 
 		 recurse_form formula_type concl)
 	  end
       | Equal( TypeAnnotation(ex_a,eq_typ), ex_b) -> 
-	  let _ = assert(eq_ccsl_types formula_type Bool) in
+	  let _ = assert(types_must_unify formula_type Bool) in
 	  let neq_typ = check_ground loc tv_bindings eq_typ 
 			  "both sides of equality"
 	  in let _ = dc ("derive " ^
@@ -910,24 +971,27 @@ and check_formula ccl loc tv_bindings formula_type formula =
 	    Equal( recurse_exp neq_typ ex_a, recurse_exp neq_typ ex_b)
 
 					(* Equal needs TypeAnnotation *)
-      | Equal _ -> (assert(false); raise Internal_error)
+      | Equal _ -> assert(false)
 
       | Forall(quant_list, f) -> 
 	  begin
-	    assert(eq_ccsl_types formula_type Bool);	    
+	    assert(types_must_unify formula_type Bool);	    
 	    Forall(quant_list, recurse_form formula_type f)
 	  end
+						  (* we are in check_formula *)
       | Exists(quant_list, f) -> 
 	  begin
-	    assert(eq_ccsl_types formula_type Bool);	    
+	    assert(types_must_unify formula_type Bool);	    
 	    Exists(quant_list, recurse_form formula_type f)
 	  end
       | Formula ex -> 
 	  Formula(recurse_exp formula_type ex)
 
+						  (* we are in check_formula *)
+
 					(* t equals None *)
       | Obseq(t, TypeAnnotation(ex_a,eq_typ), ex_b) ->		
-	  let _ = assert(eq_ccsl_types formula_type Bool) in
+	  let _ = assert(types_must_unify formula_type Bool) in
 	  let neq_typ = check_ground loc tv_bindings eq_typ 
 			  "both sides of equality" 
 	  in let _ = dc ("derive " ^
@@ -936,10 +1000,16 @@ and check_formula ccl loc tv_bindings formula_type formula =
 					(* check for greatest bisim *)
 	  in let violation = ref false in
 	  let _ = iter_components Pos 
-		    (function (_,ifa,args) -> 
-		       if (not (ifa#has_feature HasRelLiftFeature)) &&
-			 (count_self_args args <> 0)
-		       then violation := true
+		    (fun _ typ -> 
+		       let args = match typ with
+			 | Class(_,args) -> args
+			 | Adt(_,_,args) -> args
+			 | Groundtype(_,args) -> args
+			 | _ -> assert(false)
+		       in
+			 if (not (type_has_feature HasRelLiftFeature typ)) &&
+			   (count_self_args args <> 0)
+			 then violation := true
 		    ) neq_typ
 	  in let _ = (if (count_self neq_typ) <> 0 && 
 			(not (ccl#has_feature HasGreatestBisimFeature))
@@ -960,6 +1030,7 @@ and check_formula ccl loc tv_bindings formula_type formula =
 		    raise Typecheck_error
 		  end
 
+						  (* we are in check_formula *)
 	  in let obsname =
 	      (match neq_typ with
 					(* Self gets Bisim later *)
@@ -981,8 +1052,10 @@ and check_formula ccl loc tv_bindings formula_type formula =
 		   recurse_exp neq_typ ex_a, 
 		   recurse_exp neq_typ ex_b)
 
+						  (* we are in check_formula *)
+
 					(* Obseq needs TypeAnnotation *)
-      | Obseq _ -> (assert(false); raise Internal_error)
+      | Obseq _ -> assert(false)
 
 					(* not in ccsl_input_formulas *)
       | MetaImplies _
@@ -991,7 +1064,7 @@ and check_formula ccl loc tv_bindings formula_type formula =
       | ConstantPredicate _
 (* 	 | LessOrEqual _
  *)
-	-> (assert(false); raise Internal_error)
+	-> assert(false)
   in 
     nformula
     
@@ -1003,10 +1076,10 @@ let check_assertion_formula ccl loc tv_bindings formula =
     nformula
 
 
-let check_definition_formula ccl loc tv_bindings typ formula =
-  let nformula = check_formula ccl loc tv_bindings typ formula 
+let check_definition_expression ccl loc tv_bindings typ expr =
+  let nex = check_expression ccl loc tv_bindings typ expr
   in
-    nformula
+    nex
 
 
 
@@ -1036,7 +1109,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 				(* other BasicExpr cannot appear here *)
       | BasicExpr(TermVar(Unresolved _))
       | BasicExpr(Member _) -> 
-	  (assert(false); raise Internal_error)
+	  assert(false)
 
       | TypedTerm(Term _, typ) -> 
 	  let _ = dr "typed ground term" 
@@ -1051,11 +1124,12 @@ let rec derive_expression ccl loc tv_bindings expr =
 	    (TypedTerm(nex, utyp_ex),
 	     utyp_ex)
 
+					      (* we are in derive_expression *)
       | MethodSelection(ex, instifaceopt, m) ->
 	  let _ = dr "MethodSelection" in
 	  let nex, typ_ex = recurse_exp ex in
 	  let utyp = match instifaceopt with
-	    | InstIface (iface,args) -> 
+	    | InstIface (iface,args,instloc) -> 
 		assert(iface#is_class);
 		do_unify typ_ex (Class(iface,args)) (wrong_object_type loc)
 	    | CurrentIface ->
@@ -1065,7 +1139,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 	  let nutyp = normalize_type tv_bindings utyp in
 	  let mtoken = match m with
 	    | Unresolved t -> t
-	    | Resolved _ -> (assert(false); raise Internal_error) in
+	    | Resolved _ -> assert(false) in
 	  let _ = match nutyp with
 	    | Self			
 	    | Class _			(* all ok *)
@@ -1080,6 +1154,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 		    );
 		  raise Typecheck_error
 		end
+					      (* we are in derive_expression *)
 	  in let _ = 
 	      if type_is_nonground nutyp 
 	      then
@@ -1096,7 +1171,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 	  in let cl = match nutyp with
 	    | Self -> ccl
 	    | Class(cl,_) -> cl 
-	    | _ -> (assert(false);raise Internal_error)
+	    | _ -> assert(false)
 	  in let m = 
 	    try cl#find_member mtoken.token_name
 	    with | Member_not_found -> undefined_method cl mtoken 
@@ -1104,11 +1179,12 @@ let rec derive_expression ccl loc tv_bindings expr =
 	  let _ = (if not m#is_action 
 		   then undefined_method cl mtoken
 		     (remove_option mtoken.loc)) in
+					      (* we are in derive_expression *)
 	  let typ = match nutyp with
 	    | Self -> m#get_curried_type
 	    | Class(cl,args) -> 
-		snd( export_member_with_args cl args m#get_curried_type )
-	    | _ -> (assert(false);raise Internal_error)
+		snd( export_member_with_args cl m args m#get_curried_type )
+	    | _ -> assert(false)
 	  in let _ = dr ("Resolve " ^ m#get_name ^ ": " ^
 			 m#hosting_class#get_name)
 	  in
@@ -1128,6 +1204,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 		 * this branch is taken when the projection appears 
 		 * in an application 
 		 *)
+					      (* we are in derive_expression *)
       | Application(ExprLoc(Projection(i,_), ploc) as proj_ex, ex2) ->
 	  let _ = dr "ProjAppl" in
 	  let nex2, typ_ex2 = recurse_exp ex2 in
@@ -1162,6 +1239,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 		   end
 	    )
 
+					      (* we are in derive_expression *)
       | Projection(i,_) -> 
 	  let _ = dr "Projection" in
 	  let typ = 
@@ -1181,6 +1259,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 	  in
 	    (Abstraction(decl_list, nex),
 	     Function(dom_typ, typ_ex))
+					      (* we are in derive_expression *)
       | Application(ex1,ex2) -> 
 	  let _ = dr "Application" in
 	  let nex1, typ_ex1 = recurse_exp ex1 in
@@ -1193,13 +1272,14 @@ let rec derive_expression ccl loc tv_bindings expr =
 			      (get_ex_loc ex1))
 	  in let dom_type, codom_type = match ntyp_ex1 with
 	    | Function(dt,ct) -> dt,ct
-	    | _ -> (assert(false);raise Internal_error)
+	    | _ -> assert(false)
 	  in let ntyp_ex2 = do_unify dom_type typ_ex2 (dom_mismatch loc)
 	  in
 	    (Application(
 	       TypeAnnotation(nex1, ntyp_ex1),
 	       nex2),
 	     codom_type)
+					      (* we are in derive_expression *)
       | TypeAnnotation(InfixApplication(ex1,instiface,mem,ex2), memtype) ->
 	  let _ = dr "Infix" in
 	  let nex1, typ_ex1 = recurse_exp ex1 in
@@ -1208,7 +1288,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 	    (match memtype with
 	       | Function(Product [t1;t2], t3) -> t1, t2, t3
 	       | Function(t1, Function(t2,t3)) -> t1, t2, t3
-	       | _ -> (assert(false);raise Internal_error)
+	       | _ -> assert(false)
 	    )
 	  in let nlefttype = do_unify typ_ex1 lefttype 
 			       (mismatch (get_ex_loc ex1))
@@ -1221,8 +1301,9 @@ let rec derive_expression ccl loc tv_bindings expr =
 	     resulttype)
 
 			(* InfixApplication is always under TypeAnnotation *)
-      | InfixApplication _ -> (assert(false);raise Internal_error)
+      | InfixApplication _ -> assert(false)
 
+					      (* we are in derive_expression *)
       | FunUpdate(fex, changes) -> 
 	  let _ = dr "FunUpdate" in
 	  let nfex, typ_f = recurse_exp fex in
@@ -1234,7 +1315,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 			    (get_ex_loc fex))
 	  in let dom_type, codom_type = match ntyp_f with
 	    | Function(dt,ct) -> dt,ct
-	    | _ -> (assert(false);raise Internal_error)
+	    | _ -> assert(false)
 	  in let nchanges = 
 	      List.map (fun (ex1,ex2) ->
 			  let nex1,typ_ex1 = recurse_exp ex1 in
@@ -1248,6 +1329,8 @@ let rec derive_expression ccl loc tv_bindings expr =
 	  in
 	    (FunUpdate( nfex, nchanges), 
 	     ntyp_f)
+
+					      (* we are in derive_expression *)
       | Let(decl_list, ex) ->
 	  let _ = dr "Let" in
 					(* Let binds sequentially *)
@@ -1264,6 +1347,8 @@ let rec derive_expression ccl loc tv_bindings expr =
 	  in
 	    (Let(ndecl_list, nex),
 	     typ_ex)
+
+					      (* we are in derive_expression *)
       | If(conds,else_ex) -> 
 	  let _ = dr "If" in
 	  let res_type = FreeTypeVariable(Type_variable.fresh()) in
@@ -1273,7 +1358,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 		 let ncond,typ_cond = recurse_form cond in
 		 let nex, typ_ex = recurse_exp ex in
 		 let ntyp_cond = do_unify typ_cond Bool 
-				   (is_not_bool (get_form_loc ncond)) in
+				   (types_dont_match (get_form_loc ncond)) in
 		 let ntyp_ex = do_unify typ_ex res_type
 				 (if_mismatch "if" (get_ex_loc nex))
 		 in
@@ -1285,6 +1370,8 @@ let rec derive_expression ccl loc tv_bindings expr =
 			       (if_mismatch "if" (get_ex_loc nelse_ex)) 
 	  in
 	    (If(nconds, nelse_ex), ntyp_else_ex)
+
+					      (* we are in derive_expression *)
       | CCSL_Case(TypeAnnotation(ex,ex_typ), variants) -> 
 	  let _ = dr "CCSL_Case" in
 	  let nex, nex_typ = recurse_exp ex in
@@ -1300,21 +1387,21 @@ let rec derive_expression ccl loc tv_bindings expr =
 					(if_mismatch "case" (get_ex_loc cex))
 			    in
 			      (const, args, ncex)
-			| _ -> (assert(false); raise Internal_error)
+			| _ -> assert(false)
 		     ) variants
 	  in
 	    (CCSL_Case(TypeAnnotation(nex,nnex_typ), nvariants),
 	     res_type)
 
 					(* CCSL_Case needs a TypeAnnotation *)
-      | CCSL_Case _ -> 
-	  (assert(false), raise Internal_error)
+      | CCSL_Case _ -> assert(false)
 
+					      (* we are in derive_expression *)
       | Box(typ,pred,tlist) ->
 	  let _ = dr "Box" in
 	  let npred, typ_pred = recurse_exp pred in
 	  let ntyp_pred = do_unify typ_pred (Function(typ,Bool)) 
-			    (is_not_bool (get_ex_loc npred))
+			    (types_dont_match (get_ex_loc npred))
 	  in
 	    (Box(typ,npred,tlist), ntyp_pred)
 
@@ -1322,7 +1409,7 @@ let rec derive_expression ccl loc tv_bindings expr =
 	  let _ = dr "Diamond" in
 	  let npred, typ_pred = recurse_exp pred in
 	  let ntyp_pred = do_unify (Function(typ,Bool)) typ_pred 
-			    (is_not_bool (get_ex_loc npred))
+			    (types_dont_match (get_ex_loc npred))
 	  in
 	    (Diamond(typ,npred,tlist), ntyp_pred)
 
@@ -1332,6 +1419,8 @@ let rec derive_expression ccl loc tv_bindings expr =
 	  in
 	    (Expression(nform), typ_form) 
 
+					      (* we are in derive_expression *)
+
 			(* type annotations are already catched above *)
       | TypeAnnotation _
 					(* not allowed in ccsl_input_types *)
@@ -1352,7 +1441,7 @@ let rec derive_expression ccl loc tv_bindings expr =
       | Comment_str _
       | Comment_expr _
       | Comprehension _ ->
-	  (assert(false); raise Internal_error)
+	  assert(false)
   in 
   let nntype = (* normalize_type tv_bindings *) ntype 
   in let _ = 
@@ -1380,15 +1469,17 @@ and derive_formula ccl loc tv_bindings formula =
       | Not f -> 
 	  let _ = dr "Not" in
 	  let nf,typ_f = recurse_form f in
-	  let utyp_f = do_unify typ_f Bool (is_not_bool (get_form_loc nf))
+	  let utyp_f = do_unify typ_f Bool (types_dont_match (get_form_loc nf))
 	  in
 	    (Not nf, Bool)
+
+						 (* we are in derive_formula *)
       | And f_list -> 
 	  let _ = dr "And" in
 	  let nftype_list = List.map recurse_form f_list in
 	  let utyp_f_list = 
 	    List.map 
-	      (fun (f,t) -> do_unify t Bool (is_not_bool (get_form_loc f))) 
+	      (fun (f,t) -> do_unify t Bool (types_dont_match (get_form_loc f))) 
 	      nftype_list 
 	  in
 	  let nf_list, typ_f_list = List.split nftype_list 
@@ -1399,21 +1490,22 @@ and derive_formula ccl loc tv_bindings formula =
 	  let nftype_list = List.map recurse_form f_list in
 	  let utyp_f_list = 
 	    List.map 
-	      (fun (f,t) -> do_unify t Bool (is_not_bool (get_form_loc f))) 
+	      (fun (f,t) -> do_unify t Bool (types_dont_match (get_form_loc f))) 
 	      nftype_list 
 	  in
 	  let nf_list, typ_f_list = List.split nftype_list 
 	  in
 	    (Or nf_list, Bool)
 
+						 (* we are in derive_formula *)
       | Implies(assum,concl) -> 
 	  let _ = dr "Implies" in
 	  let nassum, typ_assum = recurse_form assum in
 	  let nconcl, typ_concl = recurse_form concl in
 	  let utyp_assum = do_unify typ_assum Bool
-			     (is_not_bool (get_form_loc assum)) in
+			     (types_dont_match (get_form_loc assum)) in
 	  let utyp_concl = do_unify typ_concl Bool 
-			     (is_not_bool (get_form_loc concl))
+			     (types_dont_match (get_form_loc concl))
 	  in
 	    (Implies( nassum, nconcl), Bool)
       | Iff(assum,concl) -> 
@@ -1421,9 +1513,9 @@ and derive_formula ccl loc tv_bindings formula =
 	  let nassum, typ_assum = recurse_form assum in
 	  let nconcl, typ_concl = recurse_form concl in
 	  let utyp_assum = do_unify typ_assum Bool 
-			     (is_not_bool (get_form_loc assum)) in
+			     (types_dont_match (get_form_loc assum)) in
 	  let utyp_concl = do_unify typ_concl Bool 
-			     (is_not_bool (get_form_loc concl))
+			     (types_dont_match (get_form_loc concl))
 	  in
 	    (Iff(nassum, nconcl), Bool)
       | Equal(ex_a,ex_b) -> 
@@ -1435,16 +1527,17 @@ and derive_formula ccl loc tv_bindings formula =
 	    (Equal(
 	       TypeAnnotation(nex_a, utyp_ab), nex_b),
 	     Bool)
+						 (* we are in derive_formula *)
       | Forall(quant_list, f) ->
 	  let _ = dr "Forall" in
 	  let nf,typ_f = recurse_form f in
-	  let utyp_f = do_unify typ_f Bool (is_not_bool (get_form_loc f))
+	  let utyp_f = do_unify typ_f Bool (types_dont_match (get_form_loc f))
 	  in
 	    (Forall( quant_list, nf), Bool)
       | Exists(quant_list, f) -> 
 	  let _ = dr "Exists" in
 	  let nf,typ_f = recurse_form f in
-	  let utyp_f = do_unify typ_f Bool (is_not_bool (get_form_loc f))
+	  let utyp_f = do_unify typ_f Bool (types_dont_match (get_form_loc f))
 	  in
 	    (Exists( quant_list, nf), Bool)
       | Formula ex -> 
@@ -1452,6 +1545,7 @@ and derive_formula ccl loc tv_bindings formula =
 	  let nex, typ_ex = recurse_exp ex
 	  in
 	    (Formula nex, typ_ex)
+						 (* we are in derive_formula *)
       | Obseq(t,ex_a,ex_b) ->		(* t equals None *)
 	  let _ = dr "Obseq" in
 	  let nex_a, typ_a = recurse_exp ex_a in
@@ -1467,7 +1561,7 @@ and derive_formula ccl loc tv_bindings formula =
       | ConstantPredicate _
 (* 	 | LessOrEqual _
  *)
-	-> (assert(false); raise Internal_error)
+	-> assert(false)
   in 
   let nntype = (* normalize_type tv_bindings *) ntype 
   in let _ = 
@@ -1481,16 +1575,17 @@ and derive_formula ccl loc tv_bindings formula =
 
 let derive_assertion_formula ccl loc tv_bindings formula =
   let nformula, typ_f = derive_formula ccl loc tv_bindings formula in
-  let utyp_f = unify_types tv_bindings typ_f Bool (is_not_bool loc)
+  let utyp_f = unify_types tv_bindings typ_f Bool (types_dont_match loc)
   in
     nformula
 
 
-let derive_definition_formula ccl loc tv_bindings typ formula =
-  let nformula, typ_f = derive_formula ccl loc tv_bindings formula in
-  let utyp_f = unify_types tv_bindings typ_f typ (is_not_bool loc)
+let derive_definition_expression ccl loc tv_bindings typ expr =
+  let nex, typ_ex = derive_expression ccl loc tv_bindings expr in
+  let utyp_ex = unify_types tv_bindings typ_ex typ (types_dont_match loc) in
+  let _ = assert(types_must_unify utyp_ex typ)
   in
-    nformula
+    nex
 
 
 (***********************************************************************
@@ -1529,9 +1624,17 @@ let rec check_valid_definition_expr loc expr =
       | CCSL_Case(ex,variants) -> 
 	  recurse_exp ex;
 	  List.iter (fun (mem,ids,exp) -> recurse_exp exp) variants
-      | Box _
-      | Diamond _ -> 
-	  invalid_definition loc "Modal operator"
+      | Box(typ, pred, tlist) ->
+	  (match typ with
+	     | Self -> invalid_definition loc "Modal operator"
+	     | _ -> recurse_exp pred
+	  )
+
+      | Diamond(typ, pred, tlist) -> 
+	  (match typ with
+	     | Self -> invalid_definition loc "Modal operator"
+	     | _ -> recurse_exp pred
+	  )
 
       | Expression form -> recurse_form form
       | TypeAnnotation(exp,typ) -> recurse_exp exp
@@ -1551,7 +1654,7 @@ let rec check_valid_definition_expr loc expr =
       | Comment_str _
       | Comment_expr _
       | Comprehension _ ->
-	  (assert(false); raise Internal_error)
+	  assert(false)
 	
 and check_valid_definition_form loc formula =
   let recurse_exp = check_valid_definition_expr loc in
@@ -1572,9 +1675,8 @@ and check_valid_definition_form loc formula =
       | Equal(ex_a,ex_b) ->
 	  recurse_exp ex_a; 
 	  recurse_exp ex_b
-      | Forall _
-      | Exists _ -> 
-	  invalid_definition loc "Quantification"
+      | Forall(decll, f) -> recurse_form f
+      | Exists(decll, f) -> recurse_form f
       | Formula ex -> recurse_exp ex
       | Obseq(t,ex1,ex2) -> 
 	  invalid_definition loc "Observational equality"
@@ -1583,7 +1685,7 @@ and check_valid_definition_form loc formula =
       | MetaForall _
       | Bisim _
       | ConstantPredicate _
-	-> (assert(false); raise Internal_error)
+	-> assert(false)
 
 
 
@@ -1601,22 +1703,23 @@ and check_valid_definition_form loc formula =
 
 let typecheck_definition ccl def =
   match def.definition with 
-    | Symbolic(FormLoc(defform,loc)) ->
+    | Symbolic(FormLoc(Formula(TypeAnnotation(defex, righttyp)), loc)) ->
 	let mem = def.defined_method in
 	let _ = d ("  * TC definition " ^ mem#get_name) in
+	let tv_bindings = new_tv_binding () in
+	let ex1 = derive_definition_expression ccl loc tv_bindings 
+		    righttyp defex in
+	let ex2 = check_definition_expression ccl loc tv_bindings 
+		    righttyp ex1
+	in
 	let _ = (if not !expert_mode 
-		 then check_valid_definition_form loc defform
+		 then check_valid_definition_expr loc ex2
 		 else ())
 	in
-	let tv_bindings = new_tv_binding () in
-	let form1 = derive_definition_formula ccl loc tv_bindings 
-		      mem#get_codomain defform in
-	let form2 = check_definition_formula ccl loc tv_bindings 
-		      mem#get_codomain form1
-	in
-	  def.definition <- Symbolic(FormLoc(form2,loc))
+	  def.definition <- 
+	  Symbolic(FormLoc(Formula(TypeAnnotation(ex2, righttyp)), loc))
 	      
-    | Symbolic _ -> (assert(false); raise Internal_error)
+    | Symbolic _ -> assert false
 	  
     | Pvs_String _
     | Isa_String _ -> ()
@@ -1635,7 +1738,7 @@ let typecheck_assertion ccl assertion =
 	  in
 	    assertion.assertion_formula <- Symbolic(FormLoc(form2,loc))
 	      
-      | Symbolic _ -> (assert(false); raise Internal_error)
+      | Symbolic _ -> assert(false)
 	  
       | Pvs_String _
       | Isa_String _ -> ()
@@ -1645,14 +1748,18 @@ let typecheck_class ccl =
   d (" ** TC " ^ ccl#get_name);
   List.iter (typecheck_definition ccl) ccl#get_definitions;
   List.iter (typecheck_assertion ccl) ccl#get_assertions;
-  List.iter (typecheck_assertion ccl) ccl#get_creations
+  List.iter (typecheck_assertion ccl) ccl#get_creations;
+  List.iter (typecheck_assertion ccl) ccl#get_theorems
+
+let typecheck_sig si =
+  d (" ** TC " ^ si#get_name);
+  List.iter (typecheck_definition si) si#get_definitions
 
 
 let typecheck_ast = function
   | CCSL_class_dec cl -> typecheck_class cl
   | CCSL_adt_dec adt -> ()
-  | CCSL_sig_dec si -> ()
-  | CCSL_TypeAbbrev_dec _ -> ()
+  | CCSL_sig_dec si -> typecheck_sig si
 
 
 let ccsl_typecheck_class_pass (ast: Classtypes.ccsl_ast list) = 
@@ -1664,5 +1771,6 @@ let ccsl_typecheck_class_pass (ast: Classtypes.ccsl_ast list) =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
