@@ -18,11 +18,11 @@
  *
  * Created 1.10.97 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Sunday 19 May 02 21:09:07 tews@ithif56.inf.tu-dresden.de>
  *
  * Utility functions for ccsl parser
  *
- * $Id: parser.ml,v 1.22 2002/01/24 14:44:01 tews Exp $
+ * $Id: parser.ml,v 1.24 2002/05/22 13:42:42 tews Exp $
  *
  *)
 
@@ -68,6 +68,7 @@ type parsing_type =
   | Parse_Class of  ccsl_iface_type 
   | Parse_Adt of ccsl_iface_type
   | Parse_Sig of ccsl_iface_type
+  | Parse_Anon of ccsl_iface_type
 
 let current_iface = ref (Parse_Nothing : parsing_type )
 
@@ -77,8 +78,9 @@ let ciface () = match !current_iface with
   | Parse_Class( cl ) -> cl
   | Parse_Adt( adt ) -> adt
   | Parse_Sig si -> si
+  | Parse_Anon an -> an
   | Parse_Nothing -> 
-      begin assert(false); raise Internal_error end
+      begin assert(false) end
 
  (* documentation -> mli *)
 let ccl () = match !current_iface with
@@ -86,8 +88,9 @@ let ccl () = match !current_iface with
   | Parse_IFace _
   | Parse_Adt _
   | Parse_Sig _
+  | Parse_Anon _
   | Parse_Nothing -> 
-      begin assert(false); raise Internal_error end
+      begin assert(false) end
 
  (* documentation -> mli *)
 let cadt () = match !current_iface with
@@ -95,17 +98,20 @@ let cadt () = match !current_iface with
   | Parse_IFace _
   | Parse_Class _ 
   | Parse_Sig _
+  | Parse_Anon _
   | Parse_Nothing ->
-      begin assert(false); raise Internal_error end
+      begin assert(false) end
 
  (* documentation -> mli *)
 let csig () = match !current_iface with
   | Parse_Sig( si ) -> si
+  | Parse_Anon an -> an
   | Parse_IFace _
   | Parse_Class _ 
   | Parse_Adt _
   | Parse_Nothing ->
-      begin assert(false); raise Internal_error end
+      begin assert(false) end
+
 
  (* documentation -> mli *)
 let current_visibility = ref (Public : ccsl_visibility)
@@ -114,7 +120,7 @@ let cvis() = match !current_visibility with
   | Public -> Public
   | Private -> Private
   | NoVisibility ->			(* called in wrong context *)
-	 assert(false); raise Internal_error
+	 assert(false)
 
 
 let ground_type_nat loc =
@@ -128,14 +134,13 @@ let ground_type_nat loc =
     try 
       match find (get_nat_type()) with
 	| CCSL_GroundTypeSymb(id) ->
- 	    (if check_parameters id.id_parameters []
+ 	    (if check_parameters (get_ground_type_parameters id) []
 	     then Groundtype(id,[])
 	     else nat_error()
 	    )
 	    
 	| ClassSymb _ 
 	| AdtSymb _ 
-	| CCSL_TypeAbbrevSymb _
 	| TypevarSymb _
 	| VarSymb _
 	| SigSymb _
@@ -143,7 +148,7 @@ let ground_type_nat loc =
 	    nat_error()
 					(* this is an internal error *)
 	| InstClassSymb _
-	| InstAdtSymb _ -> (assert(false);raise Internal_error)
+	| InstAdtSymb _ -> assert(false)
     with
 	Table.Not_defined -> nat_error()
 
@@ -165,7 +170,7 @@ let new_loc firstloc lastloc =
 
     (*****************************************************************
      * 
-     * anonymous ground signatures
+     * anonymous ground signatures, Part I
      *
      *)
 
@@ -173,12 +178,22 @@ let anon_ground_sig_count = ref 0
 
 let name_anon_ground_sig () =
   incr anon_ground_sig_count;
-  "_AnonSig_" ^ (string_of_int !anon_ground_sig_count)
+  Names.anon_sig_prefix ^ (string_of_int !anon_ground_sig_count) ^ "_"
 
-let anon_ground_sig = ref (None : ccsl_iface_type option)
-
-let reset_anon_ground_sig () = 
-  anon_ground_sig := None
+let close_anon_ground_sig () =
+  match !current_iface with
+    | Parse_Nothing -> ()
+    | Parse_Anon ansig ->
+	begin
+	  ignore(Symbol.close_block());
+	  Symbol.create_sig_content ansig;
+	  current_iface := Parse_Nothing
+	end
+    | Parse_IFace _
+    | Parse_Class _ 
+    | Parse_Adt _
+    | Parse_Sig _
+      -> assert(false)
 
 
 (**********************************************************************
@@ -218,10 +233,10 @@ let qualified_symbol_error_message token iface_name = begin
   raise Parsing.Parse_error
 end;;
 
-let instantiation_error token parameters actuals =
+let instantiation_error loc name parameters actuals =
   begin
-    error_message (remove_option token.loc)
-      (token.token_name ^ " takes " ^
+    error_message loc
+      (name ^ " takes " ^
        (string_of_int (List.length parameters)) ^
        " type parameter(s); " ^
        "but you supplied " ^
@@ -231,6 +246,12 @@ let instantiation_error token parameters actuals =
        )
        ^ "."
       );
+  end
+
+let instantiation_error_token token parameters actuals =
+  begin
+    instantiation_error (remove_option token.loc) 
+      token.token_name parameters actuals;
     raise Parsing.Parse_error
   end
 
@@ -253,6 +274,14 @@ let wrong_id_type token expect found =
     raise Parsing.Parse_error
   end
 
+let double_type_param token =
+  begin
+    error_message (remove_option token.loc)
+      ("Typeparameter " ^ token.token_name ^
+       " already defined\n"
+      );
+    raise Parsing.Parse_error
+  end
 
 (**********************************************************************
  **********************************************************************
@@ -271,12 +300,13 @@ let wrong_id_type token expect found =
  *)
 
 let add_type_parameter variance token =		
-  (* Hendrik: check for double parameters *)
   let var = Symbol.identifier_record token CCSL_TypeParameter in
   let _ = var.id_variance <- variance in
     match !current_iface with
-      | Parse_Nothing ->		(* in a groundtype decl *)
-	  (TypeParameter var)
+      | Parse_Nothing			(* in a groundtype decl *)
+      | Parse_Anon _ 
+      | Parse_Sig _
+	-> (TypeParameter var)
       | Parse_IFace _ ->			(* in a class/adt decl *)
 	  begin
 					(* no overloading for types *)
@@ -287,10 +317,23 @@ let add_type_parameter variance token =
 
       | Parse_Class _ 
       | Parse_Adt _ 
-      | Parse_Sig _ ->
-	  (assert(false); raise Internal_error)
+	  -> assert(false)
 
 
+let unique_type_parameter_check parameters =
+  ignore(
+    List.fold_left 
+	   (fun present_list ->
+	      function TypeParameter id -> 
+		if List.mem id.id_token.token_name present_list
+		then
+		  double_type_param id.id_token;
+		id.id_token.token_name :: present_list
+	   )
+	   [] parameters
+  )
+	   
+
 let do_exact_variance loc vn vp =
   if (* ((vn = -1) && (vp = -1)) or *)
     ((vn <> -1) && (vn mod 2 = 0)) or
@@ -315,15 +358,6 @@ let add_adt adt =
   Some( CCSL_adt_dec adt )
 
 
-let add_typeabbrev token typ =		
-  let var = Symbol.identifier_record token CCSL_TypeAbbrev in
-  let _ = var.id_type <- typ 
-  in begin
-      reset_anon_ground_sig();
-      Symbol.create_type_abbrev var;
-      Some( CCSL_TypeAbbrev_dec var )
-    end
-
 
 let add_sig si =			
   List.iter Symbol.create_ground_type (si#get_all_ground_types);
@@ -351,9 +385,9 @@ let add_member_to_iface m = (ciface())#add_member m
 
 
 
-   (* action for BEGIN ID *)
-let do_iface_start token =
-  let _ = reset_anon_ground_sig() in
+   (* setting up an iface *)
+let do_all_iface_start token =
+  let _ = close_anon_ground_sig() in
 					(* check for uniqueness *)
   let _ = 
     try match (find token.token_name) with
@@ -367,7 +401,6 @@ let do_iface_start token =
 	      raise Parsing.Parse_error
 	    end)
       | CCSL_GroundTypeSymb _
-      | CCSL_TypeAbbrevSymb _ -> ()
 
 		(* these things have not yet added to the symboltable *)
       | TypevarSymb _	
@@ -376,7 +409,7 @@ let do_iface_start token =
       | InstClassSymb _
       | InstAdtSymb _
       | MemberSymb _
-	-> (assert(false);raise Internal_error)
+	-> assert(false)
     with Table.Not_defined -> ()
   in let scope = Symbol.start_block () in
   let ifa = new ccsl_pre_iface_class token scope !parsing_prelude
@@ -386,6 +419,83 @@ let do_iface_start token =
     current_iface := Parse_IFace(ifa)
 
 
+    (********************
+     * anonymous ground signatures, Part II
+     *)
+
+let do_local_type_params parameterlist =
+         (* block for local type parameters *)
+  ignore(Symbol.start_block ());
+         (* add them to the symboltable *)
+  List.iter 
+    (function (TypeParameter sym as tp) ->
+       Symbol.create_type_parameter sym
+    )
+    parameterlist
+
+
+    (* start an anon ground sig if
+     * - we are not in a regular ground sig and 
+     * - there is no anon ground sig
+     * 
+     * val start_anon_ground_sig : 
+     *   token_type -> ccsl_parameter_type list -> ccsl_ast option
+     *)
+let start_anon_ground_sig token parameterlist =
+  match !current_iface with
+    | Parse_Nothing ->			(* outside everything *)
+	let gsig_token = 
+	  { token_name = name_anon_ground_sig();
+	    loc = token.loc
+	  } 
+	in	
+	          (* close the current anon ground sig, if there is one  *)
+	let _ = do_all_iface_start gsig_token in
+	let gsig = ciface() 
+	in
+	  begin
+	    gsig#become_sig;
+	    current_iface := Parse_Anon gsig;
+	    do_local_type_params parameterlist;
+	    (* return *)
+	    Some (CCSL_sig_dec gsig)
+	  end
+			(* have an anon ground sig without type parameters 
+			 * => keep it
+			 *)
+    | Parse_Anon gsig -> 
+	do_local_type_params parameterlist;
+	None
+			(* inside an groundsig => just add type parameters *)
+    | Parse_Sig _ -> 
+	do_local_type_params parameterlist;
+	None
+
+					(* everything else is wrong *)
+    | Parse_Class _
+    | Parse_IFace _
+    | Parse_Adt _
+	-> assert false
+
+   (* action for BEGIN ID *)
+   (* anonymous ground signatures use the entry point 
+    * do_all_iface_start
+    *)
+let do_iface_start token =
+					(* check for clash with anon sig *)
+  let anon = Names.anon_sig_prefix in
+  let _ = 
+    if String.length token.token_name >= String.length anon then
+      if String.sub token.token_name 0 (String.length anon) = anon then
+	begin
+	  error_message (remove_option token.loc)
+	    ("Identifiers starting with \"" ^ anon ^ "\"\n" ^
+	     "are reserved for anonymous ground signatures.");
+	      raise Parsing.Parse_error
+	end
+  in
+    do_all_iface_start token
+
   (**********************
    * add importings to the class;
    * Note: the import list comes reversed !
@@ -460,73 +570,52 @@ let do_sig_end startloc imports end_id =
     si
 
 
-let add_groundtype token parameterlist =
-  let gsig,new_gsig =
-    match !anon_ground_sig, parameterlist with
-      | None, _
-      | Some _, _::_  -> 
-	  let gsig_token = 
-	    { token_name = name_anon_ground_sig();
-	      loc = token.loc
-	    } in
-	  let _ = do_iface_start gsig_token in
-	  let gsig = ciface() 
-	  in
-	    List.iter 
-	      (function (TypeParameter sym as tp) ->
-		 Symbol.create_type_parameter sym;
-		 gsig# add_parameter tp
-	      )
-	      parameterlist;
-	    gsig#become_sig;
-	    current_iface := Parse_Nothing;
-	    ignore(Symbol.close_block ());
-	    anon_ground_sig := Some gsig;
-	    gsig,true
-      | Some gsig, [] -> gsig,false
-  in
-  let idrec = Symbol.identifier_record token (CCSL_GroundType gsig)
-  in begin
-      idrec.id_parameters <- parameterlist;
-      gsig#add_ground_type idrec;
-      Symbol.create_ground_type idrec;
-      if parameterlist <> [] then
-	reset_anon_ground_sig();
-      if new_gsig 
-      then Some( CCSL_sig_dec gsig )
-      else None
-    end
-	
+(* let start_groundtype token parameterlist =
+ *   let gsig,new_gsig =
+ *     match !anon_ground_sig, parameterlist with
+ * 	 | None, _
+ * 	 | Some _, _::_  -> 
+ * 	     ((start_anon_ground_sig token.loc parameterlist),
+ * 	      true)
+ * 	 | Some gsig, [] -> gsig, false
+ *   in
+ *   let idrec = Symbol.identifier_record token (CCSL_GroundTypeDecl gsig)
+ *   in begin
+ * 	 ((if new_gsig 
+ * 	   then Some (CCSL_sig_dec gsig)
+ * 	   else None),
+ * 	  idrec)
+ *     end
+ *)
 
-let add_groundterm_intern membersort token typ  =
-  let gsig,new_gsig =
-    match !anon_ground_sig with
-      | None ->
-	  let gsig_token = 
-	    { token_name = name_anon_ground_sig();
-	      loc = token.loc
-	    } in
-	  let _ = do_iface_start gsig_token in
-	  let gsig = ciface() 
-	  in
-	    gsig#become_sig;
-	    current_iface := Parse_Nothing;
-	    ignore(Symbol.close_block ());
-	    anon_ground_sig := Some gsig;
-	    gsig,true
-      | Some gsig -> gsig,false
+
+    (* add a groundtype definition or declaration to the current signature
+     * (which might be anonymous)
+     * 
+     * val add_groundtype : 
+     *   token_type -> ccsl_parameter_type list -> 
+     * 	 ccsl_input_types option -> unit
+     *)
+let add_groundtype token parameters typeopt =
+			(* close local type parameters *)
+  let _ = Symbol.close_block () in
+  let gsig = csig() in
+  let _ = unique_type_parameter_check (gsig#get_parameters @ parameters) in
+  let idrec = Symbol.identifier_record token (CCSL_GroundTypeDecl gsig) in
+  let _ = idrec.id_parameters <- parameters
   in
-  let dom = Product [] in
-  let m = new ccsl_pre_member_class gsig token [] dom typ NoVisibility
-    membersort
-  in begin
-      gsig#add_member m;
-      if new_gsig 
-      then Some( CCSL_sig_dec gsig )
-      else None
+    begin
+      (match typeopt with
+	 | None -> ()
+	 | Some typ -> 
+	     begin
+	       idrec.id_origin <- CCSL_GroundTypeDef gsig;
+	       idrec.id_type <- typ;
+	     end
+      );
+      gsig#add_ground_type idrec
     end
 
-let add_groundterm = add_groundterm_intern GroundTerm
 
 let infix_test = function
   | Function( Product( [_;_] ), _ ) -> true
@@ -534,16 +623,103 @@ let infix_test = function
   | _ -> false
 
 
-let add_infix_groundterm infixtoken typ = 
-  let _ = if not (infix_test typ) then
-    begin error_message (remove_option infixtoken.loc)
-	("Infix constants must be declared as functions " ^
-	 "taking two arguments.");
-      raise Parsing.Parse_error
-    end 
+(* Binary operators in PVS 2.4
+ * 
+ * BinOp ::= 'o' | 'IFF' | '<=>' | 'IMPLIES' | '=>' | 'WHEN' | 'OR'
+ * 	   | '\/' | 'AND' | '/\' | '&' | 'XOR' | 'ANDTHEN' | 'ORELSE'
+ * 	   | '^' | '+' | '-' | '*' | '/' | '++' | '~' | '**' | '//' | '^^'
+ * 	   | '|-' | '|=' | '<|' | '|>' | '=' | '/=' | '==' | '<' | '<='
+ * 	   | '>' | '>=' | '<<' | '>>' | '<<=' | '>>=' | '#' | '@@' | '##'
+ * 
+ * These work in CCSL
+ *
+ *   # ## & * ** + ++ - / // /= /\ < << 
+ *   <<= <= <=> <| == => > >= >> >>= @@ \/ ^ ^^ |- |= |> 
+ *
+ * excluding = and ~
+ *
+ *)
+
+let valid_infix_string infixtoken =
+  if !output_mode = Pvs_mode 
+  then match infixtoken.token_name with
+					(* let these pass *)
+    | "#" | "##" | "&" | "*" | "**" | "+" | "++" | "-"
+    | "/" | "//" | "/=" | "/\\" | "<" | "<<" | "<<=" 
+    | "<=" | "<=>" | "<|" | "==" | "=>" | ">" | ">=" | ">>"
+    | ">>=" | "@@" | "\\/" | "^" | "^^" | "|-" | "|=" 
+    | "|>" 
+      -> ()
+					(* barf on others *)
+    | _ -> 
+	begin error_message (remove_option infixtoken.loc)
+	    ("This infix operator is not allowed in PVS");
+	  raise Parsing.Parse_error
+	end 
+  else
+(* HENDRIK : add checks for Isabelle *)
+    ()
+
+    (* check for infix ground terms
+     * 
+     * val check_infix : 
+     * 	 token_type -> ccsl_input_types -> unit
+     *)
+let check_infix infixtoken typ = 
+  begin
+    if not (infix_test typ) then
+      begin error_message (remove_option infixtoken.loc)
+	  ("Infix constants must be declared as functions " ^
+	   "taking two arguments.");
+	raise Parsing.Parse_error
+      end;
+    valid_infix_string infixtoken;
+  end
+
+
+    (* add a constant symbol to a ground signature
+     *
+     * val add_groundterm : 
+     * 	 token_type -> ccsl_member_sort -> ccsl_parameter_type list ->
+     * 	     ccsl_input_types -> ccsl_assertion option -> unit
+     *)
+
+let add_groundterm token membersort parameters typ formulaoption = 
+			(* close local type parameters *)
+  let _ = Symbol.close_block () in
+  let _ = match membersort with
+    | GroundTerm -> ()
+    | InfixGroundTerm -> check_infix token typ
+    | _ -> assert false
   in
-    add_groundterm_intern InfixGroundTerm infixtoken typ 
-    
+  let gsig = csig() in
+  let _ = unique_type_parameter_check (gsig#get_parameters @ parameters) in
+  let dom = Product [] in
+  let _ = 
+    try 
+      let second = gsig#find_local_member token.token_name in
+      overloading_error (ciface()) token second
+    with Member_not_found -> () 
+  in
+  let m = new ccsl_pre_member_class gsig token [] dom typ NoVisibility
+    membersort
+  in begin
+      m#set_local_parameters parameters;
+      add_member_to_iface m;
+      (match formulaoption with
+	 | None -> ()
+	 | Some f -> (let def =
+			{ defined_method = m;
+			  variables = [];
+			  definition = f;
+			  def_sequence = -1;
+			}
+		      in
+			gsig#add_definition def
+		     )
+      );
+    end
+
 
 (**********************************************************************
  **********************************************************************
@@ -554,13 +730,13 @@ let add_infix_groundterm infixtoken typ =
 
 
 (* val do_inherit : token_type -> ccsl_argument_type list -> 
- *                       ccsl_renaming list -> ccsl_ancestor_type
+ *   (ccsl_renaming list * location_type option) -> ccsl_ancestor_type
  *
  * tries to find the symbol in the symboltable, 
  * if not produce an errormessage and an error token
 *)
 
-let do_inherit token al rename =	
+let do_inherit token al (rename, endloc) =	
   let s = 
     try find token.token_name
     with Table.Not_defined -> symbol_error_message token in 
@@ -569,8 +745,6 @@ let do_inherit token al rename =
     | ClassSymb(cl) -> cl
 					(* a ground type is not ok *)
     | CCSL_GroundTypeSymb _
-					(* a type abbreviation *)
-    | CCSL_TypeAbbrevSymb _
 					(* neither a type variable *)
     | TypevarSymb(_)
 					(* neither variable *)
@@ -585,12 +759,18 @@ let do_inherit token al rename =
 					(* cant happen *)
     | InstClassSymb _
     | InstAdtSymb _
-    | MemberSymb(_) -> (assert(false);raise Internal_error)
+    | MemberSymb(_) -> assert(false)
 
-  in if cl#check_parameters al then
-      Unresolved_renaming(cl, al, rename)
-    else 
-      instantiation_error token cl#get_parameters al
+  in
+  let _ = if not (check_parameters cl#get_parameters al) then
+    instantiation_error_token token cl#get_parameters al
+  in let inherit_loc = (match endloc with
+			  | None -> remove_option token.loc
+			  | Some endl -> 
+			      new_loc (remove_option token.loc) endl
+		       )
+  in
+    Unresolved_renaming(cl, al, rename, inherit_loc)
 
 
     (* test if this is a valid type for a class constructor *)
@@ -745,7 +925,8 @@ let do_definition token dom codom equation =
   in let def =
       { defined_method = m;
 	variables = [];
-	definition = equation
+	definition = equation;
+	def_sequence = -1;
       }
   in
     (ciface())#add_definition def
@@ -761,6 +942,7 @@ let do_id_declaration (id, typ) =
 	       id_parameters = [];
 	       id_origin = CCSL_Var;
 	       id_variance = Unset;
+	       id_sequence = -1;
 	     }
   in
     symb
@@ -771,6 +953,10 @@ let do_id_declaration (id, typ) =
    * Note: the import list comes reversed !
    *
    * val do_assertion_start : 
+   *     ccsl_importing_type list -> token_type * ccsl_input_types ->
+   *       (token_type * ccsl_input_types) list -> 
+   *         ccsl_identifier_record_type * ccsl_identifier_record_type list
+   *   
    *
    *)
 let do_assertion_start il selfvar varlist =
@@ -783,8 +969,10 @@ let do_assertion_start il selfvar varlist =
   (**********************
    * built an assertion and add it to the current class
    *
-   * val do_assertion : token_type -> ccsl_identifier_record_type list -> 
-   *      ccsl_formula -> unit
+   * val do_assertion : 
+   *     token_type -> 
+   *	    ccsl_identifier_record_type * ccsl_identifier_record_type list -> 
+   *	      ccsl_assertion -> unit
    *)
 let do_assertion token (selfvar,varlist) formula =
   let ass = { assertion_name = token;
@@ -807,7 +995,7 @@ let do_assertion token (selfvar,varlist) formula =
    * built an creation condition and add it to the current class
    *
    * val do_creation : token_type -> ccsl_identifier_record_type list -> 
-   *      ccsl_formula -> unit
+   *      ccsl_assertion -> unit
    *)
 let do_creation token varlist formula =	
   let ass = { assertion_name = token;
@@ -831,6 +1019,10 @@ let do_creation token varlist formula =
    * Note: the import list comes reversed !
    *
    * val do_creation_start : 
+   *   	 location_type -> 
+   *   	   ccsl_importing_type list -> 
+   *   	     (token_type * ccsl_input_types) list -> 
+   *   	       ccsl_identifier_record_type list
    *
    *)
 let do_creation_start crealoc il varlist =
@@ -857,6 +1049,48 @@ let do_selfvar_default () =
     Self)
 
 
+  (**********************
+   * add theorem importings to the class;
+   * Note: the import list comes reversed !
+   *
+   * val do_theorem_start : 
+   *   	   ccsl_importing_type list -> 
+   *   	     (token_type * ccsl_input_types) list -> 
+   *   	       ccsl_identifier_record_type list
+   *
+   *)
+let do_theorem_start il varlist =
+  begin
+    List.fold_right (fun i () -> (ccl())#add_theorem_import i) il ();
+    List.map do_id_declaration varlist
+  end
+
+
+  (**********************
+   * built an theorem and add it to the current class
+   *
+   * val do_theorem : token_type -> ccsl_identifier_record_type list -> 
+   *      ccsl_formula -> unit
+   *)
+let do_theorem token varlist formula =
+  let ass = { assertion_name = token;
+	      self_variable = None;
+	      free_variables = varlist;
+	      is_generated = false;
+	      assertion_formula = formula
+	    } in
+    if debug_level _DEBUG_PARSERCHECK then
+      (match formula with
+	 | Symbolic f -> print_verbose
+	     ("Ass " ^ token.token_name ^ " " ^
+	      string_of_ccsl_formula f)
+	 | _ -> ()
+      );
+    (ccl())#add_theorem ass
+
+
+
+
 (**********************************************************************
  **********************************************************************
  * 
@@ -1037,36 +1271,12 @@ let do_var_adt_constructor token accessors dom codom =
 
 let do_sig_type_decl token =
   let si = csig() in
-  let var = Symbol.identifier_record token (CCSL_GroundType si) in
-  let _ = var.id_parameters <- si#get_parameters
+  let var = Symbol.identifier_record token (CCSL_GroundTypeDecl si) in
+  let _ = var.id_parameters <- []
   in 
     si#add_ground_type var
     
 
-let do_sig_const_intern membersort token typ =
-  let dom = Product [] in
-  let _ = 
-    try 
-      let second = (csig())#find_local_member token.token_name in
-      overloading_error (ciface()) token second
-    with Member_not_found -> () in
-  let m = new ccsl_pre_member_class (ciface()) token [] dom typ NoVisibility
-    membersort
-  in
-    add_member_to_iface m
-
-let do_sig_const = do_sig_const_intern GroundTerm
-
-let do_sig_infix_const infixtoken typ =
-  let _ = if not (infix_test typ) then
-    begin error_message (remove_option infixtoken.loc)
-	("Infix constants must be declared as functions " ^
-	 "taking two arguments.");
-      raise Parsing.Parse_error
-    end 
-  in
-    do_sig_const_intern InfixGroundTerm infixtoken typ
-
 
     (*******************************************************************
      *******************************************************************
@@ -1085,24 +1295,26 @@ let do_argument_id token  : ccsl_argument_type =
     match s with
 					(* a basetype might be ok *)
       | CCSL_GroundTypeSymb id -> 
-	  (if check_parameters id.id_parameters [] 
+	  (if check_parameters (get_ground_type_parameters id) [] 
 	   then TypeArgument(Groundtype(id,[]))
-	   else instantiation_error token [] []
+	   else instantiation_error_token token [] []
 	  )
-					   (* a type abbreviation *)
-      | CCSL_TypeAbbrevSymb id -> TypeArgument(id.id_type)
 					(* a type variable is ok *)
       | TypevarSymb(id) -> TypeArgument(BoundTypeVariable(id))
 					(* a ccsl class might be ok*)
-      | ClassSymb(cl) -> (if cl#check_parameters []  then
-		       	TypeArgument(Class(cl, []))
-		      else
-		       	instantiation_error token cl#get_parameters [])
+      | ClassSymb(cl) -> (if (check_parameters cl#get_parameters [])  then
+		       	    TypeArgument(Class(cl, []))
+			  else
+		       	    instantiation_error_token token 
+			      cl#get_parameters []
+			 )
 					(* a ccsl adt might be ok*)
-      | AdtSymb(adt) -> (if adt#check_parameters []  then
-		       TypeArgument(Adt(adt, Always, []))
-		     else
-		       instantiation_error token adt#get_parameters [] )
+      | AdtSymb(adt) -> (if (check_parameters adt#get_parameters []) then
+			   TypeArgument(Adt(adt, Always, []))
+			 else
+			   instantiation_error_token token 
+			     adt#get_parameters [] 
+			)
 
 					(* a signature is wrong *)
       | SigSymb _ ->
@@ -1117,7 +1329,7 @@ let do_argument_id token  : ccsl_argument_type =
 	     else "adt constructor")
 	                    (* no instanciations are stored in CCSL_ID *)
       | InstClassSymb _ 
-      | InstAdtSymb _ -> (assert(false); raise Internal_error)
+      | InstAdtSymb _ -> assert(false)
 
 
 
@@ -1160,28 +1372,28 @@ let do_type_appl token arglist =
   try
     match find token.token_name with
 					(* a ccsl class is ok*)
-    | ClassSymb(cl) -> (if cl#check_parameters arglist then
+    | ClassSymb(cl) -> (if (check_parameters cl#get_parameters arglist) then
 			    Class(cl,arglist)
 		     	else
-			  instantiation_error 
+			  instantiation_error_token 
 			    token cl#get_parameters arglist
 		       )
 					(* an adt is ok*)
-    | AdtSymb(adt) -> (if adt#check_parameters arglist then
+    | AdtSymb(adt) -> (if (check_parameters adt#get_parameters arglist) then
 			 Adt(adt,Always,arglist)
 		       else
-			 instantiation_error 
+			 instantiation_error_token 
 			   token adt#get_parameters arglist
 		      )
 					(* a ground type might be ok *)
     | CCSL_GroundTypeSymb(id) ->
- 	(if check_parameters id.id_parameters arglist 
-	 then Groundtype(id,arglist)
-	 else instantiation_error token id.id_parameters arglist
-	)
-					(* neither a type abbreviation *)
-    | CCSL_TypeAbbrevSymb _
-					(* nor a type variable *)
+	let ground_params = get_ground_type_parameters id
+	in
+ 	  if check_parameters ground_params arglist 
+	  then Groundtype(id,arglist)
+	  else instantiation_error_token token ground_params arglist
+	  
+					(* the rest is not ok *)
     | TypevarSymb(_) 
     | VarSymb(_)
     | SigSymb _
@@ -1190,7 +1402,7 @@ let do_type_appl token arglist =
 
 					(* this is an internal error *)
     | InstClassSymb _
-    | InstAdtSymb _ -> (assert(false);raise Internal_error)
+    | InstAdtSymb _ -> assert(false)
   with
       Table.Not_defined -> symbol_error_message token;;
 
@@ -1207,8 +1419,6 @@ let do_instclass token arglist =
     | ClassSymb(cl) -> cl
 					(* a ground type is not ok *)
     | CCSL_GroundTypeSymb _
-					(* a type abbreviation *)
-    | CCSL_TypeAbbrevSymb _
 					(* neither a type variable *)
     | TypevarSymb(_)
 					(* neither variable *)
@@ -1223,17 +1433,17 @@ let do_instclass token arglist =
 					(* cant happen *)
     | InstClassSymb _
     | InstAdtSymb _
-    | MemberSymb(_) -> (assert(false);raise Internal_error)
+    | MemberSymb(_) -> assert(false)
 
-  in if cl#check_parameters arglist then
+  in if (check_parameters cl#get_parameters arglist) then
       Class(cl, arglist)
     else 
-      instantiation_error token cl#get_parameters arglist
+      instantiation_error_token token cl#get_parameters arglist
 
 
 let treat_q_id_info = function
-  | (None,None,id) -> ( NoIface, id )
-  | (Some ifacetoken, argsopt, id) ->
+  | (None,None,None,id) -> ( NoIface, id )
+  | (Some ifacetoken, argsopt, Some instloc, id) ->
       (try 
 	 let iface = 
 	   match find ifacetoken.token_name with (* Classes and ADTs are ok *)
@@ -1243,8 +1453,6 @@ let treat_q_id_info = function
 		 
 					(* a ground type is not ok *)
 	     | CCSL_GroundTypeSymb(_)
-					(* neither a type abbreviation *)
-	     | CCSL_TypeAbbrevSymb _
 					(* nor a type variable *)
 	     | TypevarSymb(_) 
 	     | VarSymb(_)
@@ -1253,20 +1461,22 @@ let treat_q_id_info = function
 	    
 	     (* this is an internal error *)
 	     | InstClassSymb _
-	     | InstAdtSymb _  -> (assert(false);raise Internal_error)
+	     | InstAdtSymb _  -> assert(false)
 	 in
 	 let arglist = match argsopt with 
 	   | None -> []
 	   | Some arglist -> arglist
+					(* check type parameters in client *)
+(* 	    in
+ * 	    let _ = if not (iface#check_parameters arglist) then
+ * 	      instantiation_error_token ifacetoken iface#get_parameters arglist 
+ *)
 	 in
-	 let _ = if not (iface#check_parameters arglist) then
-	   instantiation_error ifacetoken iface#get_parameters arglist 
-	 in
-	   (InstIface(iface, arglist),  id)
+	   (InstIface(iface, arglist, Some instloc), id)
        with
 	   Table.Not_defined -> symbol_error_message ifacetoken
       )
-  | _ -> (assert(false); raise Internal_error)
+  | _ -> assert(false)
 
 
 
@@ -1275,23 +1485,27 @@ let do_type_symbol token symbol =
     match symbol with
 					(* a basetype might be ok *)
       | CCSL_GroundTypeSymb(id) -> 
-	  if check_parameters id.id_parameters [] 
-	  then Groundtype(id, [])
-	  else instantiation_error token id.id_parameters []
-					(* a type abbreviation *)
-      | CCSL_TypeAbbrevSymb id -> id.id_type
+	  let ground_params = get_ground_type_parameters id
+	  in
+	    if check_parameters ground_params [] 
+	    then Groundtype(id, [])
+	    else instantiation_error_token token ground_params []
 					(* a type variable is ok *)
       | TypevarSymb(id) -> BoundTypeVariable(id)
 					(* a ccsl class might be ok*)
-      | ClassSymb(cl) -> (if cl#check_parameters [] then
+      | ClassSymb(cl) -> (if (check_parameters cl#get_parameters []) then
 			    Class(cl, [])
 			  else
-			    instantiation_error token cl#get_parameters [] )
+			    instantiation_error_token token 
+			      cl#get_parameters [] 
+			 )
 					(* a ccsl adt might be ok*)
-      | AdtSymb(adt) -> (if adt#check_parameters []  then
+      | AdtSymb(adt) -> (if (check_parameters adt#get_parameters []) then
 			   Adt(adt, Always, [])
 			 else
-			   instantiation_error token adt#get_parameters [] )
+			   instantiation_error_token token 
+			     adt#get_parameters [] 
+			)
 
 					(* a signature is wrong *)
       | SigSymb _ ->
@@ -1306,7 +1520,7 @@ let do_type_symbol token symbol =
 	     else "adt constructor")
 			    (* no Instanciations are stored in Classnames *)
       | InstClassSymb _
-      | InstAdtSymb _ -> (assert(false);raise Internal_error)
+      | InstAdtSymb _ -> assert(false)
 	       
 	    
 let do_qualified_type_id q_id_info = 
@@ -1319,36 +1533,41 @@ let do_qualified_type_id q_id_info =
 	in
 	  do_type_symbol id s
 
-    | InstIface(iface,arglist) -> 
+    | InstIface(iface,arglist,instloc) -> 
 	let _ = (if not iface#is_sig 
 		 then wrong_id_type id "type" ""
 		) 
-	in
-	  (try 
-	     match iface#find_local id.token_name with
+	in let gtypid = 
+	    (try 
+	       match iface#find_local id.token_name with
 					(* a ground type might be ok *)
-	       | CCSL_GroundTypeSymb(id) -> Groundtype(id,arglist)
+		 | CCSL_GroundTypeSymb(id) -> id
 
-	       | TypevarSymb(_) 
-	       | MemberSymb(_) ->
-		   wrong_id_type id "type" ""
+		 | TypevarSymb(_) 
+		 | MemberSymb(_) ->
+		     wrong_id_type id "type" ""
 					(* this is an internal error *)
-	       | ClassSymb(_) 
-	       | AdtSymb(_) 
-	       | CCSL_TypeAbbrevSymb _
-	       | VarSymb(_)
-	       | SigSymb _
+		 | ClassSymb(_) 
+		 | AdtSymb(_) 
+		 | VarSymb(_)
+		 | SigSymb _
 
-	       | InstClassSymb _
-	       | InstAdtSymb _ -> 
-		   (assert(false);raise Internal_error)
-	   with 
-	       Table.Not_defined -> 
-		 qualified_symbol_error_message id iface#get_name
-	  )
+		 | InstClassSymb _
+		 | InstAdtSymb _ -> 
+		     assert(false)
+	     with 
+		 Table.Not_defined -> 
+		   qualified_symbol_error_message id iface#get_name
+	    )
+	in let typeparams = get_ground_type_parameters gtypid
+	in let _ =
+	    if not (check_parameters typeparams arglist) then
+	      instantiation_error_token id typeparams arglist 
+	in
+	  Groundtype(gtypid, arglist)
 
     | CurrentIface -> 
-	(assert(false); raise Internal_error)
+	assert(false)
 
 
 
@@ -1364,26 +1583,33 @@ let do_qualified_type_id q_id_info =
 let do_method_selection sub_expr q_id_info =
   let instiface, id = treat_q_id_info q_id_info in
   let _ = match instiface with
-    | InstIface(iface,_) -> 
-	if not iface#is_class then
-	  let cltoken = match q_id_info with
-	    | Some id,_,_ -> id 
-	    | _ -> (assert(false); raise Internal_error)
-	  in
-	    wrong_id_type cltoken "class" id.token_name
+    | InstIface(iface,arglist,instloc) -> 
+	begin
+	  (if not iface#is_class then
+	     let cltoken = match q_id_info with
+	       | Some id,_,_,_ -> id 
+	       | _ -> assert(false)
+	     in
+	       wrong_id_type cltoken "class" id.token_name);
+	  (if not (check_parameters iface#get_parameters arglist) 
+	   then
+	     let cltoken = match q_id_info with
+	       | Some id,_,_,_ -> id 
+	       | _ -> assert(false)
+	     in
+	       instantiation_error_token cltoken iface#get_parameters arglist)
+	end
     | _ -> ()
   in
-    Formula(
-      MethodSelection( sub_expr, NoIface, 
-		       Unresolved id))
+    Formula(MethodSelection( sub_expr, instiface, Unresolved id))
 
 
 let do_qualified_term_id q_id_info = 
   let instiface, id = treat_q_id_info q_id_info 
   in match instiface with
     | NoIface -> BasicExpr(TermVar(Unresolved id))
-    | InstIface(iface,args) -> BasicExpr(Member( instiface, Unresolved id))
-    | CurrentIface -> (assert(false); raise Internal_error)
+    | InstIface _ -> BasicExpr(Member( instiface, Unresolved id))
+    | CurrentIface -> assert(false)
 
 
 
@@ -1415,6 +1641,7 @@ let do_case_pattern (mem,args) formula =
 		  id_origin = CCSL_Var;
 		  id_parameters = [];
 		  id_variance = Unset;
+		  id_sequence = -1;
 		 })
 	      args
   in
@@ -1431,11 +1658,11 @@ let do_case_pattern (mem,args) formula =
 let do_and f1 f2 = 
   let f1',f1loc = match f1 with
     | FormLoc( f, l ) -> f,l
-    | _ -> (assert(false); raise Internal_error)
+    | _ -> assert(false)
   in 
   let f2',f2loc = match f2 with
     | FormLoc( f, l ) -> f,l
-    | _ -> (assert(false); raise Internal_error)
+    | _ -> assert(false)
   in 
   let newloc = new_loc f1loc f2loc 
   in
@@ -1449,11 +1676,11 @@ let do_and f1 f2 =
 let do_or f1 f2 = 
   let f1',f1loc = match f1 with
     | FormLoc( f, l ) -> f,l
-    | _ -> (assert(false); raise Internal_error)
+    | _ -> assert(false)
   in 
   let f2',f2loc = match f2 with
     | FormLoc( f, l ) -> f,l
-    | _ -> (assert(false); raise Internal_error)
+    | _ -> assert(false)
   in 
   let newloc = new_loc f1loc f2loc 
   in
@@ -1468,5 +1695,6 @@ let do_or f1 f2 =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
