@@ -18,11 +18,11 @@
  *
  * Created 14.5.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Sunday 19 May 02 20:02:34 tews@ithif56.inf.tu-dresden.de>
  *
  * Resolution pass for CCSL
  *
- * $Id: resolution.ml,v 1.12 2002/01/23 16:00:18 tews Exp $
+ * $Id: resolution.ml,v 1.14 2002/05/22 13:42:43 tews Exp $
  *
  *)
 
@@ -134,7 +134,7 @@ let check_method_label cl token =
 
 
 let rec do_basic_expr ccl loc = function  
-  | TermVar(Resolved _ ) -> (assert false; raise Internal_error)
+  | TermVar(Resolved _ ) -> assert false
   | TermVar(Unresolved token) ->
       let symb = 
 	try Symbol.find token.token_name 
@@ -164,9 +164,13 @@ let rec do_basic_expr ccl loc = function
 		in let instiface, memtyp = 
 		    if ccl#get_name = topiface#get_name 
                     then
-                      (CurrentIface, mfulltype)
-                    else
-		      export_member topiface mfulltype
+		      if ccl#is_sig 
+		      then		(* export local typeparameters *)
+			export_member_local topiface m mfulltype
+		      else		(* no export *)
+			(CurrentIface, mfulltype)
+                    else		(* export everything *)
+		      export_member topiface m mfulltype
 		in let _ = dr ("member " ^ 
 			       (string_of_ccsl_inst_iface instiface)
 			      )
@@ -176,7 +180,6 @@ let rec do_basic_expr ccl loc = function
 		    memtyp)
 
 	    | CCSL_GroundTypeSymb _
-	    | CCSL_TypeAbbrevSymb _
 	    | TypevarSymb _
 	    | ClassSymb _ 
 	    | AdtSymb _ 
@@ -185,26 +188,31 @@ let rec do_basic_expr ccl loc = function
 
 	                    (* no instanciations are stored in CCSL_ID *)
 	    | InstClassSymb _ 
-	    | InstAdtSymb _ -> (assert(false); raise Internal_error)
+	    | InstAdtSymb _ -> assert(false)
 	 )
 
-  | Member(InstIface(iface,args), mem_container) -> 
+  | Member(InstIface(iface,args,Some instloc) as instiface, Unresolved id) -> 
 	(* iface must be different from ccl, because otherwise 
 	 * it could not have been resolved in the parser
 	 *)
-      let id = match mem_container with
-	| Unresolved id -> id
-	| _ -> (assert(false); raise Internal_error)
-      in let m = 
+      let m = 
 	try iface#find_member id.token_name
 	with Member_not_found -> undefined_member iface id 
 	    (remove_option id.loc)
-      in let _, memtype =
+      in 
+      let typeparams = get_member_parameters m in
+      let _ = (if not (check_parameters typeparams args) 
+	       then
+		 Parser.instantiation_error instloc 
+		   (iface#get_name ^ "::" ^ id.token_name)
+		   typeparams args
+	      ) in
+      let _, memtype =
 			(* make type variables unique, before exporting
 			 * this is only neccessary for (co)reduce, 
 			 * but does not harm in general
 			 *)
-	  export_member_with_args iface args 
+	  export_member_with_args iface m args 
 	    (uniquify_type_variables m#get_full_type)
       in let _ = dr ("quali member " ^ 
 		    m#get_name ^ " : " ^
@@ -212,20 +220,19 @@ let rec do_basic_expr ccl loc = function
 		   )
       in
 	TypeAnnotation(
-	  BasicExpr(Member(InstIface(iface,args), Resolved m)),
+	  BasicExpr(Member(instiface, Resolved m)),
 	  memtype)
 
 	  (* other Members are not generated in the parser *)
-  | Member( CurrentIface, _ ) 
-  | Member( NoIface, _ )
-    -> assert(false); raise Internal_error
+  | Member _
+    -> assert(false)
 
 	  (* do one branch of a CCSL_Case *)
 and do_variant ccl loc adt tvars (const, args, ex) =
   let recurse_exp = do_expression ccl loc in
   let token = match const with 
     | Unresolved t -> t
-    | _ -> (assert(false); raise Internal_error) in
+    | _ -> assert(false) in
   let member =
     try
       adt#find_local_member token.token_name
@@ -242,13 +249,13 @@ and do_variant ccl loc adt tvars (const, args, ex) =
 	  end
   in let _ = (if not member#is_adt_constructor then 
 		wrong_id_type token "adt constructor" "")
-  in let _, member_type = export_member_with_args adt 
+  in let _, member_type = export_member_with_args adt member
 			    tvars member#get_full_type
   in let arg_types = match member_type with
     | Function(Product tl, _ ) -> tl
     | Function(t , _) -> [t]
     | _ when member#get_sort = Adt_Const_Constructor -> []
-    | _ -> (assert(false); raise Internal_error) 
+    | _ -> assert(false) 
   in let _ = (if (List.length arg_types) <> (List.length args) 
 		 then begin
 		   error_message (remove_option token.loc)
@@ -324,6 +331,7 @@ and do_expression ccl loc expr =
 	  dr "Application";
 	  Application(recurse_exp ex1, recurse_exp ex2)
       | InfixApplication(ex1, NoIface, Unresolved token, ex2) ->
+	  let _ = dr "InfixApplication" in
 	  let nex1 = recurse_exp ex1 in
 	  let nex2 = recurse_exp ex2 in
 	  let symb = 
@@ -332,12 +340,23 @@ and do_expression ccl loc expr =
 	  in (match symb with
 					(* members are ok *)
 		| MemberSymb(m) -> 
+		    let _ = assert(not m#is_inherited) in
+					(* dont need trace down the heir *)
 		    let topiface = m#hosting_class in
 		    let _ = assert(m#get_sort = InfixGroundTerm) in
 		    let _ = assert(topiface#is_sig) in
-		    let _ = assert(ccl#get_name <> topiface#get_name) in
+			(* in the symboltable only special symbols like 
+			 * reduce/coreduce have free type variables
+			 *)
+		    let mfulltype = m#get_full_type in
+		    let _ = assert(not (type_is_nonground mfulltype)) in
 		    let instiface, memtyp = 
-		      export_member topiface m#get_full_type
+		      if ccl#get_name = topiface#get_name 
+                      then	   (* may happen in top level definition *)
+					(* export local type parameters *)
+			export_member_local topiface m mfulltype
+                      else		(* export everything *)
+			export_member topiface m mfulltype
 		    in let _ = dr ("infix " ^ 
 				   (string_of_ccsl_inst_iface instiface)
 				  )
@@ -349,18 +368,17 @@ and do_expression ccl loc expr =
 	         (* infixes are only stored as members of ground signatures *)
 		| VarSymb _ 
 		| CCSL_GroundTypeSymb _
-		| CCSL_TypeAbbrevSymb _
 		| TypevarSymb _
 		| ClassSymb _ 
 		| AdtSymb _ 
 		| SigSymb _ 
 		| InstClassSymb _ 
 		| InstAdtSymb _ 
-		  -> (assert(false); raise Internal_error)
+		  -> assert(false)
 	     )
 					(* no other InfixApplication *)
       | InfixApplication _ -> 
-	  (assert(false); raise Internal_error)
+	  assert(false)
 
       | FunUpdate(fex, changes) -> 
 	  dr "FunUpdate";
@@ -397,7 +415,7 @@ and do_expression ccl loc expr =
 	  let _ = dr "Case" in
 	  let first_id  = match variants with
 	    | (Unresolved id,_,_) :: _ -> id
-	    | _ -> (assert(false); raise Internal_error) in
+	    | _ -> assert(false) in
 	  let symb = 
 	    try Symbol.find first_id.token_name 
 	    with Table.Not_defined -> undefined_symbol first_id
@@ -414,14 +432,13 @@ and do_expression ccl loc expr =
 
 					   (* types are not ok *)
 	    | CCSL_GroundTypeSymb _
-	    | CCSL_TypeAbbrevSymb _
 	    | TypevarSymb _
 	    | ClassSymb _ 
 	    | AdtSymb _ 
 	    | SigSymb _ -> wrong_id_type first_id "expression" "type"
 	                    (* no instanciations are stored in CCSL_ID *)
 	    | InstClassSymb _ 
-	    | InstAdtSymb _ -> (assert(false); raise Internal_error)
+	    | InstAdtSymb _ -> assert(false)
 	  in let _ = assert(adt#is_adt) in
 	  let tvars = (List.map (fun _ -> TypeArgument(
 				   FreeTypeVariable(Type_variable.fresh())))
@@ -438,7 +455,7 @@ and do_expression ccl loc expr =
 	  let cl = match typ with
 	    | Self -> ccl
 	    | Class(cl,_) -> cl
-	    | _ -> (assert(false); raise Internal_error) 
+	    | _ -> assert(false) 
 	  in let ntlist = 
 	      List.fold_right 
 		(fun t tl -> match check_method_label cl t with 
@@ -453,7 +470,7 @@ and do_expression ccl loc expr =
 	  let cl = match typ with
 	    | Self -> ccl
 	    | Class(cl,_) -> cl
-	    | _ -> (assert(false); raise Internal_error) 
+	    | _ -> assert(false) 
 	  in let ntlist = 
 	      List.fold_right 
 		(fun t tl -> match check_method_label cl t with 
@@ -486,7 +503,7 @@ and do_expression ccl loc expr =
       | Comment_str _
       | Comment_expr _
       | Comprehension _ ->
-	  (assert(false); raise Internal_error)
+	  assert(false)
 	
 and do_formula ccl loc formula =
   let recurse_exp = do_expression ccl loc in
@@ -547,7 +564,7 @@ and do_formula ccl loc formula =
       | ConstantPredicate _
 (* 	 | LessOrEqual _
  *)
-	-> (assert(false); raise Internal_error)
+	-> assert(false)
 
 
 
@@ -571,18 +588,36 @@ let do_assertion ccl assertion =
 		assertion.assertion_formula <- Symbolic(FormLoc(new_form,loc))
 	    end
 	  
-	| Symbolic _ -> (assert(false); raise Internal_error)
+	| Symbolic _ -> assert(false)
 	| Pvs_String _
 	| Isa_String _ -> ()
 
 
-let do_symbolic_definition ccl def =
+(* Process a definition in a ground signature or in a class
+ * 
+ * From the parser the definition comes in as a formula, which 
+ * is should better be an equation.
+ * From the left hand side of the equation I extract variable declarations, 
+ * by matching these variables against the declared type.
+ * After that the left hand side is thrown away, the extracted variables 
+ * are stored in the definition record and the right hand side is wrapped
+ * in a TypeAnnotation to remember its supposed type
+ *)
+
+let do_symbolic_definition iface def =
   let def_eq = match def.definition with
     | Symbolic eq -> eq
-    | _ -> assert(false); raise Internal_error
+    | _ -> assert(false)
   in let mem = def.defined_method in
+				(* create the block for this equation *)
   let local_block = Symbol.start_block() in
   let _ = d ("  * Resolve definition " ^ def.defined_method#get_name) in
+
+					(* create local type parameters *)
+  let _ = List.iter (function TypeParameter id_rec ->
+		       Symbol.create_type_parameter id_rec) 
+	    mem#get_local_parameters
+  in					(* extract left and right hand sides *)
   let left, right, right_loc = 
     match def_eq with
       | FormLoc(Equal(left, ExprLoc(right,right_loc)), _)
@@ -593,67 +628,119 @@ let do_symbolic_definition ccl def =
 	      "Definition not in form of an equation.";
 	    raise Resolution_error
 	  end
-  in let args = match left with
-    | ExprLoc(Application(ExprLoc(BasicExpr(TermVar(Unresolved defname)),_),
-			  args),_) 
+		(* recursively extract variables from the left hand side 
+		 * take care to handle cases like
+		 *   f = ...
+		 *   f x y = ...
+		 *   f (x,y) (a,b) = ...
+		 *   (x + y) (a,b) = ...
+		 *)
+  in let rec get_args accu = function
+					(* base case normal application *)
+    | ExprLoc(Application(
+		ExprLoc(BasicExpr(TermVar(Unresolved defname)),_),
+		args),_) 
 	when defname.token_name = mem#get_name ->
-	args
-    | _ ->
+	args :: accu
+					(* base case def =  *)
+    | ExprLoc(BasicExpr(TermVar(Unresolved defname)),_)
+	when defname.token_name = mem#get_name 
+	  ->
+	accu
+					(* base case with infixes *)
+    | ExprLoc(InfixApplication(leftex, _, Unresolved op, rightex), loc) 
+	when op.token_name = mem#get_name ->
+	(match mem#get_full_type with
+	   | Function(Product [_;_], _) -> 
+	       ExprLoc(Tuple([leftex;rightex]), loc) :: accu
+	   | Function(_, Function(_)) -> leftex :: rightex :: accu
+	   | _ -> assert false
+	)
+					(* non base case *)
+    | ExprLoc(Application(innerterm, args),_) ->
+	get_args (args :: accu) innerterm
+    | _ ->				(* error no application *)
 	begin
 	  error_message (get_ex_loc left)
 	    ("Wrong form of definition. Expected \"" ^ 
-	     mem#get_name ^ "(x ... )\"");
+	     (if mem#get_sort = InfixGroundTerm 
+	      then
+		"x " ^ mem#get_name ^ " y\""
+	      else
+		mem#get_name ^ " ... =\"")
+	    );
 	  raise Resolution_error
 	end
-  in let arg_list = match args with
-    | ExprLoc(Tuple l,_) -> l
-    | x -> [x]
-  in let argids = 
-      List.map 
-	(fun v -> match v with
-	   | ExprLoc(BasicExpr(TermVar(Unresolved tok)), vloc) ->
-	       let _ =			(* check for doubble occurences *)
-		 try
-		   ignore(Symbol.find_local local_block tok.token_name);
-		   error_message vloc
-		     ("Variables on the left hand side of a defining " ^
-		      "equation must be unique.");
-		   raise Resolution_error
-		 with 
-		   | Table.Not_defined -> ()
-	       in 
-	       let id_rec = Symbol.identifier_record tok CCSL_Var
-	       in
-		 Symbol.create_var id_rec;
-		 id_rec
-	   | _ ->
-	       begin
-		 error_message (get_ex_loc v) "Variable expected.";
-		 raise Resolution_error
-	       end
-	)
-	arg_list
-  in let typlist = match mem#get_domain with
-    | Product tl -> tl
-    | t -> [t]
-  in let _ = (if List.length typlist <> List.length argids then
-	     begin
-	       error_message (get_ex_loc left) "Wrong number of arguments.";
-	       raise Resolution_error
-	     end
-	  )
-  in let _ = List.iter2 (fun id typ -> id.id_type <- typ)
-	       argids typlist
-  in let nright = do_expression ccl right_loc right
+  in let arg_list_list = get_args [] left in
+			(* pair one argument and one type in a id record *)
+  let make_arg_id typ = function
+    | ExprLoc(BasicExpr(TermVar(Unresolved tok)), vloc) ->
+	let _ =				(* check for doubble occurences *)
+	  try
+	    ignore(Symbol.find_local local_block tok.token_name);
+	    error_message vloc
+	      ("Variables on the left hand side of a defining " ^
+	       "equation must be unique.");
+	    raise Resolution_error
+	  with 
+	    | Table.Not_defined -> ()
+	in 
+	let id_rec = Symbol.identifier_record tok CCSL_Var
+	in
+	  id_rec.id_type <- typ;
+	  Symbol.create_var id_rec;
+	  id_rec
+    | v ->
+	begin
+	  error_message (get_ex_loc v) "Variable expected.";
+	  raise Resolution_error
+	end
+  in
+				(* recursively match arguments and types *)
+  let rec match_args argslist funtype = 
+    let recurse restargs codomain ids =
+      let rest_ids, typ = match_args restargs codomain 
+      in
+	((ids :: rest_ids), typ)
+    in
+      match argslist,funtype with
+	| ExprLoc(Tuple arggroup, loc ) :: restargs, 
+	  Function(Product grouptyp, codomain)
+	  -> 
+	    if List.length grouptyp <> List.length arggroup 
+	    then
+	      begin
+		error_message loc
+		  "Number of arguments does not match type declaration.";
+		raise Resolution_error
+	      end
+	    else
+	      recurse restargs codomain
+		(List.map2 make_arg_id grouptyp arggroup)
+	| (ExprLoc _ as singlearg) :: restargs, 
+	  Function(typ, codomain) ->
+	    recurse restargs codomain [make_arg_id typ singlearg]
+	| [], resttyp -> [], resttyp
+	| ExprLoc(_, loc) :: _, _ ->
+	    error_message loc "Superfluous variable.";
+	    raise Resolution_error
+	| _ -> let x = 2 + 2 in
+	    assert false
+
+  in let id_list_list, righttyp = match_args arg_list_list mem#get_full_type
+
+  in let nright = do_expression iface right_loc right
   in 
-    def.definition <- Symbolic(FormLoc(Formula nright, right_loc));
-    def.variables <- argids;
+    def.definition <- 
+    Symbolic(FormLoc(Formula( TypeAnnotation(nright, righttyp)),
+		     right_loc));
+    def.variables <- id_list_list;
     ignore(Symbol.close_block())
       
 	     
-let do_definition ccl def =
+let do_definition iface def =
   match def.definition with
-    | Symbolic _ -> do_symbolic_definition ccl def
+    | Symbolic _ -> do_symbolic_definition iface def
     | _ -> ()
 
 
@@ -704,6 +791,8 @@ let do_class ccl =
 	ccl#get_constructors;
       List.iter (fun crea -> do_assertion ccl crea) 
 	ccl#get_creations; 
+      List.iter (fun thm -> do_assertion ccl thm) 
+	ccl#get_theorems; 
 
       (* print_verbose( "Symboltable: " );
        * print_verbose (Symbol.symbol_table_dump 
@@ -716,6 +805,45 @@ let do_class ccl =
     end
 
 
+let do_sig si =
+  let nest = Symbol.nesting_size() in
+			(* pair the definitions with the declarations *)
+  let rec pair_consts members definitions =
+    match members,definitions with
+      | [],[] -> []
+      | m::ms,[] ->  (m,None) :: (pair_consts ms definitions)
+      | [], d::_ -> assert(false)
+      | m::ms, d::ds ->
+	  if m == d.defined_method 
+	  then
+	    (m, Some d) :: (pair_consts ms ds)
+	  else
+	    (m, None) :: (pair_consts ms definitions)
+  in
+  let const_list = pair_consts si#get_members si#get_definitions
+  in
+    begin
+      print_verbose ("Resolve signature in " ^ si#get_name );
+      ignore(Symbol.start_block());
+      List.iter (function TypeParameter id_rec ->
+		   Symbol.create_type_parameter id_rec) si#get_parameters;
+
+      List.iter (fun (m,defopt) -> 
+		   (match defopt with
+		      | None -> ()
+		      | Some def -> do_definition si def
+		   );
+
+		   Symbol.create_member m
+		)
+	const_list;
+
+      assert(Symbol.nesting_size() = nest + 1);
+      ignore(Symbol.close_block());
+      Symbol.create_sig_content si
+    end
+
+
 let do_adt adt =
   begin
     print_verbose ("Resolve adt in " 
@@ -724,34 +852,10 @@ let do_adt adt =
   end
 
 
-let do_sig si =
-  begin
-    print_verbose ("Resolve signature in " 
-		   ^ si#get_name );
-    Symbol.create_sig_content si
-  end
-
-
-let do_type_abbrev ta = 
-  begin
-    print_verbose ("Resolve type abbreviation in " 
-		   ^ ta.id_token.token_name );
-    Symbol.create_type_abbrev ta
-  end
-
-
-let do_ground_type gtype = 
-  begin
-    print_verbose ("Resolve ground type in " ^ gtype.id_token.token_name );
-    Symbol.create_ground_type gtype
-  end
-
-
 let resolve_ast = function
   | CCSL_class_dec cl -> do_class cl
   | CCSL_adt_dec adt -> do_adt adt
   | CCSL_sig_dec si -> do_sig si
-  | CCSL_TypeAbbrev_dec ta -> do_type_abbrev ta
 
 
 let ccsl_resolution_pass (ast: Classtypes.ccsl_ast list) = 
@@ -765,5 +869,6 @@ let ccsl_resolution_pass (ast: Classtypes.ccsl_ast list) =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
