@@ -18,11 +18,11 @@
  *
  * Created 3.11.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Wednesday 15 May 02 18:42:48 tews@ithif51>
  *
  * Semantics
  *
- * $Id: semantics_theory.ml,v 1.11 2002/01/23 16:00:19 tews Exp $
+ * $Id: semantics_theory.ml,v 1.13 2002/05/22 13:42:43 tews Exp $
  *
  *)
 
@@ -31,9 +31,9 @@ open Global
 open Top_variant_types
 open Name_space 
 open Names
-open Logic_util
 open Classtypes
 open Types_util
+open Pvs_proof_util
 open Theory_class 
 open Pre_printing;;
 
@@ -98,6 +98,13 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 		 self#simple_arguments)]
 	   else [])
 	  @
+	  (if cl#has_feature HasMorphismFeature
+	   then
+	     [ccsl_morphism_rewrite_theory_name cl, []]
+	   else
+	     []
+	  )
+	  @
 	  [(ccsl_box_theory_name cl,
 	    self#simple_arguments)] 
 	  @
@@ -218,7 +225,7 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 	   (fun ass -> 
 	      let base_decl = 
 	      	match ass.self_variable with
-	      	  | None -> (assert(false); raise Internal_error)
+	      	  | None -> assert(false)
 	      	  | Some id -> (id.id_token.token_name,
 			      	id.id_type)
 	      in
@@ -257,15 +264,15 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 	      else match m#get_sort with
 		| Defined_Method ->
 		    (m#get_name, 
-		     Function(m#get_domain,m#get_codomain),
+		     m#get_full_type,
 		     coalgebra_term)
 		| Var_Constructor ->
 		    (m#get_name, 
-		     Function(m#get_domain,m#get_codomain),
+		     m#get_full_type,
 		     algebra_term)
 		| Const_Constructor -> 
 		    (m#get_name, 
-		     m#get_codomain,
+		     m#get_full_type,
 		     algebra_term)
 
 		| Proper_Attribute _
@@ -274,9 +281,10 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 		| Adt_Const_Constructor
 		| Adt_Var_Constructor 
 		| Adt_Accessor 
-		| Adt_Recognizer | Adt_Reduce | Class_Coreduce 
+		| Adt_Recognizer | Adt_Reduce 
+		| Class_Coreduce | Class_Special
 		| GroundTerm | InfixGroundTerm
-		    -> (assert(false);raise Internal_error)
+		    -> assert(false)
 	   )
 	   (cl#get_all_actions @ cl#get_constructors))
       in
@@ -342,22 +350,15 @@ class ['class_type, 'member_type] ccsl_pre_semantics_theory
 
     method private do_method_wise_assert =
       let sub_ns = sub_space ns in 
-      let method_name m = (("o_" ^ m#get_name), m#get_full_type) in
-      let pre_mems = List.map method_name cl#get_all_sig_actions in
-      let mems = create_ids_with_preference sub_ns pre_mems 
+      let (decl_list, coalgebra_from_tuple ) = self#coalgebra_as_tuple sub_ns
       in
 	Defn(name_of_method_assert cl,
-	     [List.map (fun (name,typ) -> Undeclared(name,typ)) mems],
+	     [decl_list],
 	     Bool,
 	     Application(
 	       Term(name_of_assert cl, Always, []),
-	       Application(
-		 Term(name_of_struct_of cl, Always,[]),
-		 match mems with
-		   | [(id,_)] -> Term(id,Always,[])
-		   | l -> Tuple(
-		       List.map (fun (id,_) -> Term(id,Always,[])) l)
-	       )))
+	       coalgebra_from_tuple
+	     ))
 
 
     method private do_create =
@@ -457,91 +458,181 @@ class ['class_type, 'member_type] ccsl_pre_basic_theory
     initializer top_theory#override_file_name (ccsl_class_file_name cl)
 
     method private imports =
-      Import [ccsl_semantics_theory_name cl, 
-	      self#simple_arguments]
+      Import [ccsl_semantics_theory_name cl, self#simple_arguments]
 
     method private do_inherited_assert_lemmas = 
+      let proof = 
+	make_simple_proof
+	  [
+	    skosimp_star;
+	    expand (name_of_assert cl);
+	    pvs_assert;
+	  ]
+      in
+      let lemma ianc =
+	Lemma(
+	  name_of_inherited_assert_lemma ianc,
+	  Forall(
+	    [(name_of_coalgebra, self#coalgebra_type)],
+	    Implies(
+	      self#assert_coalgebra_hypothesis,
+	      Formula(
+		Application(
+		  Term(name_of_assert ianc, Always, []),
+		  Application(Term(super_access_method(ianc),
+				   Always,[]), 
+			      coalgebra_term)
+		)))))
+      in
       List.map
-	(fun ianc ->
-	   Lemma(
-	     name_of_inherited_assert_lemma ianc,
-	     Forall(
-	       [(name_of_coalgebra, self#coalgebra_type)],
-	       Implies(
-		 self#assert_coalgebra_hypothesis,
-		 Formula(
-		   Application(
-		     Term(name_of_assert ianc, Always, []),
-		     Application(Term(super_access_method(ianc),
-				      Always,[]), 
-				 coalgebra_term)
-		   ))))))
+	(fun ianc -> 
+	   Proved(
+	     lemma ianc,
+	     Anon_proof proof))
 	cl#get_resolved_ancestors
 
-    method private do_assertion_lemmas =
-      List.map
-	(fun ass ->
-	   Lemma(
-	     name_of_assertion_lemma ass.assertion_name.token_name,
-	     
-	     match ass.assertion_formula with
-	       | Pvs_String _ 
-	       | Isa_String _ ->
-		   let sub_ns = sub_space ns in
-		   let x = create_one_id sub_ns Self in
-		     Forall( [name_of_coalgebra, self#assert_coalgebra_type;
-			      x,Self],
-			     Formula(
-			       Application(
-				 Application(
-				   Term(name_of_assertion_predicate 
-					  ass.assertion_name.token_name,
-					  Always,[]),
-				   coalgebra_term),
-				 Term(x,Always,[]))))
-	       | Symbolic f -> 
-		   let decl_from_rec id = 
-		     id.id_token.token_name, id.id_type  in
-		   let x_decl = 
-	      	     match ass.self_variable with
-	      	       | None -> (assert(false); raise Internal_error)
-	      	       | Some id -> (id.id_token.token_name,
-			      	     id.id_type) in
-		   let pre_pretty_ass = 
-		     ccsl_pre_pretty_formula cl 
-		       self#get_member_fun self#get_iface_fun
-		       f
-		   in 
-		     (* For PVS snip the first forall off the assertion 
-			and merge it with the quantifier for the coalgebra
-		     *)
-		   let (all_decl, all_body) = match pre_pretty_ass with
-		     | Forall(decllist, body) -> (decllist, body)
-		     | x -> ([], x)
-		   in
-		     match !output_mode with
-		       | Pvs_mode -> 
-			   Forall(
-			     (name_of_coalgebra, self#coalgebra_type) ::
-			     x_decl ::
-			     (List.map decl_from_rec ass.free_variables)
-				    @ all_decl,
-			     Implies(
-			       self#assert_coalgebra_hypothesis,
-			       all_body))
-		       | Isa_mode -> 
-			     MetaImplies(
-			       self#assert_coalgebra_hypothesis,
-			       pre_pretty_ass)
-	   ))
-	cl#get_assertions
+    method private assertion_lemma_proof ass number_of_ass = 
+      let ancestor_number = List.length cl#get_ancestors in
+      let self_var = 
+	match ass.self_variable with
+	  | None -> assert(false)
+	  | Some id -> id.id_token.token_name
+      in
+	PTree( 
+	  [ 
+	    skolem_bang;
+	    case (Pvs_pretty.string_of_pvs_top_expression
+		    (Application(
+		       Term(name_of_assert cl, Always, []),
+		       Term(pvs_skolem name_of_coalgebra, Always, []))))
+	  ],
+	  [make_simple_proof
+	     [
+	       pvs_assert;
+	       expand (name_of_assert cl);
+	       flatten_antecedent;
+	       hide_all_but [-(ancestor_number +1); 1];
+	       inst_val (-1) (pvs_skolem self_var);
+	       flatten_antecedent;
+	       hide_all_but [- number_of_ass; 1];
+	       expand (name_of_assertion_predicate 
+			 ass.assertion_name.token_name);
+	       inst_question;
+	       pvs_assert;
+	     ];
+	   make_simple_proof
+	     [pvs_assert]
+	  ])
 
-    method private do_creation_lemmas =
-      List.map
-	(fun crea ->
-	   Lemma(
-	     name_of_assertion_lemma crea.assertion_name.token_name,
-	     
+    method private do_assertion_lemma ass proof =
+      let form =
+	match ass.assertion_formula with
+	  | Pvs_String _ 
+	  | Isa_String _ ->
+	      let sub_ns = sub_space ns in
+	      let x = create_one_id sub_ns Self in
+		Forall( [name_of_coalgebra, self#assert_coalgebra_type;
+			 x,Self],
+			Formula(
+			  Application(
+			    Application(
+			      Term(name_of_assertion_predicate 
+				     ass.assertion_name.token_name,
+				     Always,[]),
+			      coalgebra_term),
+			    Term(x,Always,[]))))
+	  | Symbolic f -> 
+	      let decl_from_rec id = 
+		id.id_token.token_name, id.id_type  in
+	      let x_decl = 
+	      	match ass.self_variable with
+	      	  | None -> assert(false)
+	      	  | Some id -> (id.id_token.token_name,
+			      	id.id_type) in
+	      let pre_pretty_ass = 
+		ccsl_pre_pretty_formula cl 
+		  self#get_member_fun self#get_iface_fun
+		  f
+	      in 
+		(* For PVS snip the first forall off the assertion 
+		   and merge it with the quantifier for the coalgebra
+		*)
+	      let (all_decl, all_body) = match pre_pretty_ass with
+		| Forall(decllist, body) -> (decllist, body)
+		| x -> ([], x)
+	      in
+		match !output_mode with
+		  | Pvs_mode -> 
+		      Forall(
+			(name_of_coalgebra, self#coalgebra_type) ::
+			x_decl :: (List.map decl_from_rec ass.free_variables)
+			@ all_decl,
+			Implies(
+			  self#assert_coalgebra_hypothesis,
+			  all_body))
+		  | Isa_mode -> 
+		      MetaImplies(
+			self#assert_coalgebra_hypothesis,
+			pre_pretty_ass)
+      in	
+	Proved(
+	  Lemma(
+	    name_of_assertion_lemma ass.assertion_name.token_name,
+	    form),
+	  Anon_proof proof)
+
+    method private do_assertion_lemmas = 
+      let ass_count = ref 1 
+      in
+	List.map 
+	  (fun ass -> 
+	     let lemma = self#do_assertion_lemma ass 
+			 (self#assertion_lemma_proof ass !ass_count)
+	     in
+	       incr ass_count;
+	       lemma
+	  )
+	  cl#get_assertions
+
+
+    method private creation_lemma_proof crea number_of_crea = 
+      PTree( 
+	[ 
+	  skolem_bang;
+	  case (Pvs_pretty.string_of_pvs_top_expression
+		  (Application(
+		     Term(name_of_assert cl, Always, []),
+		     Term(pvs_skolem name_of_coalgebra, Always, []))))
+	],
+	[PTree(
+	   [
+	     case (Pvs_pretty.string_of_pvs_top_expression
+		     (Application(
+			Application(
+			  Expression(ConstantPredicate(name_of_creation cl)),
+			  Term(pvs_skolem name_of_coalgebra,Always,[])),
+			Term(pvs_skolem name_of_algebra,Always,[]))))
+	   ],
+	   [make_simple_proof
+	      [
+		pvs_assert;
+		expand (name_of_creation cl);
+		flatten_antecedent;
+		hide_all_but [-number_of_crea; 1];
+		expand (name_of_creation_predicate 
+			  crea.assertion_name.token_name);
+		pvs_assert;
+	      ];
+	    make_simple_proof
+	      [pvs_assert]
+	   ]);
+	 make_simple_proof
+	   [pvs_assert]
+	])
+
+
+    method private do_creation_lemma crea proof =
+      let form =
 	     match crea.assertion_formula with
 	       | Pvs_String _ 
 	       | Isa_String _ ->
@@ -579,10 +670,27 @@ class ['class_type, 'member_type] ccsl_pre_basic_theory
 			       MetaImplies(
 				 self#creation_algebra_hypothesis,
 				 pre_pretty_crea))
-	))
-	cl#get_creations
+      in
+	Proved(
+	  Lemma(
+	    name_of_assertion_lemma crea.assertion_name.token_name,
+	    form),
+	  Anon_proof proof)
 
 
+    method private do_creation_lemmas =
+      let crea_count = ref 1 
+      in
+	List.map 
+	  (fun crea -> 
+	     let lemma = self#do_creation_lemma crea 
+			 (self#creation_lemma_proof crea !crea_count)
+	     in
+	       incr crea_count;
+	       lemma
+	  )
+	  cl#get_creations
+
 
     method make_body =
       [
@@ -608,5 +716,6 @@ class ccsl_basic_theory cl =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
