@@ -18,11 +18,11 @@
  *
  * Created 14.5.99 by Hendrik
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Tuesday 21 May 02 14:17:56 tews@ithif51>
  *
  * Utility functions for ccsl input types
  *
- * $Id: types_util.ml,v 1.21 2002/01/24 14:44:01 tews Exp $
+ * $Id: types_util.ml,v 1.23 2002/05/22 13:42:44 tews Exp $
  *
  *)
 
@@ -31,6 +31,7 @@ open Top_variant_types
 open Top_variant_types_util (* for resolution_of *)
 open Logic_util
 open Classtypes
+;;
 
 (***********************************************************************
  *
@@ -59,7 +60,7 @@ let rec count_self = function
   | FreeTypeVariable _
   | Array _
   | Predtype _
-  | SmartFunction _ -> (assert(false);raise Internal_error)
+  | SmartFunction _ -> assert(false)
 
 
 and count_self_args args = List.fold_left
@@ -89,7 +90,7 @@ let rec count_carrier = function
   | FreeTypeVariable _
   | Array _
   | Predtype _
-  | SmartFunction _ -> (assert(false);raise Internal_error)
+  | SmartFunction _ -> assert(false)
 
 
 and count_carrier_args args = List.fold_left
@@ -105,7 +106,10 @@ and count_carrier_args args = List.fold_left
      * typeparameter from param_p, if one parameter 
      * from param_p is found, it returns false.
      *
-     * param_p : top_pre_identifier_record_type -> bool
+     * val check_for_params : 
+     * 	 (('cl,'mem) top_pre_identifier_record_type -> bool) ->
+     * 	   ('cl,'mem) top_pre_types -> bool
+     * 
      *)
 let rec check_for_params param_p = function
   | Groundtype (_, args) 	-> check_for_params_args param_p args
@@ -128,7 +132,7 @@ let rec check_for_params param_p = function
   | IFace _
   | Array _
   | Predtype _
-  | SmartFunction _ -> (assert(false);raise Internal_error)
+  | SmartFunction _ -> assert(false)
 
 
 and check_for_params_args param_p args = 
@@ -156,6 +160,7 @@ let constant_arg_list param_p argumentlist =
     (function TypeArgument t -> constant_type param_p t)
     argumentlist
 
+
 (**********************************************************************
  * returns true if argument contains a free type variable
  * 
@@ -182,7 +187,7 @@ let rec type_is_nonground = function
   | IFace _
   | Array _
   | Predtype _
-  | SmartFunction _ -> (assert(false);raise Internal_error)
+  | SmartFunction _ -> assert(false)
 
 and args_are_nonground args =
   List.exists
@@ -228,7 +233,7 @@ let rec occur_check tv t =
       | IFace _
       | Array _
       | Predtype _
-      | SmartFunction _ 	-> (assert(false);raise Internal_error)
+      | SmartFunction _ 	-> assert(false)
 
 
 
@@ -257,7 +262,7 @@ let rec has_successor_state = function
       | Array _ 	      
       | FreeTypeVariable _
       | TypeConstant _
-      | SmartFunction _ -> (assert(false);raise Internal_error)
+      | SmartFunction _ -> assert(false)
 and args_have_successor_state args = 
   List.exists
     (function 
@@ -266,6 +271,46 @@ and args_have_successor_state args =
     args
 
 
+(*******************************************************************
+ *
+ * utility functions for ground types (more below)
+ *
+ *)
+
+let get_ground_type_parameters idrec = match idrec.id_origin with
+  | CCSL_GroundTypeDef si
+  | CCSL_GroundTypeDecl si -> si#get_parameters @ idrec.id_parameters
+  | _ -> assert(false)
+
+
+let get_definition_parameters def =
+  def.defined_method#hosting_class#get_parameters @
+  def.defined_method#get_local_parameters
+
+
+let is_type_def idrec = match idrec.id_origin with
+  | CCSL_GroundTypeDef _ -> true
+  | CCSL_GroundTypeDecl _ -> false
+  | _ -> assert(false)
+
+let sig_of_typedef idrec = 
+  match idrec.id_origin with
+    | CCSL_GroundTypeDef si -> si
+    | CCSL_GroundTypeDecl si -> si
+    | _ -> assert(false)
+
+
+(************************************************
+ *
+ * extract type parameters of a member, taking local parameters 
+ * into account
+ *)
+
+let get_member_parameters m =
+  m#hosting_class#get_parameters @ m#get_local_parameters
+
+
+
 
 (***********************************************************************
  *
@@ -276,14 +321,11 @@ and args_have_successor_state args =
  *
  *)
 
-let rec iter_components variance action = function
+let rec iter_components variance action typ = match typ with
   | Groundtype(id, args)-> 
-      iter_component_arglist variance action args id.id_parameters;
-      let si = match id.id_origin with
-	| CCSL_GroundType si -> si
-	| _ -> assert(false); raise Internal_error
-      in
-	action (variance, si, args)
+      iter_component_arglist variance action args 
+	(get_ground_type_parameters id);
+      action variance typ
   | BoundTypeVariable id 	-> ()
   | Self 		-> ()
   | Carrier 		-> ()
@@ -295,10 +337,10 @@ let rec iter_components variance action = function
       List.iter (iter_components variance action) type_list
   | Class(cl, args) 	-> 
       iter_component_arglist variance action args cl#get_parameters;
-      action (variance,cl,args)
+      action variance typ
   | Adt(adt,flag,args) 	-> 
       iter_component_arglist variance action args adt#get_parameters;
-      action (variance,adt,args)
+      action variance typ
 				(* not in ccsl input, but occurs for reduce *)
   | FreeTypeVariable _  -> ()
 
@@ -308,7 +350,7 @@ let rec iter_components variance action = function
   | Record _
   | IFace _
   | Array _ 	      
-  | SmartFunction _ -> (assert(false);raise Internal_error)
+  | SmartFunction _ -> assert(false)
 
 
 and iter_component_arglist variance action arguments parameters = 
@@ -320,6 +362,25 @@ and iter_component_arglist variance action arguments parameters =
     arguments parameters
 
 
+let do_add_component iface v = function
+  | Class(cl,args) -> iface#add_component (v, cl, args)
+  | Adt(adt,_,args) -> iface#add_component (v, adt, args)
+  | Groundtype(id,args) ->
+      let osi = sig_of_typedef id 
+      in
+	if osi#get_name <> iface#get_name
+	then
+	  iface#add_component (v, osi, args)
+  | _ -> assert(false)
+
+
+let iter_components_add_component iface typ =
+  iter_components Pos (do_add_component iface) typ
+
+let iter_component_arglist_add_component iface arguments parameters =
+  iter_component_arglist Pos (do_add_component iface) arguments parameters
+
+
 (***********************************************************************
  *
  * equality on ccsl output types
@@ -395,7 +456,7 @@ let rec eq_ccsl_types t1 t2 = t1 == t2 or match t1,t2 with
 
 
      (* not allowed constructors in t2 *)
-  | _, Array _          -> (assert(false); raise Internal_error)
+  | _, Array _          -> assert(false)
 
 
 
@@ -710,12 +771,6 @@ and eq_ccsl_expressions e1 e2 =   e1 == e2 or match e1,e2 with
 		       var_list1 var_list2)
 		 && (eq_ccsl_expressions ex_b1 ex_b2))
 	    match_list1 match_list2)
-(*   | Reduce(adt1,flag1,args1), Reduce(adt2,flag2, args2) ->
- * 	 (*ignoring flags *)
- * 	 (eq_string adt1#get_name adt2#get_name)
- * 	 && (List.length args1 = List.length args2)
- * 	 && (List.for_all2 eq_ccsl_args args1 args2)
- *)
   | Box(typ1,pred1,tlist1),
     Box(typ2,pred2,tlist2) ->
       (eq_ccsl_types typ1 typ2)
@@ -789,8 +844,6 @@ and eq_ccsl_expressions e1 e2 =   e1 == e2 or match e1,e2 with
   | If _ ,_
   | Case _ ,_
   | CCSL_Case _ ,_
-(*   | Reduce _,_
- *)
   | Box _,_
   | Diamond _,_
   | Every _, _
@@ -822,8 +875,6 @@ and eq_ccsl_expressions e1 e2 =   e1 == e2 or match e1,e2 with
   | _, If _
   | _, Case _
   | _, CCSL_Case _
-(*   | _, Reduce _
- *)
   | _, Box _
   | _, Diamond _
   | _, Every _
@@ -842,6 +893,15 @@ and eq_ccsl_args a1 a2 = match a1,a2 with
   | TypeArgument typ1, TypeArgument typ2 ->
       eq_ccsl_types typ1 typ2
 
+let eq_ccsl_params pl1 pl2 = 
+  try
+    List.for_all2 
+      (fun p1 p2 -> match p1,p2 with
+	 | TypeParameter id1, TypeParameter id2 -> 
+	     eq_string id1.id_token.token_name id2.id_token.token_name)
+      pl1 pl2
+  with
+    | Invalid_argument _ -> false
 
 (***********************************************************************
  ***********************************************************************
@@ -950,8 +1010,6 @@ let rec curry_formula formula =
     | Implies _ -> formula
     | Iff _ -> formula
     | Equal(ex_a,ex_b) -> formula
-(*     | LessOrEqual(ex_a,ex_b) -> formula
- *)
     | Forall(quant_list, f) -> formula
     | Exists(quant_list, f) -> formula
     | ConstantPredicate name -> formula
@@ -980,8 +1038,6 @@ let rec meta_implies formula =
     | Iff(a,b) -> And([MetaImplies(a, meta_implies b); 
 		       MetaImplies(b, meta_implies a)])
     | Equal(ex_a,ex_b) -> formula
-(*     | LessOrEqual(ex_a,ex_b) -> formula
- *)
     | Forall(quant_list, f) -> formula
     | Exists(quant_list, f) -> formula
     | ConstantPredicate name -> formula
@@ -1048,8 +1104,6 @@ let rec pull_quant_formula forall formula =
 	 *)
     | Iff(p1,p2) -> ([], formula)
     | Equal(ex_a,ex_b) -> ([], formula)
-(*     | LessOrEqual(ex_a,ex_b) -> ([], formula)
- *)
     | Forall(quant_list, f) -> 
 	if forall 
 	then 
@@ -1135,6 +1189,13 @@ let types_from_arguments args =
     args
 
 
+let arguments_from_parameters params = 
+  List.map (function
+	      | TypeParameter id -> TypeArgument(BoundTypeVariable(id))
+	   )
+    params
+
+
 let make_substitution param_list arg_list =
   List.combine
     (types_from_parameters param_list)
@@ -1147,6 +1208,50 @@ let make_substitution_param_param param_list_from param_list_to =
     (types_from_parameters param_list_to)
 
 
+(*******************************************************************
+ *
+ * more utility functions for ground types
+ *
+ *)
+
+let rec expand_type_def eq_types typ = match typ with
+  | Groundtype(id, args) ->
+      if is_type_def id 
+      then
+	expand_type_def eq_types
+	  (substitute_types_only eq_types 
+	     (make_substitution (get_ground_type_parameters id) args)
+	     id.id_type
+	  )
+      else
+	typ
+				(* expand the second level for functions *)
+  | Function(dom, codom) ->
+      let ndom = expand_type_def eq_types dom in
+      let ncodom = expand_type_def eq_types codom 
+      in
+	if dom == ndom && codom == ncodom 
+	then
+	  typ
+	else
+	  Function(ndom, ncodom)
+					(* do nothing in all other cases *)
+  | BoundTypeVariable _
+  | Self
+  | Carrier
+  | Bool
+  | Product _
+  | Class _ 
+  | Adt _
+  | Record _
+  | TypeConstant _
+  | IFace _
+  | FreeTypeVariable _
+  | Array _
+  | Predtype _
+  | SmartFunction _         -> typ
+
+
 (***********************************************************************
  *
  * export method / constructor types
@@ -1165,39 +1270,49 @@ let make_substitution_param_param param_list_from param_list_to =
  * 
  *)
 
-let export_member_with_args iface args memtype =
-  let _ = assert(List.length iface#get_parameters = List.length args) in
+let export_member_with_args iface member args memtype =
+  let _ = assert(List.length 
+		   (get_member_parameters member) = List.length args)
+  in
   let types = types_from_arguments args in
-  let bounds = types_from_parameters iface#get_parameters in
+  let bounds = types_from_parameters (get_member_parameters member) in
   let iiface = 
     match iface#get_kind with
       | Spec_adt -> Adt(iface, Always, args)
       | Spec_class -> Class(iface, args)
       | Spec_sig -> Product []		(* this is never used *)
-      | Spec_Spec -> (assert(false);raise Internal_error)
+      | Spec_Spec -> assert(false)
   in
   let subst = 
     (match iface#get_kind with
        | Spec_adt -> [ (Carrier, iiface) ]
        | Spec_class -> [ (Self, iiface) ]
        | Spec_sig -> []
-       | Spec_Spec -> (assert(false);raise Internal_error)
+       | Spec_Spec -> assert(false)
     )
     @ List.combine bounds types 
   in
-    (InstIface(iface,args),
+    (InstIface(iface,args,None),
      ccsl_substitute_types subst memtype)
 
 
-let export_member iface memtyp =
+let export_member iface member memtyp =
   let args = List.map 
 	       (fun _ -> TypeArgument(
 		  FreeTypeVariable(Type_variable.fresh())))
-	       iface#get_parameters
+	       (get_member_parameters member)
   in 
-    export_member_with_args iface args memtyp
+    export_member_with_args iface member args memtyp
      
-
+let export_member_local iface member memtyp = 
+  let global_args = arguments_from_parameters iface#get_parameters in
+  let localparams = member#get_local_parameters
+  in let local_args = List.map 
+		     (fun _ -> TypeArgument(
+			FreeTypeVariable(Type_variable.fresh())))
+		     localparams
+  in
+    export_member_with_args iface member (global_args @ local_args) memtyp
 
 (***********************************************************************
  *
@@ -1215,13 +1330,13 @@ let uniquify_type_variables typ =
   let subst_list = ref ([] : (Type_variable.t * Type_variable.t) list) in
   let subst tv = 
     try
-	 Util.assoc Type_variable.eq tv !subst_list
+      Util.assoc Type_variable.eq tv !subst_list
     with
-	 | Not_found ->
-	     let ntv = Type_variable.fresh () in
-	     let _ = subst_list := (tv,ntv) :: !subst_list
-	     in
-	       ntv
+      | Not_found ->
+	  let ntv = Type_variable.fresh () in
+	  let _ = subst_list := (tv,ntv) :: !subst_list
+	  in
+	    ntv
   in 
   let rec doit_typ typ = match typ with
     | Groundtype(id,args) -> Groundtype(id, doit_args args)
@@ -1242,7 +1357,7 @@ let uniquify_type_variables typ =
     | Array(cl,elemtyp,size) -> Array(cl, doit_typ elemtyp, size)
 					   (* not allowed here *)
     | Predtype(formula) -> 
-	   (assert(false); raise Internal_error)
+	   assert(false)
   and doit_args args = 
     List.map (function TypeArgument(t) -> TypeArgument(doit_typ t))
 	 args
@@ -1268,14 +1383,7 @@ let method_arg_list m =
   match m#get_domain with
       | Product(Self::tl) -> tl
       | Self -> [] 
-      | _ -> (assert(false); raise Internal_error) 
-
-let arguments_from_parameters params = 
-  List.map (function
-	      | TypeParameter id -> TypeArgument(BoundTypeVariable(id))
-	   )
-    params
-
+      | _ -> assert(false) 
 
   (**************
    * check if the arguments suit the parameters in number kind and type
@@ -1293,9 +1401,52 @@ let id_record_from_string name =
     id_parameters = [];
     id_origin = CCSL_Output;
     id_variance = Unset;
+    id_sequence = -1;
   }
 
 
+  (**************
+   * utility functions for features
+   * at the moment groundtypes have a fixed feature set
+   *)
+
+let groundtype_feature id = function 
+  | HasRelLiftFeature
+  | HasMapFeature
+      -> true
+  | BuildinFeature
+  | NeedsMapFeature
+  | FinalSemanticsFeature
+  | MixedSelfInstFeature
+  | ComponentNoRelLiftFeature
+  | ComponentNoFullRelLiftFeature
+  | ComponentNoMapFeature
+  | ComponentNoFullMapFeature
+  | ClassComponentFeature
+  | CarrierClassComponentFeature
+  | HasBisimulationFeature
+  | HasFullRelLiftingFeature
+  | HasGreatestBisimFeature
+  | HasMorphismFeature
+      -> assert(false)
+
+  (*
+   * supplied type must be one of Class / Adt / Groundtype
+   *
+   * val type_has_feature : 
+   *   feature_type ->
+   *     (< has_feature : feature_type -> bool; .. > as 'cl,
+   * 	'mem) top_pre_types ->
+   * 	 bool
+   *)
+
+let type_has_feature feature = function
+  | Class(cl,_) -> cl#has_feature feature
+  | Adt(adt,_,_) -> adt#has_feature feature
+  | Groundtype(id,args) -> groundtype_feature id feature
+  | _ -> assert(false)
+
+
   (**************
    * the next two functions are for extracting the location field
    * !! they fail if the top constructor is not a location !!
@@ -1307,11 +1458,11 @@ let id_record_from_string name =
 
 let rec get_ex_loc = function
   | ExprLoc(_,l) -> l
-  | _ -> (assert(false); raise Internal_error)
+  | _ -> assert false
 
 let rec get_form_loc = function
   | FormLoc(_,l) -> l
-  | _ -> (assert(false); raise Internal_error)
+  | _ -> assert(false)
 
 
 
@@ -1320,4 +1471,5 @@ let rec get_form_loc = function
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
