@@ -18,11 +18,11 @@
  *
  * Created 26.2.99 by Bart
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Saturday 11 May 02 18:51:30 tews@ithif56.inf.tu-dresden.de>
  *
  * PVS pretty printer
  *
- * $Id: pvs_pretty.ml,v 1.19 2002/01/24 13:39:10 tews Exp $
+ * $Id: pvs_pretty.ml,v 1.21 2002/05/22 13:42:46 tews Exp $
  *
  *)
 
@@ -83,7 +83,7 @@ let rec pvs_pp_top_type
 	 * Use substitution, to substitute all Carriers with the 
 	 * type of the Adt.
 	 *)
-	(assert(false);  raise Internal_error)
+	assert(false)
     | Function(dom, codom) ->
 	begin
 	  open_box 0;
@@ -99,7 +99,7 @@ let rec pvs_pp_top_type
 	end	
     | SmartFunction(doml,codom) ->
 	(match doml with
-	   | [] -> (assert(false); raise Internal_error)
+	   | [] -> assert(false)
 	   | [t] -> pvs_pp_top_type (Function(t,codom))
 	   | tl -> pvs_pp_top_type (Function(Product(tl),codom))
 	)
@@ -192,8 +192,8 @@ let rec pvs_pp_top_type
 			      name_of_array_of_class
 				c (string_of_int dim)
 			  | _ ->
-			      assert false;
-			      raise Internal_error);			      
+			      assert false
+		       );
 	  close_box()
 	end
 
@@ -206,14 +206,14 @@ let rec pvs_pp_top_type
 	  close_box()
 	end
 					(* not allowed *)
-    | FreeTypeVariable _ -> (assert(false); raise Internal_error)
+    | FreeTypeVariable _ -> assert(false)
 
 and pvs_pp_top_basic_expression = function
     | TermVar idc ->
 	print_string (resolution_of idc).id_token.token_name
     | Member(_,_)  ->
 			(* Methods are treated in the prepritting pass *)
-	(assert false; raise Internal_error)
+	assert false
 
 
 and pvs_pp_top_expression (* : top_expressions -> unit *)
@@ -352,7 +352,7 @@ and pvs_pp_top_expression (* : top_expressions -> unit *)
 	  (match arg with
 	     | SmartAbstraction (str_typ_list, expr) -> (str_typ_list, expr)
 	     | Abstraction (str_typ_list, expr) -> (str_typ_list, expr)
-	     | _ -> (assert(false); raise Internal_error)
+	     | _ -> assert(false)
 	  )
 	in
 	  (match str_typ_list with
@@ -425,7 +425,7 @@ and pvs_pp_top_expression (* : top_expressions -> unit *)
 
     | SmartApplication(func, arg_list) ->
 	(match arg_list with
-	   | [] -> (assert(false); raise Internal_error)
+	   | [] -> assert(false)
 	   | [t] -> pvs_pp_top_expression (Application(func, t))
 	   | tl -> pvs_pp_top_expression (Application(func, Tuple(tl))))
     | FunUpdate (func, expr_expr_list) ->
@@ -574,8 +574,7 @@ and pvs_pp_top_expression (* : top_expressions -> unit *)
 	       end)
     | CCSL_Case (expr, match_list) ->
 	(match match_list with
-	   | [] -> 
-	       (assert false; raise Internal_error)
+	   | [] -> assert false
 	   | (mem1, var_list1, expr1) :: cases ->
  	       begin
 		 open_box 0;		                   (* 1 *)
@@ -662,7 +661,8 @@ and pvs_pp_top_expression (* : top_expressions -> unit *)
 	     | Groundtype(id,_) -> 
 		 Term(name_of_const_every id.id_token.token_name,
 		      Always,[])
-	     | _ -> assert(false); raise Internal_error)
+	     | _ -> assert(false)
+	  )
 	in
 	  pvs_pp_top_expression
 	    (if expr_list = [] 
@@ -691,7 +691,8 @@ and pvs_pp_top_expression (* : top_expressions -> unit *)
 	     | Groundtype(id,_) -> 
 		 Term(name_of_const_rel_every id.id_token.token_name,
 		      Always,[])
-	     | _ -> assert(false); raise Internal_error)
+	     | _ -> assert(false)
+	  )
 	in
 	  pvs_pp_top_expression
 	    (if expr_list = [] 
@@ -712,7 +713,8 @@ and pvs_pp_top_expression (* : top_expressions -> unit *)
 	     | Groundtype(id,_) -> 
 		 Term(name_of_const_map id.id_token.token_name,
 		      Always,[])
-	     | _ -> assert(false); raise Internal_error)
+	     | _ -> assert(false)
+	  )
 	in
 	  pvs_pp_top_expression
 	    (if expr_list = [] 
@@ -764,7 +766,7 @@ and pvs_pp_top_expression (* : top_expressions -> unit *)
     | MethodSelection _
     | Box _
     | Diamond _ ->
-      	(assert(false); raise Internal_error)
+      	assert(false)
 
 	
 and pvs_pp_top_formula (* : top_formulas -> unit *)
@@ -906,7 +908,8 @@ and pvs_pp_top_formula (* : top_formulas -> unit *)
 		     Term(name_of_greatest_bisim cl, Always, []),
 		     [Term(cl_name,Always,[]);Term(cl_name,Always,[])]
 		   )
-	     | _ -> assert(false); raise Internal_error)
+	     | _ -> assert(false)
+	  )
 	in
 	  pvs_pp_top_expression
 	    (Application(
@@ -935,8 +938,7 @@ and pvs_pp_top_formula (* : top_formulas -> unit *)
 	  close_box()					    (* 1 *)
 	end
     | Obseq (None,_,_) ->
-	assert false;
-	raise Internal_error
+	assert false
 
 and pvs_pp_arg_list al =
     let pp_arg = function
@@ -1072,7 +1074,7 @@ and pvs_pp_top_theory_declaration (* : top_theory_declaration -> unit *)
 	  close_box()
 	end
     | Primrecdefn _ -> 
-	(assert(false); raise Internal_error)
+	assert(false)
     | Defnuninterpret (name, var_listlist, typ) ->
 	begin
 	  open_box 2;
@@ -1380,7 +1382,7 @@ let pvs_pp_proof = function
 	  print_string( ")");
 	  close_box();
 	  force_newline()
-  | Anon_proof _ -> (assert(false);raise Internal_error)
+  | Anon_proof _ -> assert(false)
 
 let pvs_pp_th_proofs thname proofs =
   open_box 0;
@@ -1440,11 +1442,19 @@ let pvs_pp_edit_proof name tree =
   close_box()
 	
 
+(******************************************************************
+ ******************************************************************
+ *
+ * string wrapper section
+ *)
+
+let string_of_pvs_top_expression ex = stringwrapper pvs_pp_top_expression ex
 
 
 (*** Local Variables: ***)
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
 
