@@ -18,11 +18,11 @@
  *
  * Created 22.4.99 by Bart
  *
- * Time-stamp: <Monday 8 October 01 17:58:00 tews@ithif51>
+ * Time-stamp: <Sunday 19 May 02 16:09:20 tews@ithif56.inf.tu-dresden.de>
  *
  * top level type definitions
  *
- * $Id: top_variant_types.ml,v 1.23 2002/01/24 14:44:03 tews Exp $
+ * $Id: top_variant_types.ml,v 1.25 2002/05/22 13:42:47 tews Exp $
  *
  *)
 
@@ -185,8 +185,9 @@ type feature_type =
 				       with a parameter or self that 
 				       has not map
 				    *)
-  | ClassComponentFeature	    (* adt with nonconstant class component *)
-  | CarrierClassComponentFeature    (* adt with class component 
+  | ClassComponentFeature	    (* adt with a nonconstant component
+				     * which is a class or a ground type *)
+  | CarrierClassComponentFeature    (* adt with class/groundtype component 
 				       instanciated with carrier
 				    *)
   | HasBisimulationFeature	    (* class has no definition of
@@ -227,11 +228,11 @@ type top_visibility =
   | Private
 
       
-type 'mem top_member_sort = 
+type 'mem top_pre_member_sort = 
   | Proper_Attribute of 'mem option   (* declared attribute with set_method *)
   | Update_Method			(* update method for an attribute *)
   | Defined_Method			(* definitional extention *)
-  | Normal_Method			(* method before variance checking *)
+  | Normal_Method			(* other method *)
   | Var_Constructor			(* class constructor with args *)
   | Const_Constructor			(* class constructor without args *)
   | Adt_Var_Constructor			(* adt constructor with args *)
@@ -240,6 +241,7 @@ type 'mem top_member_sort =
   | Adt_Recognizer			(* an recognizer *)
   | Adt_Reduce
   | Class_Coreduce
+  | Class_Special			(* morphism? and friends *)
   | GroundTerm				(* member of a ground signature *)
   | InfixGroundTerm			(* infix GroundTerm *)
 
@@ -344,7 +346,8 @@ and ('cl, 'mem) top_pre_inst_iface_type =
   | NoIface				(* no info given *)
   | CurrentIface			(* the current class, ie. Self *)
 					(* some other class/adt/sig *)
-  | InstIface of 'cl * ('cl, 'mem) top_pre_argument_type list
+  | InstIface of 'cl * ('cl, 'mem) top_pre_argument_type list 
+                     * location_type option
 
 and ('cl, 'mem) top_pre_basic_expressions =
       (* 
@@ -487,14 +490,6 @@ and ('cl, 'mem) top_pre_expressions =
       ('mem container * ('cl, 'mem) top_pre_identifier_record_type list * 
        ('cl, 'mem) top_pre_expressions) list
 
-(* 	     (* One usually has reduce in an application
- * 	      * but we need it as a function symbol (without arguments)
- * 	      * here, since we might have it without application in our Logic
- * 	      *)
- *   | Reduce of
- * 	 'cl * argument_flag * ('cl, 'mem) top_pre_argument_type list 
- *)
-
       (* The Modal Operators (Box, Diamond)
        * - they are only needed while parsing the Logic
        * - they carry 
@@ -564,9 +559,6 @@ and ('cl, 'mem) top_pre_formulas =
       ('cl, 'mem) top_pre_formulas * ('cl, 'mem) top_pre_formulas
   | Equal of
       ('cl, 'mem) top_pre_expressions * ('cl, 'mem) top_pre_expressions
-(*   | LessOrEqual of
- * 	 ('cl, 'mem) top_pre_expressions * ('cl, 'mem) top_pre_expressions
- *)
   | Forall of
       (string * ('cl, 'mem) top_pre_types) list * 
       ('cl, 'mem) top_pre_formulas
@@ -608,19 +600,32 @@ and ('cl, 'mem) top_pre_assertion_record =
       mutable assertion_formula : ('cl, 'mem) top_pre_assertion
     }
 
+(* definition_record is used for both: for definitional extensions 
+ * in classes and for top level definitions. 
+ * The parser fills the definition field with an formula and leaves the 
+ * variables list empty. The resolution pass checks that the formula is 
+ * an equation and extracts variable declarations from the left hand side. 
+ * Only the right hand side of the equation is kept together with a 
+ * TypeAnnotation for the (expected) type of the right hand side.
+ * 
+ * An definition 
+ *    f a (b,c) d = ....
+ * gives rise to the following variable list
+ * 	[ [a]; [b,c], d]
+ *)
 
 and ('cl, 'mem) top_pre_definition_record =
     { defined_method : 'mem;
-      mutable variables : ('cl, 'mem) top_pre_identifier_record_type list;
-      mutable definition : ('cl, 'mem) top_pre_assertion
+      mutable variables : 
+	('cl, 'mem) top_pre_identifier_record_type list list;
+      mutable definition : ('cl, 'mem) top_pre_assertion;
+      mutable def_sequence : int;
     }
 
 
 and ('cl, 'mem) top_pre_symbol_type =
 					(* a ground type *)
   | CCSL_GroundTypeSymb of ('cl, 'mem) top_pre_identifier_record_type
-					(* a ccsl type abbreviation *)
-  | CCSL_TypeAbbrevSymb of ('cl, 'mem) top_pre_identifier_record_type
 					(* a ccsl type variable *)
   | TypevarSymb of ('cl, 'mem) top_pre_identifier_record_type
 					(* a ccsl variable *)
@@ -734,8 +739,8 @@ and ('cl, 'mem) top_pre_argument_type =
 and ('cl, 'mem) top_pre_origin_type =
   | CCSL_TypeParameter			(* id_type irrelevant *)
   | CCSL_Var				(* id_type, id_parameters irrelevant *)
-  | CCSL_TypeAbbrev			(* id_parameters irrelevant *)
-  | CCSL_GroundType of 'cl
+  | CCSL_GroundTypeDef of 'cl
+  | CCSL_GroundTypeDecl of 'cl		(* id_type irrelevant *)
   | CCSL_Output				(* everything irrelvant 
 					   except the name *)
 
@@ -744,8 +749,9 @@ and ('cl, 'mem) top_pre_identifier_record_type =
     id_token  : token_type;
     mutable id_type : ('cl, 'mem) top_pre_types;
     mutable id_parameters : ('cl, 'mem) top_pre_parameter_type list;
-    id_origin : ('cl, 'mem) top_pre_origin_type;
+    mutable id_origin : ('cl, 'mem) top_pre_origin_type;
     mutable id_variance : variance_type;
+    mutable id_sequence : int;
   }
 
 
@@ -780,17 +786,22 @@ type ('cl, 'mem) top_pre_renaming =
  * depend on them
  *)
 type ('cl, 'mem) top_pre_ancestor_type =
-  | Unresolved_ancestor of token_type list
-  | Resolved_ancestor of 'cl
+(* DELETE
+ * no java ancestors
+ *   | Unresolved_ancestor of token_type list
+ *   | Resolved_ancestor of 'cl
+ *)
   | Unresolved_renaming of
       'cl *     (* the original class *)
       ('cl, 'mem) top_pre_argument_type list *
-      ('cl, 'mem) top_pre_renaming list
+      ('cl, 'mem) top_pre_renaming list *
+      location_type 
   | Resolved_renaming of
       'cl *     (* the original class *)
       ('cl, 'mem) top_pre_argument_type list * 
       'mem list *
-      'cl       (* the instanciated class *)
+      'cl *      (* the instanciated class *)
+      location_type
 
 (* 
  * the type of components,
@@ -802,11 +813,12 @@ type ('cl, 'mem) top_pre_ancestor_type =
 
 type ('cl, 'mem) top_pre_component_type = 
     variance_type * 'cl * ('cl, 'mem) top_pre_argument_type list
-constraint 'cl = < get_parameters : ('cl,'mem) top_pre_parameter_type list;
-                   ..>
+(* 
+ * constraint 'cl = < get_parameters : ('cl,'mem) top_pre_parameter_type list;
+ * 		      ..>
+ *)
 
 type ('cl, 'mem) top_pre_ast = 
-  | CCSL_TypeAbbrev_dec of ('cl, 'mem) top_pre_identifier_record_type
   | CCSL_class_dec of 'cl
   | CCSL_adt_dec of 'cl
   | CCSL_sig_dec of 'cl
@@ -839,4 +851,5 @@ type ('cl, 'mem) top_pre_scope_type =
 (*** version-control: t ***)
 (*** kept-new-versions: 5 ***)
 (*** delete-old-versions: t ***)
+(*** time-stamp-line-limit: 30 ***)
 (*** End: ***)
